@@ -1,7 +1,7 @@
 // qtractorAudioBuffer.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -880,7 +880,8 @@ bool qtractorAudioBuffer::seekSync ( unsigned long iFrame )
 {
 #ifdef CONFIG_DEBUG_0
 	qDebug("qtractorAudioBuffer[%p]::seekSync(%lu) pending(%d, %lu) wo=%lu ro=%lu",
-		this, iFrame, m_iSeekPending, m_iSeekOffset, m_iWriteOffset, m_iReadOffset);
+		this, iFrame, ATOMIC_GET(&m_seekPending), m_iSeekOffset,
+		m_iWriteOffset, m_iReadOffset);
 #endif
 
 #ifdef CONFIG_LIBSAMPLERATE
@@ -889,7 +890,7 @@ bool qtractorAudioBuffer::seekSync ( unsigned long iFrame )
 		for (unsigned short i = 0; i < m_pRingBuffer->channels(); ++i) {
 			if (m_ppSrcState && m_ppSrcState[i])
 				src_reset(m_ppSrcState[i]);
-			m_ppFrames[i] = m_ppInBuffer[i];
+			m_ppInBuffer[i] = m_ppFrames[i];
 		}
 	}
 #endif
@@ -966,23 +967,16 @@ int qtractorAudioBuffer::readBuffer ( unsigned int iFrames )
 	if (m_bResample) {
 
 		if (iFrames > m_iInputPending)
-			nread = m_pFile->read(m_ppFrames, iFrames - m_iInputPending);
+			nread = m_pFile->read(m_ppInBuffer, iFrames - m_iInputPending);
 
 		nread += m_iInputPending;
 
-		if (m_ppOutBuffer == NULL) {
-			if (nread > 0)
-				return writeFrames(m_ppFrames, nread);
-			else
-				return flushFrames(iFrames); // Maybe EoF!
-		}
-
 		int ngen = 0;
 		SRC_DATA src_data;
 
 		for (unsigned short i = 0; i < m_pRingBuffer->channels(); ++i) {
 			// Fill all resampler parameter data...
-			src_data.data_in       = m_ppInBuffer[i];
+			src_data.data_in       = m_ppFrames[i];
 			src_data.data_out      = m_ppOutBuffer[i];
 			src_data.input_frames  = nread;
 			src_data.output_frames = iFrames;
@@ -997,11 +991,11 @@ int qtractorAudioBuffer::readBuffer ( unsigned int iFrames )
 					ngen = src_data.output_frames_gen;
 				}
 				if (m_iInputPending > 0 && src_data.input_frames_used > 0) {
-					::memmove(m_ppInBuffer[i],
-						m_ppInBuffer[i] + src_data.input_frames_used,
+					::memcpy(m_ppFrames[i],
+						m_ppFrames[i] + src_data.input_frames_used,
 						m_iInputPending * sizeof(float));
 				}
-				m_ppFrames[i] = m_ppInBuffer[i] + m_iInputPending;
+				m_ppInBuffer[i] = m_ppFrames[i] + m_iInputPending;
 			}
 		}
 
@@ -1140,10 +1134,6 @@ void qtractorAudioBuffer::deleteIOBuffers (void)
 			delete [] m_ppOutBuffer[i];
 			m_ppOutBuffer[i] = NULL;
 		}
-		if (m_ppInBuffer && m_ppInBuffer[i]) {
-			delete [] m_ppInBuffer[i];
-			m_ppInBuffer[i] = NULL;
-		}
 	}
 	if (m_ppSrcState) {
 		delete [] m_ppSrcState;
@@ -1161,6 +1151,12 @@ void qtractorAudioBuffer::deleteIOBuffers (void)
 #endif
 
 	if (m_ppFrames) {
+		for (unsigned short i = 0; i < m_pRingBuffer->channels(); ++i) {
+			if (m_ppFrames[i]) {
+				delete [] m_ppFrames[i];
+				m_ppFrames[i] = NULL;
+			}
+		}
 		delete [] m_ppFrames;
 		m_ppFrames = NULL;
 	}
@@ -1460,9 +1456,9 @@ void qtractorAudioBufferThread::run (void)
 	m_bRunState = m_pAudioBuffer->initSync();
 	while (m_bRunState) {
 		// Do whatever we must, then wait for more...
-		m_mutex.unlock();
+		//m_mutex.unlock();
 		m_pAudioBuffer->sync();
-		m_mutex.lock();
+		//m_mutex.lock();
 		m_cond.wait(&m_mutex);
 	}
 	m_mutex.unlock();
