@@ -1,7 +1,7 @@
 // qtractorAudioConnect.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -352,7 +352,7 @@ void qtractorAudioConnect::disconnectPortsUpdate (
 				qDeleteAll(pBus->outputs());
 				pBus->outputs().clear();
 			}
-			return;
+			break;
 		}
 	}
 }
