@@ -1,7 +1,7 @@
 // qtractorAudioEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,21 +44,29 @@
 // SSE detection.
 static inline bool sse_enabled (void)
 {
-	bool bSSE = false;
 #if defined(__GNUC__)
-#if defined(__i386__) || defined(__x86_64__)
 	unsigned int eax, ebx, ecx, edx;
+#if defined(__x86_64__) || (!defined(PIC) && !defined(__PIC__))
 	__asm__ __volatile__ (
-		"movl %%ebx, %%esi\n\t" \
 		"cpuid\n\t" \
-		"xchgl %%ebx, %%esi" \
-		: "=a" (eax), "=S" (ebx), "=c" (ecx), "=d" (edx) \
-		: "0" (1));
+		: "=a" (eax), "=b" (ebx), "=c" (ecx), "=d" (edx) \
+		: "a" (1) : "cc");
+#else
+	__asm__ __volatile__ (
+		"push %%ebx\n\t" \
+		"cpuid\n\t" \
+		"movl %%ebx,%1\n\t" \
+		"pop %%ebx\n\t" \
+		: "=a" (eax), "=r" (ebx), "=c" (ecx), "=d" (edx) \
+		: "a" (1) : "cc");
 #endif
+	return (edx & (1 << 25));
+#else
+	return false;
 #endif
-	return bSSE;
 }
 
+
 // SSE enabled mix-down processor version.
 static inline void sse_buffer_add (
 	float **ppBuffer, float **ppFrames, unsigned int iFrames,
@@ -132,31 +140,29 @@ static void qtractorAudioEngine_timebase ( jack_transport_state_t,
 		= static_cast<qtractorAudioEngine *> (pvArg);
 
 	qtractorSession *pSession = pAudioEngine->session();
-	if (pSession) {
-		unsigned short iTicksPerBeat = pSession->ticksPerBeat();
-		unsigned short iBeatsPerBar  = pSession->beatsPerBar();
-		unsigned int   bars  = 0;
-		unsigned int   beats = 0;
-		unsigned long  ticks = pSession->tickFromFrame(pPos->frame);
-		if (ticks >= (unsigned long) iTicksPerBeat) {
-			beats  = (unsigned int)  (ticks / iTicksPerBeat);
-			ticks -= (unsigned long) (beats * iTicksPerBeat);
-		}
-		if (beats >= (unsigned int) iBeatsPerBar) {
-			bars   = (unsigned int) (beats / iBeatsPerBar);
-			beats -= (unsigned int) (bars  * iBeatsPerBar);
-		}
-		// Time frame code in bars.beats.ticks ...
-		pPos->valid = JackPositionBBT;
-		pPos->bar   = bars  + 1;
-		pPos->beat  = beats + 1;
-		pPos->tick  = ticks;
-		// Keep current tempo (BPM)...
-		pPos->beats_per_bar    = iBeatsPerBar;
-		pPos->ticks_per_beat   = iTicksPerBeat;
-		pPos->beats_per_minute = pSession->tempo();
-	//	pPos->beat_type        = 4.0;	// Quarter note.
+	qtractorTimeScale::Cursor& cursor = pSession->timeScale()->cursor();
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(pPos->frame);
+	unsigned short bars  = 0;
+	unsigned int   beats = 0;
+	unsigned long  ticks = pNode->tickFromFrame(pPos->frame) - pNode->tick;
+	if (ticks >= (unsigned long) pNode->ticksPerBeat) {
+		beats  = (unsigned int) (ticks / pNode->ticksPerBeat);
+		ticks -= (unsigned long) (beats * pNode->ticksPerBeat);
 	}
+	if (beats >= (unsigned int) pNode->beatsPerBar) {
+		bars   = (unsigned short) (beats / pNode->beatsPerBar);
+		beats -= (unsigned int) (bars * pNode->beatsPerBar);
+	}
+	// Time frame code in bars.beats.ticks ...
+	pPos->valid = JackPositionBBT;
+	pPos->bar   = pNode->bar + bars + 1;
+	pPos->beat  = beats + 1;
+	pPos->tick  = ticks;
+	// Keep current tempo (BPM)...
+	pPos->beats_per_bar    = pNode->beatsPerBar;
+	pPos->ticks_per_beat   = pNode->ticksPerBeat;
+	pPos->beats_per_minute = pNode->tempo;
+	pPos->beat_type        = float(1 << pNode->beatDivisor);
 }
 
 
@@ -399,6 +405,11 @@ void qtractorAudioEngine::shutdown (void)
 // Device engine initialization method.
 bool qtractorAudioEngine::init ( const QString& sClientName )
 {
+	// There must a session reference...
+	qtractorSession *pSession = session();
+	if (pSession == NULL)
+		return false;
+
 	// Try open a new client...
 	m_pJackClient = jack_client_open(
 		sClientName.toUtf8().constData(), JackUseExactName, NULL);
@@ -406,7 +417,7 @@ bool qtractorAudioEngine::init ( const QString& sClientName )
 		return false;
 
 	// ATTN: First thing to remember to set session sample rate.
-	session()->setSampleRate(sampleRate());
+	pSession->setSampleRate(sampleRate());
 
 	// Open player/metronome buses, at least try...
 	openPlayerBus();
@@ -414,7 +425,7 @@ bool qtractorAudioEngine::init ( const QString& sClientName )
 
 	// MIDI plugin managers output buses...
 	qtractorMidiManager *pMidiManager
-		= session()->midiManagers().first();
+		= pSession->midiManagers().first();
 	while (pMidiManager) {
 		qtractorAudioBus *pAudioBus = pMidiManager->audioOutputBus();
 		if (pAudioBus && pMidiManager->isAudioOutputBus())
@@ -429,6 +440,9 @@ bool qtractorAudioEngine::init ( const QString& sClientName )
 // Device engine activation method.
 bool qtractorAudioEngine::activate (void)
 {
+	// Ensure (not) freewheeling state...
+	jack_set_freewheel(m_pJackClient, 0);
+
 	// Set our main engine processor callbacks.
 	jack_set_process_callback(m_pJackClient,
 			qtractorAudioEngine_process, this);
@@ -495,7 +509,7 @@ bool qtractorAudioEngine::start (void)
 	// Make sure we have an actual session cursor...
 	resetMetro();
 
-	// Start trnsport rolling...
+	// Start transport rolling...
 	jack_transport_start(m_pJackClient);
 
 	// We're now ready and running...
@@ -579,13 +593,13 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 	if (m_bFreewheel) {
 		// Make sure we're in a valid state...
 		if (m_pExportFile && m_pExportBus && !m_bExportDone) {
-			// Force/sync every audio clip approaching...
-			syncExport();
 			// This the legal process cycle frame range...
 			unsigned long iFrameStart = pAudioCursor->frame();
 			unsigned long iFrameEnd   = iFrameStart + nframes;
 			// Write output bus buffer to export audio file...
 			if (iFrameStart < m_iExportEnd && iFrameEnd > m_iExportStart) {
+				// Force/sync every audio clip approaching...
+				syncExport(iFrameStart, iFrameEnd);
 				// Prepare the output bus only...
 				m_pExportBus->process_prepare(nframes);
 				// Export cycle...
@@ -713,9 +727,11 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 	unsigned long iFrameEnd   = iFrameStart + nframes;
 
 	// Metronome stuff...
+	qtractorTimeScale::Cursor& cursor = pSession->timeScale()->cursor();
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(iFrameStart);
 	if (m_bMetronome && m_pMetroBus && iFrameEnd > m_iMetroBeatStart) {
 		qtractorAudioBuffer *pMetroBuff
-			= (pSession->beatIsBar(m_iMetroBeat)
+			= (pNode->beatIsBar(m_iMetroBeat)
 				? m_pMetroBarBuff : m_pMetroBeatBuff);
 		if (iFrameStart < m_iMetroBeatStart) {
 			pMetroBuff->readMix(m_pMetroBus->out(),
@@ -725,7 +741,7 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 			pMetroBuff->readMix(m_pMetroBus->out(),
 				nframes, m_pMetroBus->channels(), 0, 1.0f);
 		} else {
-			m_iMetroBeatStart = pSession->frameFromBeat(++m_iMetroBeat);
+			m_iMetroBeatStart = pNode->frameFromBeat(++m_iMetroBeat);
 			pMetroBuff->reset(false);
 		}
 		if (m_bMetroBus)
@@ -965,11 +981,10 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	pSession->setPlayHead(m_iExportStart);
 	pExportBus->setPassthru(false);
 
-	// Force sync...
-	syncExport();
+	// Force initial full-sync...
+	syncExport(m_iExportStart, m_iExportEnd);
 
 	// Special initialization.
-	pSession->resetAllPlugins();
     m_iBufferOffset = 0;
 
 	// Start export (freewheeling)...
@@ -1008,7 +1023,8 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 
 
 // Direct sync method (needed for export)
-void qtractorAudioEngine::syncExport (void)
+void qtractorAudioEngine::syncExport (
+	unsigned long iFrameStart, unsigned long iFrameEnd )
 {
 	qtractorSession *pSession = session();
 	if (pSession == NULL)
@@ -1018,22 +1034,22 @@ void qtractorAudioEngine::syncExport (void)
 	if (pAudioCursor == NULL)
 		return;
 
-	unsigned long iFrameSync = pAudioCursor->frame() + sampleRate();
-
 	int iTrack = 0;
 	for (qtractorTrack *pTrack = pSession->tracks().first();
 			pTrack; pTrack = pTrack->next()) {
 		if (pTrack->trackType() == qtractorTrack::Audio) {
-			qtractorAudioClip *pAudioClip
-				= static_cast<qtractorAudioClip *> (
-					pAudioCursor->clip(iTrack));
-			while (pAudioClip && pAudioClip->clipStart() < iFrameSync) {
-				pAudioClip->syncExport();
-				pAudioClip = static_cast<qtractorAudioClip *> (
-					pAudioClip->next());
+			qtractorClip *pClip = pAudioCursor->clip(iTrack);
+			while (pClip
+				&& pClip->clipStart() < iFrameEnd
+				&& pClip->clipStart() + pClip->clipLength() > iFrameStart) {
+				qtractorAudioClip *pAudioClip
+					= static_cast<qtractorAudioClip *> (pClip);
+				if (pAudioClip)
+					pAudioClip->syncExport();
+				pClip = pClip->next();
 			}
 		}
-		iTrack++;
+		++iTrack;
 	}
 }
 
@@ -1210,19 +1226,23 @@ void qtractorAudioEngine::resetMetro (void)
 	if (!m_bMetronome)
 		return;
 
-	qtractorSession *pSession = session();
-	if (pSession == NULL)
-		return;
-
 	qtractorSessionCursor *pAudioCursor = sessionCursor();
 	if (pAudioCursor == NULL)
 		return;
 
-	// Reset the next beat position...
-	m_iMetroBeat = pSession->beatFromFrame(pAudioCursor->frame()) + 1;
-	m_iMetroBeatStart = pSession->frameFromBeat(m_iMetroBeat);
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
 
-	// Now each sample buffer must be bounded properly...
+	// Reset to the next beat position...
+	unsigned long iFrame = pAudioCursor->frame();
+	qtractorTimeScale::Cursor& cursor = pSession->timeScale()->cursor();
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(iFrame);
+
+	m_iMetroBeat = pNode->beatFromFrame(iFrame) + 1;
+	m_iMetroBeatStart = pNode->frameFromBeat(m_iMetroBeat);
+
+	// FIXME: Each sample buffer must be bounded properly...
 	unsigned long iMaxLength = (m_iMetroBeatStart / m_iMetroBeat);
 
 	if (m_pMetroBarBuff) {
