@@ -1,7 +1,7 @@
 // qtractorAudioEngine.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -97,7 +97,7 @@ public:
 		qtractorAudioBus *pExportBus = NULL);
 
 	// Direct sync method (needed for export)
-	void syncExport();
+	void syncExport(unsigned long iFrameStart, unsigned long iFrameEnd);
 
 	// Metronome switching.
 	void setMetronome(bool bMetronome);
@@ -173,12 +173,12 @@ private:
 	bool m_bFreewheel;
 
 	// Audio-export (in)active state.
-	bool                 m_bExporting;
+	volatile bool        m_bExporting;
 	qtractorAudioBus    *m_pExportBus;
 	qtractorAudioFile   *m_pExportFile;
 	unsigned long        m_iExportStart;
 	unsigned long        m_iExportEnd;
-	bool                 m_bExportDone;
+	volatile bool        m_bExportDone;
 
 	// Audio metronome stuff.
 	bool                 m_bMetronome;
