@@ -182,6 +182,21 @@ void qtractorClipCommand::pitchShiftClip ( qtractorClip *pClip,
 }
 
 
+void qtractorClipCommand::resetClip ( qtractorClip *pClip,
+	unsigned long iClipLength )
+{
+	Item *pItem = new Item(ResetClip, pClip, pClip->track());
+	pItem->clipLength = iClipLength;
+	long iFadeOutLength = long(pClip->fadeOutLength());
+	if (iFadeOutLength > 0) {
+		iFadeOutLength += long(iClipLength)	- long(pClip->clipLength());
+		if (iFadeOutLength > 0)
+			pItem->fadeOutLength = iFadeOutLength;
+	}
+	m_items.append(pItem);
+}
+
+
 // Special clip record nethod.
 bool qtractorClipCommand::addClipRecord ( qtractorTrack *pTrack )
 {
@@ -264,6 +279,13 @@ void qtractorClipCommand::addTrack ( qtractorTrack *pTrack )
 }
 
 
+// Composite predicate.
+bool qtractorClipCommand::isEmpty (void) const
+{
+	return m_trackCommands.isEmpty() && m_items.isEmpty();
+}
+
+
 // Common executive method.
 bool qtractorClipCommand::execute ( bool bRedo )
 {
@@ -368,18 +390,23 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			if (pTrack->trackType() == qtractorTrack::Audio
 				&& pItem->timeStretch > 0.0f)
 				pAudioClip = static_cast<qtractorAudioClip *> (pClip);
-			if (pAudioClip)
+			if (pAudioClip) {
 				fOldTimeStretch = pAudioClip->timeStretch();
+				pAudioClip->close(true);	// Scrap peak file.
+			}
+			if (iOldStart != pItem->clipStart)
+				pTrack->unlinkClip(pClip);
 			pClip->setClipStart(pItem->clipStart);
 			pClip->setClipOffset(pItem->clipOffset);
 			pClip->setClipLength(pItem->clipLength);
 			pClip->setFadeInLength(pItem->fadeInLength);
 			pClip->setFadeOutLength(pItem->fadeOutLength);
-			if (pAudioClip) {
+			if (pAudioClip)
 				pAudioClip->setTimeStretch(pItem->timeStretch);
-				pAudioClip->close(true);	// Scrap peak file.
-			}
-			pClip->open();
+			if (iOldStart != pItem->clipStart)
+				pTrack->addClip(pClip);
+			else
+				pClip->open();
 			pItem->clipStart  = iOldStart;
 			pItem->clipOffset = iOldOffset;
 			pItem->clipLength = iOldLength;
@@ -440,6 +467,17 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			}
 			break;
 		}
+		case ResetClip: {
+			unsigned long iOldLength  = pClip->clipLength();
+			unsigned long iOldFadeOut = pClip->fadeOutLength();
+			pClip->setClipLength(pItem->clipLength);
+			pClip->setFadeOutLength(pItem->fadeOutLength);
+			pClip->open();
+			pItem->clipLength = iOldLength;
+			pItem->fadeOutLength = iOldFadeOut;
+			pSession->updateTrack(pTrack);
+			break;
+		}
 		default:
 			break;
 		}
