@@ -1,7 +1,7 @@
 // qtractorClipForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -93,7 +93,7 @@ qtractorClipForm::qtractorClipForm (
 		SLOT(formatChanged()));
 	QObject::connect(m_ui.ClipStartSpinBox,
 		SIGNAL(valueChanged(unsigned long)),
-		SLOT(changed()));
+		SLOT(clipStartChanged(unsigned long)));
 	QObject::connect(m_ui.ClipOffsetSpinBox,
 		SIGNAL(valueChanged(unsigned long)),
 		SLOT(changed()));
@@ -166,10 +166,7 @@ void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 	m_ui.FadeInLengthSpinBox->setTimeScale(m_pTimeScale);
 	m_ui.FadeOutLengthSpinBox->setTimeScale(m_pTimeScale);
 	// These have special delta formats...
-	m_ui.ClipOffsetSpinBox->setDeltaValue(true);
-	m_ui.ClipLengthSpinBox->setDeltaValue(true);
-	m_ui.FadeInLengthSpinBox->setDeltaValue(true);
-	m_ui.FadeOutLengthSpinBox->setDeltaValue(true);
+	clipStartChanged(m_pClip->clipStart());
 	// Initialize dialog widgets...
 	m_ui.ClipNameLineEdit->setText(m_pClip->clipName());
 	// Parameters...
@@ -474,11 +471,13 @@ void qtractorClipForm::reject (void)
 			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
-			tr("Apply"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Apply...
+			QMessageBox::Apply |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Apply:
 			accept();
 			return;
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bReject = false;
@@ -743,4 +742,17 @@ void qtractorClipForm::trackChannelChanged ( int iTrackChannel )
 }
 
 
+// Adjust delta-value spin-boxes to new anchor frame.
+void qtractorClipForm::clipStartChanged ( unsigned long iClipStart )
+{
+	m_ui.ClipOffsetSpinBox->setDeltaValue(true, iClipStart);
+	m_ui.ClipLengthSpinBox->setDeltaValue(true, iClipStart);
+	m_ui.FadeInLengthSpinBox->setDeltaValue(true, iClipStart);
+	m_ui.FadeOutLengthSpinBox->setDeltaValue(true, iClipStart);
+
+	if (m_iDirtySetup == 0)
+		changed();
+}
+
+
 // end of qtractorClipForm.cpp
