@@ -1,7 +1,7 @@
 // qtractorCommand.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -97,7 +97,11 @@ public:
 	// Remove last command from command chain.
 	void removeLastCommand();
 
+	// Special backout method (EXPERIMENTAL).
+	void backout(qtractorCommand *pCommand);
+
 	// Cannonical command methods.
+	bool push(qtractorCommand *pCommand);
 	bool exec(qtractorCommand *pCommand);
 
 	bool undo();
@@ -123,4 +127,3 @@ private:
 #endif	// __qtractorCommand_h
 
 // end of qtractorCommand.h
-
