@@ -1,7 +1,7 @@
 // qtractorConnectForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -102,10 +102,10 @@ qtractorConnectForm::qtractorConnectForm (
 
 	QObject::connect(m_pAudioConnect,
 		SIGNAL(connectChanged()),
-		SLOT(audioStabilize()));
+		SLOT(audioConnectChanged()));
 	QObject::connect(m_pMidiConnect,
 		SIGNAL(connectChanged()),
-		SLOT(midiStabilize()));
+		SLOT(midiConnectChanged()));
 }
 
 
@@ -199,6 +199,19 @@ void qtractorConnectForm::audioUpdate ( bool bClear )
 }
 
 
+// A helper connection change slot.
+void qtractorConnectForm::audioConnectChanged (void)
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorConnectForm::audioConectChanged()");
+#endif
+
+	audioStabilize();
+
+	emit connectChanged();
+}
+
+
 // A helper stabilization slot.
 void qtractorConnectForm::audioStabilize (void)
 {
@@ -297,6 +310,19 @@ void qtractorConnectForm::midiUpdate ( bool bClear )
 }
 
 
+// A helper connection change slot.
+void qtractorConnectForm::midiConnectChanged (void)
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorConnectForm::midiConectChanged()");
+#endif
+
+	midiStabilize();
+
+	emit connectChanged();
+}
+
+
 // A helper stabilization slot.
 void qtractorConnectForm::midiStabilize (void)
 {
