@@ -1,7 +1,7 @@
 // qtractorExportForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -217,7 +217,7 @@ void qtractorExportForm::accept (void)
 			"\"%1\"\n\n"
 			"Do you want to replace it?")
 			.arg(sExportPath),
-			tr("Replace"), tr("Cancel")) > 0) {
+			QMessageBox::Yes | QMessageBox::No) == QMessageBox::No) {
 			m_ui.ExportPathComboBox->setFocus();
 			return;
 		}
