@@ -59,6 +59,12 @@ public:
 	// Clear evrything on sight.
 	void clear();
 
+	// Check whether one of the widgets has focus (oveerride method).
+	bool hasFocus() const;
+
+	// Tell whether a file item is currently selected.
+	bool isFileSelected() const;
+
 	// File addition Convenience helper methods.
 	void addAudioFile (const QString& sFilename);
 	void addMidiFile  (const QString& sFilename);
@@ -71,15 +77,21 @@ public:
 	void setPlayState(bool bOn);
 	bool isPlayState() const;
 
-protected slots:
+public slots:
 
 	// Add a new group item below the current one.
 	void newGroupSlot();
 	// Add a new file item below the current group one.
 	void openFileSlot();
+	// Cut current file item(s) to clipboard.
+	void cutItemSlot();
+	// Copy current file item(s) to clipboard.
+	void copyItemSlot();
+	// Paste file item(s) from clipboard.
+	void pasteItemSlot();
 	// Rename current group/file item.
 	void renameItemSlot();
-	// Remove current group/file item.
+	// Remove current group/file item(s).
 	void deleteItemSlot();
 
 	// Audition/pre-listening player slots.
@@ -117,6 +129,9 @@ private:
 	// List view actions.
 	QAction *m_pNewGroupAction;
 	QAction *m_pOpenFileAction;
+	QAction *m_pCutItemAction;
+	QAction *m_pCopyItemAction;
+	QAction *m_pPasteItemAction;
 	QAction *m_pRenameItemAction;
 	QAction *m_pDeleteItemAction;
 	QAction *m_pPlayItemAction;
