@@ -1,7 +1,7 @@
 // qtractorInstrumentForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -202,7 +202,7 @@ void qtractorInstrumentForm::importSlot (void)
 			if (pItem) {
 				QFileInfo info(sPath);
 				pItem->setIcon(0, QIcon(":/icons/itemFile.png"));
-				pItem->setText(0, info.fileName());
+				pItem->setText(0, info.completeBaseName());
 				pItem->setText(1, sPath);
 				m_ui.FilesListView->setCurrentItem(pItem);
 				pOptions->sInstrumentDir = info.absolutePath();
@@ -327,7 +327,7 @@ void qtractorInstrumentForm::exportSlot (void)
 	const QString& sFilter = tr("Instrument files (*.%1)").arg(sExt);
 #if QT_VERSION < 0x040400
 	// Ask for the filename to open...
-	sPath = QFileDialog::getOpenFileName(this,
+	sPath = QFileDialog::getSaveFileName(this,
 		sTitle, pOptions->sInstrumentDir, sFilter);
 #else
 	// Construct open-files dialog...
@@ -351,19 +351,19 @@ void qtractorInstrumentForm::exportSlot (void)
 		return;
 
 	// Enforce .ins extension...
-	if (QFileInfo(sPath).suffix().isEmpty())
+	if (QFileInfo(sPath).suffix() != sExt) {
 		sPath += '.' + sExt;
-
-	// Check if already exists...
-	if (QFileInfo(sPath).exists()) {
-		if (QMessageBox::warning(this,
-			tr("Warning") + " - " QTRACTOR_TITLE,
-			tr("The instrument file already exists:\n\n"
-			"\"%1\"\n\n"
-			"Do you want to replace it?")
-			.arg(sPath),
-			tr("Replace"), tr("Cancel")) > 0)
-			return;
+		// Check if already exists...
+		if (QFileInfo(sPath).exists()) {
+			if (QMessageBox::warning(this,
+				tr("Warning") + " - " QTRACTOR_TITLE,
+				tr("The instrument file already exists:\n\n"
+				"\"%1\"\n\n"
+				"Do you want to replace it?")
+				.arg(sPath),
+				QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
+				return;
+		}
 	}
 
 	// Just save the whole bunch...
@@ -395,11 +395,13 @@ void qtractorInstrumentForm::reject (void)
 			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("Instrument settings have been changed.") + "\n\n" +
 			tr("Do you want to apply the changes?"),
-			tr("Apply"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Apply...
+			QMessageBox::Apply |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Apply:
 			accept();
 			return;
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bReject = false;
@@ -461,7 +463,7 @@ void qtractorInstrumentForm::refreshForm (void)
 		const QString& sPath = ifile.next();
 		QTreeWidgetItem *pFileItem = new QTreeWidgetItem();
 		pFileItem->setIcon(0, QIcon(":/icons/itemFile.png"));
-		pFileItem->setText(0, QFileInfo(sPath).fileName());
+		pFileItem->setText(0, QFileInfo(sPath).completeBaseName());
 		pFileItem->setText(1, sPath);
 		files.append(pFileItem);
 	}
