@@ -1,7 +1,7 @@
 // qtractorInstrumentForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -79,7 +79,7 @@ private:
 	Ui::qtractorInstrumentForm m_ui;
 
 	// Instance variables...
-	int *m_iDirtyCount;
+	int m_iDirtyCount;
 };
 
 
