@@ -1,7 +1,7 @@
 // qtractorMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -39,7 +39,11 @@ class qtractorConnections;
 class qtractorMixer;
 class qtractorMmcEvent;
 class qtractorMidiControlEvent;
-class qtractorSpinBox;
+class qtractorMidiControl;
+class qtractorMidiSppEvent;
+class qtractorTimeSpinBox;
+class qtractorTempoSpinBox;
+class qtractorTempoCursor;
 
 class qtractorMidiEditorForm;
 class qtractorMidiEditor;
@@ -48,7 +52,6 @@ class QLabel;
 class QComboBox;
 class QProgressBar;
 class QActionGroup;
-class QDoubleSpinBox;
 class QPalette;
 
 
@@ -75,7 +78,7 @@ public:
 	qtractorConnections *connections() const;
 	qtractorMixer *mixer() const;
 	qtractorThumbView *thumbView() const;
-
+	
 	QString sessionName(const QString& sFilename);
 
 	void updateTransportTime(unsigned long iPlayHead);
@@ -127,7 +130,9 @@ public slots:
 	void editClipNew();
 	void editClipEdit();
 	void editClipSplit();
+	void editClipMerge();
 	void editClipNormalize();
+	void editClipQuantize();
 	void editClipExport();
 
 	void trackAdd();
@@ -170,10 +175,15 @@ public slots:
 	void viewZoomIn();
 	void viewZoomOut();
 	void viewZoomReset();
+	void viewZoomHorizontal();
+	void viewZoomVertical();
+	void viewZoomAll();
 	void viewSnap();
 	void viewRefresh();
 	void viewInstruments();
+	void viewControllers();
 	void viewBuses();
+	void viewTempoMap();
 	void viewOptions();
 
 	void transportBackward();
@@ -204,6 +214,7 @@ public slots:
 
 	void updateRecentFilesMenu();
 	void updateExportMenu();
+	void updateZoomMenu();
 	void updateSnapMenu();
 
 	void addAudioFile(const QString& sFilename);
@@ -219,12 +230,14 @@ public slots:
 	void changeNotifySlot(qtractorMidiEditor *pMidiEditor);
 	void updateNotifySlot(bool bRefresh);
 
-	void tempoChanged(double fTempo);
-	void snapPerBeatChanged(int iSnap);
 	void transportTimeChanged(unsigned long iPlayHead);
+	void transportTempoChanged(float fTempo,
+		unsigned short iBeatsPerBar, unsigned short iBeatDivisor);
+	void snapPerBeatChanged(int iSnap);
 	void contentsChanged();
 
 	void transportTimeContextMenu(const QPoint& pos);
+	void transportTempoContextMenu(const QPoint& pos);
 
 protected:
 
@@ -235,6 +248,7 @@ protected:
 
 	void mmcEvent(qtractorMmcEvent *pMmcEvent);
 	void midiControlEvent(qtractorMidiControlEvent *pCtlEvent);
+	void midiSppEvent(qtractorMidiSppEvent *pSppEvent);
 
 	bool queryClose();
 
@@ -261,6 +275,8 @@ protected:
 
 	void setTrack(int scmd, int iTrack, bool bOn);
 
+	void setSongPos(unsigned short iSongPos);
+
 	void updateSession();
 	void updateRecentFiles(const QString& sFilename);
 	void updatePeakAutoRemove();
@@ -293,11 +309,13 @@ private:
 	int m_iUntitled;
 	int m_iDirtyCount;
 	QActionGroup *m_pSelectModeActionGroup;
-	qtractorSpinBox *m_pTransportTimeSpinBox;
-	QDoubleSpinBox *m_pTempoSpinBox;
+	qtractorTimeSpinBox *m_pTimeSpinBox;
+	qtractorTempoSpinBox *m_pTempoSpinBox;
 	QComboBox *m_pSnapPerBeatComboBox;
 	QProgressBar *m_pProgressBar;
 	qtractorThumbView *m_pThumbView;
+	qtractorTempoCursor *m_pTempoCursor;
+	qtractorMidiControl *m_pMidiControl;
 	unsigned long m_iPlayHead;
 	long m_iDeltaTimer;
 	int m_iPeakTimer;
@@ -305,6 +323,7 @@ private:
 	int m_iTransportUpdate;
 	int m_iTransportDelta;
 	int m_iTransportRolling;
+	bool m_bTransportPlaying;
 	float m_fTransportShuttle;
 	int m_iTransportStep;
 	int m_iXrunCount;
