@@ -1,7 +1,7 @@
 // qtractorMidiClip.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -133,23 +133,49 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 	// Initialize event container...
 	m_pSeq->clear();
 	m_pSeq->setTicksPerBeat(pSession->ticksPerBeat());
-	m_pSeq->setTimeOffset(pSession->tickFromFrame(clipOffset()));
-	m_pSeq->setTimeLength(pSession->tickFromFrame(clipLength()));
+
+	qtractorTimeScale::Cursor cursor(pSession->timeScale());
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(clipStart());
+	unsigned long t0 = pNode->tickFromFrame(clipStart());
+
+	if (clipStart() > clipOffset()) {
+		unsigned long iOffset = clipStart() - clipOffset();
+		pNode = cursor.seekFrame(iOffset);
+		m_pSeq->setTimeOffset(t0 - pNode->tickFromFrame(iOffset));
+	} else {
+		pNode = cursor.seekFrame(clipOffset());
+		m_pSeq->setTimeOffset(pNode->tickFromFrame(clipOffset()));
+	}
+
+	unsigned long iLength = clipStart() + clipLength();
+	pNode = cursor.seekFrame(iLength);
+	m_pSeq->setTimeLength(pNode->tickFromFrame(iLength) - t0);
 
 	// Are we on a pre-writing status?
 	if (pFile->mode() == qtractorMidiFile::Write) {
 		// On write mode, iTrackChannel holds the SMF format,
 		// so we'll convert it here as properly.
+		unsigned short iFormat = 0;
+		unsigned short iTracks = 1;
 		if (iTrackChannel == 0) {
-			setFormat(0); // SMF format 0 (1 track, 1 channel)
+			// SMF format 0 (1 track, 1 channel)
 			iTrackChannel = pTrack->midiChannel();
 		} else {
-			setFormat(1); // SMF format 1 (2 tracks, 1 channel)
+			// SMF format 1 (2 tracks, 1 channel)
 			iTrackChannel = 1;
+			iFormat = 1;
+			iTracks++;
+		}
+		// That's it.
+		setFormat(iFormat);
+		// Write SMF header...
+		if (pFile->writeHeader(iFormat, iTracks, m_pSeq->ticksPerBeat())) {
+			// Set initial local properties...
+			if (pFile->tempoMap()) {
+				pFile->tempoMap()->fromTimeScale(
+					pSession->timeScale(), m_pSeq->timeOffset());
+			}
 		}
-		// Set initial local properties...
-		pFile->setTempo(pSession->tempo());
-		pFile->setBeatsPerBar(pSession->beatsPerBar());
 		// And initial clip name...
 		m_pSeq->setName(QFileInfo(pFile->filename()).baseName());
 		m_pSeq->setChannel(pTrack->midiChannel());
@@ -161,9 +187,8 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 		if (!pFile->readTrack(m_pSeq, iTrackChannel))
 			return false;
 		// FIXME: On demand, set session time properties from MIDI file...
-		if (m_bSessionFlag) {
-			pSession->setTempo(pFile->tempo());
-			pSession->setBeatsPerBar(pFile->beatsPerBar());
+		if (m_bSessionFlag && pFile->tempoMap()) {
+			pFile->tempoMap()->intoTimeScale(pSession->timeScale(), t0);
 			pSession->updateTimeScale();
 			// Reset session flag now.
 			m_bSessionFlag = false;
@@ -186,8 +211,9 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 
 	// Default clip length will be whole sequence duration.
 	if (clipLength() == 0 && m_pSeq->timeLength() > m_pSeq->timeOffset()) {
-		setClipLength(pSession->frameFromTick(
-			m_pSeq->timeLength() - m_pSeq->timeOffset()));
+		unsigned long t1 = t0 + (m_pSeq->timeLength() - m_pSeq->timeOffset());
+		pNode = cursor.seekTick(t1);
+		setClipLength(pNode->frameFromTick(t1) - clipStart());
 	}
 
 	// Uh oh...
@@ -296,8 +322,11 @@ void qtractorMidiClip::seek ( unsigned long iFrame )
 	if (pSession == NULL)
 		return;
 
+	unsigned long t0 = pSession->tickFromFrame(clipStart());
+	unsigned long t1 = pSession->tickFromFrame(iFrame);
+
 	// Seek for the nearest sequence event...
-	m_playCursor.seek(m_pSeq, pSession->tickFromFrame(iFrame));
+	m_playCursor.seek(m_pSeq, (t1 > t0 ? t1 - t0 : 0));
 }
 
 
@@ -358,11 +387,6 @@ void qtractorMidiClip::close ( bool bForce )
 	// Now's time to write the whole thing...
 	bool bNewFile = (m_pFile && m_pFile->mode() == qtractorMidiFile::Write);
 	if (bNewFile && iClipLength > 0) {
-		// Write SMF header...
-		unsigned short iTracks = 1;
-		if (m_iFormat == 1)
-			iTracks++;
-		m_pFile->writeHeader(m_iFormat, iTracks, m_pSeq->ticksPerBeat());
 		// Write channel tracks...
 		if (m_iFormat == 1)
 			m_pFile->writeTrack(NULL);  // Setup track (SMF format 1).
@@ -432,25 +456,22 @@ void qtractorMidiClip::process ( unsigned long iFrameStart,
 	bool bMute = (pTrack->isMute()
 		|| (pSession->soloTracks() && !pTrack->isSolo()));
 
-	unsigned long iTimeClip  = pSession->tickFromFrame(clipStart());
+	unsigned long t0 = pSession->tickFromFrame(clipStart());
+
 	unsigned long iTimeStart = pSession->tickFromFrame(iFrameStart);
 	unsigned long iTimeEnd   = pSession->tickFromFrame(iFrameEnd);
 
-	// Set precise event cursory positioning...
-	unsigned long iTimeSeek = 0;
-	if (iTimeStart > iTimeClip)
-		iTimeSeek = iTimeStart - iTimeClip;
-
 	// Enqueue the requested events...
-	qtractorMidiEvent *pEvent = m_playCursor.seek(m_pSeq, iTimeSeek);
+	qtractorMidiEvent *pEvent
+		= m_playCursor.seek(m_pSeq, iTimeStart > t0 ? iTimeStart - t0 : 0);
 	while (pEvent) {
-		unsigned long iTimeEvent = iTimeClip + pEvent->time();
-		if (iTimeEvent >= iTimeEnd)
+		unsigned long t1 = t0 + pEvent->time();
+		if (t1 >= iTimeEnd)
 			break;
-		if (iTimeEvent >= iTimeStart
+		if (t1 >= iTimeStart
 			&& (!bMute || pEvent->type() != qtractorMidiEvent::NOTEON))
-			pMidiEngine->enqueue(pTrack, pEvent, iTimeEvent,
-				gain(pSession->frameFromTick(pEvent->time())));
+			pMidiEngine->enqueue(pTrack, pEvent, t1,
+				gain(pSession->frameFromTick(t1) - clipStart()));
 		pEvent = pEvent->next();
 	}
 }
@@ -469,32 +490,44 @@ void qtractorMidiClip::draw ( QPainter *pPainter, const QRect& clipRect,
 	if (iNoteSpan < 4)
 		iNoteSpan = 4;
 
-	unsigned long iTimeStart = pSession->tickFromFrame(iClipOffset);
-	unsigned long iTimeEnd
-		= iTimeStart + pSession->tickFromPixel(clipRect.width());
+	unsigned long iFrameStart = clipStart() + iClipOffset;
+	int cx = pSession->pixelFromFrame(iFrameStart);
+
+	qtractorTimeScale::Cursor cursor(pSession->timeScale());
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(clipStart());
+	unsigned long t0 = pNode->tickFromFrame(clipStart());
+
+	pNode = cursor.seekFrame(iFrameStart);	
+	unsigned long iTimeStart = pNode->tickFromFrame(iFrameStart);
+	pNode = cursor.seekPixel(cx + clipRect.width());	
+	unsigned long iTimeEnd = pNode->tickFromPixel(cx + clipRect.width());
 
 	const QColor& fg = track()->foreground();
 	pPainter->setPen(fg);
 	pPainter->setBrush(fg.lighter());
 
-	int cx = pSession->pixelFromTick(iTimeStart);
-	int h1 = clipRect.height() - 4;
+	bool bClipRecord = (track()->clipRecord() == this);
+	int h1 = clipRect.height() - 3;
 	int h  = h1 / iNoteSpan;
-	if (h < 4) h = 4;
+	if (h < 3) h = 3;
 
-	qtractorMidiEvent *pEvent = m_drawCursor.reset(m_pSeq, iTimeStart);
-	while (pEvent && pEvent->time() < iTimeEnd) {
-		if (pEvent->type() == qtractorMidiEvent::NOTEON
-			&& pEvent->time() + pEvent->duration() >= iTimeStart) {
-			int x = clipRect.x()
-				+ pSession->pixelFromTick(pEvent->time()) - cx;
+	qtractorMidiEvent *pEvent
+		= m_drawCursor.reset(m_pSeq, iTimeStart > t0 ? iTimeStart - t0 : 0);
+	while (pEvent) {
+		unsigned long t1 = t0 + pEvent->time();
+		if (t1 >= iTimeEnd)
+			break;
+		unsigned long t2 = t1 + pEvent->duration();
+		if (pEvent->type() == qtractorMidiEvent::NOTEON && t2 >= iTimeStart) {
+			pNode = cursor.seekTick(t1);
+			int x = clipRect.x() + pNode->pixelFromTick(t1) - cx;
 			int y = clipRect.bottom()
 				- (h1 * (pEvent->note() - m_pSeq->noteMin() + 1)) / iNoteSpan;
-			int w = (pEvent->duration() > 0
-				? pSession->pixelFromTick(pEvent->duration())
-				: clipRect.right() - x); // Pending note-off? (while recording)
-			if (h > 4) {
-				if (w < 5) w = 5;
+			int w = (pEvent->duration() > 0 || !bClipRecord
+				? clipRect.x() + pNode->pixelFromTick(t2) - cx
+				: clipRect.right()) - x; // Pending note-off? (while recording)
+			if (h > 3 && w > 3) {
+			//	if (w < 5) w = 5;
 				pPainter->fillRect(x, y, w, h - 1, fg);
 				pPainter->fillRect(x + 1, y + 1, w - 4, h - 4, fg.lighter());
 			} else {
@@ -520,6 +553,9 @@ bool qtractorMidiClip::startEditor ( QWidget *pParent )
 		Qt::WindowFlags wflags = Qt::Window
 		#if QT_VERSION >= 0x040200
 			| Qt::CustomizeWindowHint
+		#if QT_VERSION >= 0x040500
+			| Qt::WindowCloseButtonHint
+		#endif
 		#endif
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
@@ -556,27 +592,37 @@ bool qtractorMidiClip::queryEditor (void)
 }
 
 
+// Clip editor reset.
+void qtractorMidiClip::resetEditor (void)
+{
+	if (m_pMidiEditorForm) {
+		qtractorMidiEditor *pMidiEditor = m_pMidiEditorForm->editor();
+		if (pMidiEditor)
+			pMidiEditor->reset();
+	}
+}
+
+
 // Clip editor update.
 void qtractorMidiClip::updateEditor (void)
 {
 	if (m_pMidiEditorForm == NULL)
 		return;
 
-	qtractorTrack *pTrack = track();
-	if (pTrack == NULL)
-		return;
-
 	qtractorMidiEditor *pMidiEditor = m_pMidiEditorForm->editor();
 	if (pMidiEditor) {
+		pMidiEditor->reset();
 		pMidiEditor->setOffset(clipStart());
 		pMidiEditor->setLength(clipLength());
-		pMidiEditor->setForeground(pTrack->foreground());
-		pMidiEditor->setBackground(pTrack->background());
+		qtractorTrack *pTrack = track();
+		if (pTrack) {
+			pMidiEditor->setForeground(pTrack->foreground());
+			pMidiEditor->setBackground(pTrack->background());
+		}
 		pMidiEditor->updateContents();
 	}
 
 	m_pMidiEditorForm->updateInstrumentNames();
-
 	m_pMidiEditorForm->stabilizeForm();
 }
 
@@ -593,7 +639,7 @@ QString qtractorMidiClip::toolTip (void) const
 		sToolTip += QObject::tr("Track %1").arg(m_iTrackChannel);
 
 	if (m_pFile) {
-		sToolTip += QObject::tr(", %1 tracks, %2 tpb")
+		sToolTip += QObject::tr(", %1 tracks, %2 tpqn")
 			.arg(m_pFile->tracks())
 			.arg(m_pFile->ticksPerBeat());
 	}
@@ -649,9 +695,11 @@ bool qtractorMidiClip::saveClipElement (
 			qtractorMidiFile::saveCopyFile(sFilename,
 				qtractorMidiClip::filename(),
 				qtractorMidiClip::trackChannel(),
+				qtractorMidiClip::format(),
 				qtractorMidiClip::sequence(),
 				pSession->timeScale(),
-				qtractorMidiClip::format());
+				pSession->tickFromFrame(
+					qtractorMidiClip::clipStart()));
 			// Pre-commit dirty changes...
 			qtractorMidiClip::setFilename(sFilename);
 			qtractorMidiClip::setDirty(false);
@@ -686,28 +734,41 @@ bool qtractorMidiClip::clipExport ( ClipExport pfnClipExport, void *pvArg,
 	if (pSession == NULL)
 		return false;
 
-	iOffset += clipOffset();
 	if (iLength < 1)
 		iLength = clipLength();
 
 	qtractorMidiSequence *pSeq = sequence();
 	unsigned short iTicksPerBeat = pSession->ticksPerBeat();
-	unsigned long iTimeOffset = pSeq->timep(pSeq->timeOffset(), iTicksPerBeat);
-	unsigned long iTimeStart = pSession->tickFromFrame(iOffset);
+	unsigned long iTimeOffset = pSeq->timeOffset();
+
+	qtractorTimeScale::Cursor cursor(pSession->timeScale());
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(clipStart());
+	unsigned long t0 = pNode->tickFromFrame(clipStart());
+
+	unsigned long f1 = clipStart() + clipOffset() + iOffset;
+	pNode = cursor.seekFrame(f1);
+	unsigned long t1 = pNode->tickFromFrame(f1);
+	unsigned long iTimeStart = t1 - t0;
 	iTimeStart = (iTimeStart > iTimeOffset ? iTimeStart - iTimeOffset : 0);
-	unsigned long iTimeEnd = iTimeStart + pSession->tickFromFrame(iLength);
+	pNode = cursor.seekFrame(f1 += iLength);
+	unsigned long iTimeEnd = iTimeStart + pNode->tickFromFrame(f1) - t1;
 
 	qtractorMidiSequence seq(pSeq->name(), pSeq->channel(), iTicksPerBeat);
 
+	seq.setBank(pTrack->midiBank());
+	seq.setProgram(pTrack->midiProgram());
+
 	for (qtractorMidiEvent *pEvent = pSeq->events().first();
 			pEvent; pEvent = pEvent->next()) {
-		unsigned long iTime = seq.timeq(pEvent->time(), pSeq->ticksPerBeat());
-		if (iTime >= iTimeStart &&	iTime < iTimeEnd) {
+		unsigned long iTime = pEvent->time();
+		if (iTime >= iTimeStart && iTime < iTimeEnd) {
 			qtractorMidiEvent *pNewEvent = new qtractorMidiEvent(*pEvent);
 			pNewEvent->setTime(iTime - iTimeStart);
 			if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
 				pNewEvent->setVelocity((unsigned char)
 					(clipGain() * float(pNewEvent->velocity())) & 0x7f);
+				if (iTime + pEvent->duration() > iTimeEnd)
+					pNewEvent->setDuration(iTimeEnd - iTime);
 			}
 			seq.insertEvent(pNewEvent);
 		}
