@@ -1,7 +1,7 @@
 // qtractorMidiConnect.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -431,8 +431,10 @@ void qtractorMidiConnect::disconnectPortsUpdate (
 			} else {
 				qDeleteAll(pBus->outputs());
 				pBus->outputs().clear();
+				// Remember to resend all session/tracks control stuff...
+				pMidiEngine->resetAllControllers(false); // Deferred++
 			}
-			return;
+			break;
 		}
 	}
 }
