@@ -1,7 +1,7 @@
 // qtractorMidiEditEvent.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -315,29 +315,46 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	}
 
 	// Account for the editing offset:
-	int dx = cx + pTimeScale->pixelFromFrame(m_pEditor->offset());
+	qtractorTimeScale::Cursor cursor(pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_pEditor->offset());
+	unsigned long t0 = pNode->tickFromFrame(m_pEditor->offset());
+	int x0 = pTimeScale->pixelFromFrame(m_pEditor->offset());
+	int dx = x0 + cx;
 
 	// Draw vertical grid lines...
-	unsigned short iBeat = pTimeScale->beatFromPixel(dx);
-	unsigned short iTicksPerBeat  = pTimeScale->ticksPerBeat();
-	unsigned short iPixelsPerBeat = pTimeScale->pixelsPerBeat();
-	unsigned long  iTickFromBeat  = iBeat * iTicksPerBeat;
-	int x = pTimeScale->pixelFromTick(iTickFromBeat) - dx;
+	pNode = cursor.seekPixel(dx);
+	unsigned short iSnapPerBeat
+		= (m_pEditor->isSnapGrid() ? pTimeScale->snapPerBeat() : 0);
+	unsigned short iPixelsPerBeat = pNode->pixelsPerBeat();
+	unsigned int iBeat = pNode->beatFromPixel(dx - (iPixelsPerBeat >> 1));
+	int x = pNode->pixelFromBeat(iBeat) - dx;
 	while (x < w) {
 		if (x >= 0) {
-			bool bBeatIsBar = pTimeScale->beatIsBar(iBeat);
+			bool bBeatIsBar = pNode->beatIsBar(iBeat);
 			if (bBeatIsBar) {
 				p.setPen(rgbLight);
 				p.drawLine(x, 0, x, h);
+				if (iBeat == pNode->beat)
+					iPixelsPerBeat = pNode->pixelsPerBeat();
 			}
-			if (bBeatIsBar || iPixelsPerBeat > 16) {
+			if (bBeatIsBar || iPixelsPerBeat > 8) {
 				p.setPen(rgbDark);
 				p.drawLine(x - 1, 0, x - 1, h);
 			}
 		}
-		iTickFromBeat += iTicksPerBeat;
-		x = pTimeScale->pixelFromTick(iTickFromBeat) - dx;
-		iBeat++;
+		if (iSnapPerBeat > 1) {
+			int q = iPixelsPerBeat / iSnapPerBeat;
+			if (q > 4) {  
+				p.setPen(rgbBase.value() < 0x7f
+					? rgbDark.darker(105) : rgbLight.lighter(120));
+				for (int i = 1; i < iSnapPerBeat; ++i) {
+					x = pTimeScale->pixelSnap(x + dx + q) - dx - 1;
+					p.drawLine(x, 0, x, h);
+				}
+			}
+		}
+		pNode = cursor.seekBeat(++iBeat);
+		x = pNode->pixelFromBeat(iBeat) - dx;
 	}
 
 	//
@@ -348,8 +365,10 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	if (pSeq == NULL)
 		return;
 
-	unsigned long iTickStart = pTimeScale->tickFromPixel(cx);
-	unsigned long iTickEnd   = iTickStart + pTimeScale->tickFromPixel(w);
+	pNode = cursor.seekPixel(x = dx);
+	unsigned long iTickStart = pNode->tickFromPixel(x);
+	pNode = cursor.seekPixel(x += w);
+	unsigned long iTickEnd = pNode->tickFromPixel(x);
 
 	// This is the zero-line...
 	int y0 = h;
@@ -369,17 +388,23 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 	rgbValue.getHsv(&hue, &sat, &val); sat = 86;
 
 	bool bController = (m_eventType == qtractorMidiEvent::CONTROLLER);
-	qtractorMidiEvent *pEvent = m_pEditor->seekEvent(iTickStart);
-	while (pEvent && pEvent->time() < iTickEnd) {
-		if (pEvent->type() == m_eventType	// Filter event type!
-			&& (!bController || pEvent->controller() == m_controller)
-			&& pEvent->time() + pEvent->duration() >= iTickStart) {
+	qtractorMidiEvent *pEvent
+		= m_pEditor->seekEvent(iTickStart > t0 ? iTickStart - t0 : 0);
+	while (pEvent) {
+		unsigned long t1 = t0 + pEvent->time();
+		if (t1 >= iTickEnd)
+			break;
+		unsigned long t2 = t1 + pEvent->duration();
+		// Filter event type!...
+		if (pEvent->type() == m_eventType && t2 >= iTickStart
+			&& (!bController || pEvent->controller() == m_controller)) {
 			if (m_eventType == qtractorMidiEvent::PITCHBEND)
 				y = y0 - (y0 * pEvent->pitchBend()) / 8192;
 			else
 				y = y0 - (y0 * pEvent->value()) / 128;
-			x = pTimeScale->pixelFromTick(pEvent->time()) - cx;
-			int w1 = pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+			pNode = cursor.seekTick(t1);
+			x = pNode->pixelFromTick(t1) - x0 - cx;
+			int w1 = pNode->pixelFromTick(t2) - x0 - cx - x;
 			if (w1 < 5 || !m_pEditor->isNoteDuration())
 				w1 = 5;
 			if (m_eventType == qtractorMidiEvent::NOTEON ||
