@@ -1,7 +1,7 @@
 // qtractorMidiEditor.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -288,9 +288,15 @@ qtractorMidiEditor::qtractorMidiEditor ( QWidget *pParent )
 
 	m_pRubberBand = NULL;
 
+	// Zoom mode flag.
+	m_iZoomMode = ZoomAll;
+
 	// Edit mode flag.
 	m_bEditMode = false;
 
+	// Snap-to-beat grid mode.
+	m_bSnapGrid = false;
+
 	// Last default editing values.
 	m_last.note      = 0x3c;	// middle-C
 	m_last.value     = 0x40;
@@ -302,6 +308,11 @@ qtractorMidiEditor::qtractorMidiEditor ( QWidget *pParent )
 
 	// Local time-scale.
 	m_pTimeScale = new qtractorTimeScale();
+
+	// The original clip time-scale length/time.
+	m_iClipLengthTime = 0;
+
+	// The local time-scale offset/length.
 	m_iOffset = 0;
 	m_iLength = 0;
 
@@ -436,6 +447,7 @@ void qtractorMidiEditor::setMidiClip ( qtractorMidiClip *pMidiClip )
 
 	if (m_pMidiClip) {
 		// Now set the editing MIDI sequence alright...
+		setClipLength(m_pMidiClip->clipLength());
 		setOffset(m_pMidiClip->clipStart());
 		setLength(m_pMidiClip->clipLength());
 		// Set its most outstanding properties...
@@ -458,6 +470,7 @@ void qtractorMidiEditor::setMidiClip ( qtractorMidiClip *pMidiClip )
 		// Got clip!
 	} else {
 		// Reset those little things too..
+		setClipLength(0);
 		setOffset(0);
 		setLength(0);
 	}
@@ -519,6 +532,54 @@ const QColor& qtractorMidiEditor::background (void) const
 }
 
 
+// Zoom (view) mode.
+void qtractorMidiEditor::setZoomMode ( int iZoomMode )
+{
+	m_iZoomMode = iZoomMode;
+}
+
+int qtractorMidiEditor::zoomMode (void) const
+{
+	return m_iZoomMode;
+}
+
+
+
+// Zoom ratio accessors.
+void qtractorMidiEditor::setHorizontalZoom ( unsigned short iHorizontalZoom )
+{
+	m_pTimeScale->setHorizontalZoom(iHorizontalZoom);
+	m_pTimeScale->updateScale();
+}
+
+unsigned short qtractorMidiEditor::horizontalZoom (void) const
+{
+	return m_pTimeScale->horizontalZoom();
+}
+
+
+void qtractorMidiEditor::setVerticalZoom ( unsigned short iVerticalZoom )
+{
+	// Hold and try setting new item height...
+	int iZoomStep = int(iVerticalZoom) - int(verticalZoom());
+	int iItemHeight
+		= (iVerticalZoom * qtractorMidiEditList::ItemHeightBase) / 100;
+	if (iItemHeight < qtractorMidiEditList::ItemHeightMax && iZoomStep > 0)
+		iItemHeight++;
+	else
+	if (iItemHeight > qtractorMidiEditList::ItemHeightMin && iZoomStep < 0)
+		iItemHeight--;
+	m_pEditList->setItemHeight(iItemHeight);
+
+	m_pTimeScale->setVerticalZoom(iVerticalZoom);
+}
+
+unsigned short qtractorMidiEditor::verticalZoom (void) const
+{
+	return m_pTimeScale->verticalZoom();
+}
+
+
 // Edit (creational) mode.
 void qtractorMidiEditor::setEditMode ( bool bEditMode )
 {
@@ -529,18 +590,59 @@ void qtractorMidiEditor::setEditMode ( bool bEditMode )
 //	updateContents();
 }
 
-bool qtractorMidiEditor::isEditMode() const
+bool qtractorMidiEditor::isEditMode (void) const
 {
 	return m_bEditMode;
 }
 
 
+// Snap-to-beat grid mode.
+void qtractorMidiEditor::setSnapGrid ( bool bSnapGrid )
+{
+	m_bSnapGrid = bSnapGrid;
+
+//	updateContents();
+}
+
+bool qtractorMidiEditor::isSnapGrid (void) const
+{
+	return m_bSnapGrid;
+}
+
+
 // Local time scale accessor.
 qtractorTimeScale *qtractorMidiEditor::timeScale (void) const
 {
 	return m_pTimeScale;
 }
 
+unsigned long qtractorMidiEditor::timeOffset (void) const
+{
+	return (m_pTimeScale ? m_pTimeScale->tickFromFrame(m_iOffset) : 0);
+}
+
+
+// The original clip time-scale length/time.
+void qtractorMidiEditor::setClipLength ( unsigned long iClipLength )
+{
+	if (m_pTimeScale) {
+		m_iClipLengthTime
+			= m_pTimeScale->tickFromFrame(m_iOffset + iClipLength)
+			- m_pTimeScale->tickFromFrame(m_iOffset);
+	} else {
+		m_iClipLengthTime = 0;
+	}
+}
+
+unsigned long qtractorMidiEditor::clipLength (void) const
+{
+	if (m_pTimeScale == NULL)
+		return 0;
+
+	return m_pTimeScale->frameFromTick(
+		m_pTimeScale->tickFromFrame(m_iOffset) + m_iClipLengthTime) - m_iOffset;
+}
+
 
 // Time-scale offset (in frames) accessors.
 void qtractorMidiEditor::setOffset ( unsigned long iOffset )
@@ -652,6 +754,30 @@ int qtractorMidiEditor::playHeadX (void) const
 }
 
 
+// Update time-scale to master session.
+void qtractorMidiEditor::updateTimeScale (void)
+{
+	if (m_pMidiClip == NULL)
+		return;
+
+	if (m_pTimeScale == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	m_pTimeScale->sync(*pSession->timeScale());
+
+	setOffset(m_pMidiClip->clipStart());
+	setLength(m_pMidiClip->clipLength());
+
+	setPlayHead(pSession->playHead(), false);
+	setEditHead(pSession->editHead(), false);
+	setEditTail(pSession->editTail(), false);
+}
+
+
 // Play-head follow-ness.
 void qtractorMidiEditor::setSyncView ( bool bSyncView )
 {
@@ -793,66 +919,64 @@ QFrame *qtractorMidiEditor::editEventFrame (void) const
 // Horizontal zoom factor.
 void qtractorMidiEditor::horizontalZoomStep ( int iZoomStep )
 {
-	int iHorizontalZoom = m_pTimeScale->horizontalZoom() + iZoomStep;
+	int iHorizontalZoom = horizontalZoom() + iZoomStep;
 	if (iHorizontalZoom < ZoomMin)
 		iHorizontalZoom = ZoomMin;
 	else if (iHorizontalZoom > ZoomMax)
 		iHorizontalZoom = ZoomMax;
-	if (iHorizontalZoom == m_pTimeScale->horizontalZoom())
+	if (iHorizontalZoom == horizontalZoom())
 		return;
 
-	// Fix the local time scale zoom determinant.
-	m_pTimeScale->setHorizontalZoom(iHorizontalZoom);
-	m_pTimeScale->updateScale();
+	// Fix the local horizontal view zoom.
+	setHorizontalZoom(iHorizontalZoom);
 }
 
 
 // Vertical zoom factor.
 void qtractorMidiEditor::verticalZoomStep ( int iZoomStep )
 {
-	int iVerticalZoom = m_pTimeScale->verticalZoom() + iZoomStep;
+	int iVerticalZoom = verticalZoom() + iZoomStep;
 	if (iVerticalZoom < ZoomMin)
 		iVerticalZoom = ZoomMin;
 	else if (iVerticalZoom > ZoomMax)
 		iVerticalZoom = ZoomMax;
-	if (iVerticalZoom == m_pTimeScale->verticalZoom())
+	if (iVerticalZoom == verticalZoom())
 		return;
 
-	// Hold and try setting new item height...
-	int iItemHeight
-		= (iVerticalZoom * qtractorMidiEditList::ItemHeightBase) / 100;
-	if (iItemHeight < qtractorMidiEditList::ItemHeightMax && iZoomStep > 0)
-		iItemHeight++;
-	else
-	if (iItemHeight > qtractorMidiEditList::ItemHeightMin && iZoomStep < 0)
-		iItemHeight--;
-	m_pEditList->setItemHeight(iItemHeight);
-
 	// Fix the local vertical view zoom.
-	m_pTimeScale->setVerticalZoom(iVerticalZoom);
+	setVerticalZoom(iVerticalZoom);
 }
 
 
 // Zoom view slots.
 void qtractorMidiEditor::zoomIn (void)
 {
-	horizontalZoomStep(+ ZoomStep);
-	verticalZoomStep(+ ZoomStep);
+	if (m_iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(+ ZoomStep);
+	if (m_iZoomMode & ZoomVertical)
+		verticalZoomStep(+ ZoomStep);
+
 	centerContents();
 }
 
 void qtractorMidiEditor::zoomOut (void)
 {
-	horizontalZoomStep(- ZoomStep);
-	verticalZoomStep(- ZoomStep);
+	if (m_iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(- ZoomStep);
+	if (m_iZoomMode & ZoomVertical)
+		verticalZoomStep(- ZoomStep);
+
 	centerContents();
 }
 
 
 void qtractorMidiEditor::zoomReset (void)
 {
-	horizontalZoomStep(ZoomBase - m_pTimeScale->horizontalZoom());
-	verticalZoomStep(ZoomBase - m_pTimeScale->verticalZoom());
+	if (m_iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(ZoomBase - m_pTimeScale->horizontalZoom());
+	if (m_iZoomMode & ZoomVertical)
+		verticalZoomStep(ZoomBase - m_pTimeScale->verticalZoom());
+
 	centerContents();
 }
 
@@ -1001,21 +1125,24 @@ void qtractorMidiEditor::copyClipboard (void)
 // (as from current clipboard width)
 unsigned long qtractorMidiEditor::pastePeriod (void) const
 {
-	unsigned long t0 = 0;
+	unsigned long t0 = m_pTimeScale->tickFromFrame(m_iOffset);
 	unsigned long t1 = 0;
+	unsigned long t2 = 0;
 
+	int k = 0;
 	QListIterator<qtractorMidiEvent *> iter(g_clipboard.items);
 	while (iter.hasNext()) {
 		qtractorMidiEvent *pEvent = iter.next();
-		unsigned long t2 = pEvent->time();
-		if (t0 > t2 || t0 == 0)
-			t0 = t2;
-		t2 += pEvent->duration();
-		if (t1 < t2)
-			t1 = t2;
+		unsigned long t = t0 + pEvent->time();
+		if (t1 > t || k == 0)
+			t1 = t;
+		t += pEvent->duration();
+		if (t2 < t)
+			t2 = t;
+		++k;
 	}
 
-	return m_pTimeScale->frameFromTick(t1 - t0);
+	return m_pTimeScale->frameFromTick(t2) - m_pTimeScale->frameFromTick(t1);
 }
 
 
@@ -1042,12 +1169,12 @@ void qtractorMidiEditor::pasteClipboard (
 	// Multi-paste period...
 	if (iPastePeriod < 1)
 		iPastePeriod = pastePeriod();
-		
-	int x0 = 0;
-	int dx = m_pTimeScale->pixelFromFrame(iPastePeriod);
 
-	unsigned long t0 = 0;
-	unsigned long dt = m_pTimeScale->tickFromFrame(iPastePeriod);
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+	int dx = m_pTimeScale->pixelFromFrame(iPastePeriod);
 	
 	// This is the edit-view spacifics...
 	int h1 = m_pEditList->itemHeight();
@@ -1058,22 +1185,28 @@ void qtractorMidiEditor::pasteClipboard (
 	if (m_pEditEvent->eventType() == qtractorMidiEvent::PITCHBEND)
 		y0 = ((y0 >> 3) << 2);
 
+	int k, x1;
+	unsigned long d0 = t0;
 	QListIterator<qtractorMidiEvent *> iter(g_clipboard.items);
 	for (unsigned short i = 0; i < iPasteCount; ++i) {
 		iter.toFront();
+		k = x1 = 0;
 		while (iter.hasNext()) {
 			qtractorMidiEvent *pEvent = iter.next();
 			// Common event coords...
 			int y;
-			int x  = m_pTimeScale->pixelFromTick(pEvent->time()) + x0;
-			int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+			unsigned long t1 = t0 + pEvent->time();
+			unsigned long t2 = t1 + pEvent->duration();
+			pNode = cursor.seekTick(t1);
+			int x  = pNode->pixelFromTick(t1);
+			int w1 = pNode->pixelFromTick(t2) - x;
 			if (w1 < 5)
 				w1 = 5;
 			// View item...
 			QRect rectView;
 			if (pEvent->type() == m_pEditView->eventType()) {
 				y = ch - h1 * (pEvent->note() + 1);
-				rectView.setRect(x, y, w1, h1);
+				rectView.setRect(x - x0, y, w1, h1);
 			}
 			// Event item...
 			QRect rectEvent;
@@ -1085,20 +1218,25 @@ void qtractorMidiEditor::pasteClipboard (
 				if (!m_bNoteDuration)
 					w1 = 5;
 				if (y < y0)
-					rectEvent.setRect(x, y, w1, y0 - y);
+					rectEvent.setRect(x - x0, y, w1, y0 - y);
 				else if (y > y0)
-					rectEvent.setRect(x, y0, w1, y - y0);
+					rectEvent.setRect(x - x0, y0, w1, y - y0);
 				else
-					rectEvent.setRect(x, y0 - 2, w1, 4);
+					rectEvent.setRect(x - x0, y0 - 2, w1, 4);
 			}
-			m_select.addItem(pEvent, rectEvent, rectView, t0);
+			m_select.addItem(pEvent, rectEvent, rectView, t0 - d0);
 			if (m_pEventDrag == NULL) {
 				m_pEventDrag = pEvent;
 				m_rectDrag = (bEditView ? rectView : rectEvent);
 			}
+			if (x1 > x || k == 0)
+				x1 = x;
+			++k;
 		}
-		x0 += dx;
-		t0 += dt;
+		pNode = cursor.seekTick(x1 + dx);
+		t0 += pNode->tickFromPixel(x1 + dx);
+		pNode = cursor.seekTick(x1);
+		t0 -= pNode->tickFromPixel(x1);
 	}
 
 	// We'll start a brand new floating state...
@@ -1110,9 +1248,9 @@ void qtractorMidiEditor::pasteClipboard (
 	m_pEditPaste = pScrollView;
 
 	// It doesn't matter which one, both pasteable views are due...
-	QCursor cursor(QPixmap(":/icons/editPaste.png"), 20, 20);
-	m_pEditView->setCursor(cursor);
-	m_pEditEvent->setCursor(cursor);
+	QCursor cursr(QPixmap(":/icons/editPaste.png"), 20, 20);
+	m_pEditView->setCursor(cursr);
+	m_pEditEvent->setCursor(cursr);
 
 	// Make sure the mouse pointer is properly located...
 	const QPoint& pos = pScrollView->viewportToContents(
@@ -1155,6 +1293,7 @@ void qtractorMidiEditor::selectAll ( bool bSelect, bool bToggle )
 			m_pEditView->contentsHeight());
 		selectRect(rect, bToggle, true);
 	} else {
+		m_select.clear();
 		updateContents();
 	}
 }
@@ -1174,10 +1313,73 @@ void qtractorMidiEditor::selectRect ( const QRect& rect, bool bToggle,
 }
 
 
+// Update all selection rectangular areas.
+void qtractorMidiEditor::updateSelect (void)
+{
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+
+	// This is the edit-view specifics...
+	int h1 = m_pEditList->itemHeight();
+	int ch = m_pEditView->contentsHeight(); // + 1;
+
+	// This is the edit-event zero-line...
+	int y0 = (m_pEditEvent->viewport())->height();
+	if (m_pEditEvent->eventType() == qtractorMidiEvent::PITCHBEND)
+		y0 = ((y0 >> 3) << 2);
+
+	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
+	while (iter.hasNext()) {
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		qtractorMidiEvent *pEvent = pItem->event;
+		// Common event coords...
+		int y;
+		unsigned long t1 = t0 + pEvent->time();
+		unsigned long t2 = t1 + pEvent->duration();
+		pNode = cursor.seekTick(t1);
+		int x  = pNode->pixelFromTick(t1);
+		int w1 = pNode->pixelFromTick(t2) - x;
+		if (w1 < 5)
+			w1 = 5;
+		// View item...
+		if (pEvent->type() == m_pEditView->eventType()) {
+			y = ch - h1 * (pEvent->note() + 1);
+			pItem->rectView.setRect(x - x0, y, w1, h1);
+		} else {
+			pItem->rectView.setRect(0, 0, 0, 0);
+		}
+		// Event item...
+		if (pEvent->type() == m_pEditEvent->eventType()) {
+			if (pEvent->type() == qtractorMidiEvent::PITCHBEND)
+				y = y0 - (y0 * pEvent->pitchBend()) / 8192;
+			else
+				y = y0 - (y0 * pEvent->value()) / 128;
+			if (!m_bNoteDuration)
+				w1 = 5;
+			if (y < y0)
+				pItem->rectEvent.setRect(x - x0, y, w1, y0 - y);
+			else if (y > y0)
+				pItem->rectEvent.setRect(x - x0, y0, w1, y - y0);
+			else
+				pItem->rectEvent.setRect(x - x0, y0 - 2, w1, 4);
+		} else {
+			pItem->rectEvent.setRect(0, 0, 0, 0);
+		}
+	}
+
+	// Final touch.
+	m_select.commit();
+	m_pEventDrag = NULL;
+}
+
+
 // Update/sync integral contents.
 void qtractorMidiEditor::updateContents (void)
 {
-	m_select.clear();
+//	m_select.clear();
+	updateSelect();
 
 	// Update dependant views.
 	m_pEditList->updateContentsHeight();
@@ -1194,7 +1396,8 @@ void qtractorMidiEditor::updateContents (void)
 // Try to center vertically the edit-view...
 void qtractorMidiEditor::centerContents (void)
 {
-	m_select.clear();
+//	m_select.clear();
+	updateSelect();
 
 	// Update dependant views.
 	m_pEditList->updateContentsHeight();
@@ -1225,6 +1428,23 @@ void qtractorMidiEditor::centerContents (void)
 }
 
 
+// Reset event cursors.
+void qtractorMidiEditor::reset (void)
+{
+	// Force reset of all current selection first...
+	m_select.clear();
+
+	// Reset some internal state...
+	if (m_pMidiClip) {
+		qtractorMidiSequence *pSeq = m_pMidiClip->sequence();
+		if (pSeq) {
+			m_cursor.reset(pSeq);
+			m_cursorAt.reset(pSeq);
+		}
+	}
+}
+
+
 // Intra-clip tick/time positioning reset.
 qtractorMidiEvent *qtractorMidiEditor::seekEvent ( unsigned long iTime )
 {
@@ -1247,7 +1467,14 @@ qtractorMidiEvent *qtractorMidiEditor::eventAt (
 	bool bEditView
 		= (static_cast<qtractorScrollView *> (m_pEditView) == pScrollView);
 
-	unsigned long iTime = m_pTimeScale->tickFromPixel(pos.x());
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+
+	pNode = cursor.seekPixel(x0 + pos.x());
+	unsigned long iTime = pNode->tickFromPixel(x0 + pos.x());
+	iTime = (iTime > t0 ? iTime - t0 : 0);
 
 	// This is the edit-view spacifics...
 	int h1 = m_pEditList->itemHeight();
@@ -1263,15 +1490,6 @@ qtractorMidiEvent *qtractorMidiEditor::eventAt (
 	unsigned char controller = m_pEditEvent->controller();
 
 	qtractorMidiEvent *pEvent = m_cursorAt.reset(pSeq, iTime);
-#if 0
-	if (pEvent && pEvent->prev()) {
-		unsigned long iPrevTime = (pEvent->prev())->time();
-		while (pEvent && pEvent->time() >= iPrevTime)
-			pEvent = pEvent->prev();
-		if (pEvent == NULL)
-			pEvent = pSeq->events().first();
-	}
-#endif
 	qtractorMidiEvent *pEventAt = NULL;
 	while (pEvent && iTime >= pEvent->time()) {
 		if (((bEditView && pEvent->type() == m_pEditView->eventType()) ||
@@ -1279,8 +1497,11 @@ qtractorMidiEvent *qtractorMidiEditor::eventAt (
 				(!bController || pEvent->controller() == controller))))) {
 			// Common event coords...
 			int y;
-			int x  = m_pTimeScale->pixelFromTick(pEvent->time());
-			int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+			unsigned long t1 = t0 + pEvent->time();
+			unsigned long t2 = t1 + pEvent->duration();
+			pNode = cursor.seekTick(t1);
+			int x  = pNode->pixelFromTick(t1) - x0;
+			int w1 = pNode->pixelFromTick(t2) - x0 - x;
 			if (w1 < 5)
 				w1 = 5;
 			QRect rect;
@@ -1338,14 +1559,20 @@ qtractorMidiEvent *qtractorMidiEditor::dragEditEvent (
 	int y0 = (m_pEditEvent->viewport())->height();
 	int h1 = m_pEditList->itemHeight();
 
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);	
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
 	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
-	int x1 = m_pTimeScale->pixelSnap(pos.x() + x0) - x0;
+	
+	int x1 = x0 + pos.x();
 	int y1 = 0;
 
 	// This will be the new editing event...
-	qtractorMidiEvent *pEvent = new qtractorMidiEvent(
-		m_pTimeScale->tickFromPixel(x1),
+	pNode = cursor.seekPixel(x1);
+	unsigned long t1 = pNode->tickSnap(pNode->tickFromPixel(x1));
+	qtractorMidiEvent *pEvent = new qtractorMidiEvent(t1 - t0,
 		bEditView ? m_pEditView->eventType() : m_pEditEvent->eventType());
+	x1 = pNode->pixelFromTick(t1);
 
 	switch (pEvent->type()) {
 	case qtractorMidiEvent::NOTEON:
@@ -1394,7 +1621,8 @@ qtractorMidiEvent *qtractorMidiEditor::dragEditEvent (
 	}
 
 	// Now try to get the visual rectangular coordinates...
-	int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+	int w1 = pNode->pixelFromTick(
+		t0 + pEvent->time() + pEvent->duration()) - x1;
 	if (w1 < 5)
 		w1 = 5;
 
@@ -1404,7 +1632,7 @@ qtractorMidiEvent *qtractorMidiEditor::dragEditEvent (
 		(pEvent->type() == qtractorMidiEvent::NOTEON ||
 			pEvent->type() == qtractorMidiEvent::KEYPRESS)) {
 		y1 = ch - h1 * (pEvent->note() + 1);
-		rectView.setRect(x1, y1, w1, h1);
+		rectView.setRect(x1 - x0, y1, w1, h1);
 	}
 
 	// Event item...
@@ -1427,7 +1655,7 @@ qtractorMidiEvent *qtractorMidiEditor::dragEditEvent (
 			w1 = 5;
 		if (h1 < 3)
 			h1 = 3;
-		rectEvent.setRect(x1, y1, w1, h1);
+		rectEvent.setRect(x1 - x0, y1, w1, h1);
 	}
 
 	// Set the correct target rectangle...
@@ -1563,11 +1791,9 @@ void qtractorMidiEditor::dragMoveUpdate ( qtractorScrollView *pScrollView,
 		if ((pos - m_posDrag).manhattanLength()
 			< QApplication::startDragDistance())
 			break;
-	#if 0
 		// Take care of selection modifier...
 		if ((modifiers & (Qt::ShiftModifier | Qt::ControlModifier)) == 0)
 			flags |= SelectClear;
-	#endif
 		// Are we about to move something around?
 		if (m_pEventDrag) {
 			if (m_resizeMode == ResizeNone) {
@@ -1576,7 +1802,7 @@ void qtractorMidiEditor::dragMoveUpdate ( qtractorScrollView *pScrollView,
 					= m_select.findItem(m_pEventDrag);
 				if	(pItem == NULL || (pItem->flags & 1) == 0) {
 					updateDragSelect(pScrollView,
-						QRect(m_posDrag, QSize()), flags | SelectCommit);
+						QRect(m_posDrag, QSize(1, 1)), flags | SelectCommit);
 				}
 				// Start drag-moving...
 				m_dragState = DragMove;
@@ -1585,7 +1811,7 @@ void qtractorMidiEditor::dragMoveUpdate ( qtractorScrollView *pScrollView,
 				// Start resizing... take care of yet initial selection...
 				if (!m_bEventDragEdit) {
 					updateDragSelect(pScrollView,
-						QRect(m_posDrag, QSize()), flags | SelectCommit);
+						QRect(m_posDrag, QSize(1, 1)), flags | SelectCommit);
 				}
 				// Start drag-resizing...
 				m_dragState = DragResize;
@@ -1754,6 +1980,11 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 	if (flags & SelectClear)
 		m_select.clear();
 
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);	
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+
 	int x1, x2;
 	if (bRectSelect) {
 		x1 = pScrollView->contentsX();
@@ -1766,8 +1997,13 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 		x1 = x2 = rectSelect.x();
 	}
 
-	unsigned long iTickStart = m_pTimeScale->tickFromPixel(x1);
-	unsigned long iTickEnd   = m_pTimeScale->tickFromPixel(x2);
+	pNode = cursor.seekPixel(x0 + x1);
+	unsigned long t1 = pNode->tickFromPixel(x0 + x1);
+	unsigned long iTickStart = (t1 > t0 ? t1 - t0 : 0);
+
+	pNode = cursor.seekPixel(x0 + x2);
+	unsigned long t2 = pNode->tickFromPixel(x0 + x2);
+	unsigned long iTickEnd = (t2 > t0 ? t2 - t0 : 0);
 
 	// This is the edit-view spacifics...
 	int h1 = m_pEditList->itemHeight();
@@ -1797,8 +2033,11 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 			bool bSelect = false;
 			// Common event coords...
 			int y;
-			int x  = m_pTimeScale->pixelFromTick(pEvent->time());
-			int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+			t1 = t0 + pEvent->time();
+			t2 = t1 + pEvent->duration();
+			pNode = cursor.seekTick(t1);
+			int x  = pNode->pixelFromTick(t1) - x0;
+			int w1 = pNode->pixelFromTick(t2) - x0 - x;
 			if (w1 < 5)
 				w1 = 5;
 			// View item...
@@ -1861,12 +2100,45 @@ void qtractorMidiEditor::updateDragSelect ( qtractorScrollView *pScrollView,
 		m_pEditEvent->contentsToViewport(rectUpdateEvent.topLeft()),
 		rectUpdateEvent.size()));
 
+#if 0
 	if (bEditView) {
 		setEditHead(m_pTimeScale->frameSnap(m_iOffset
 			+ m_pTimeScale->frameFromPixel(rectSelect.left())), bCommit);
 		setEditTail(m_pTimeScale->frameSnap(m_iOffset
 			+ m_pTimeScale->frameFromPixel(rectSelect.right())), bCommit);
 	}
+#endif
+}
+
+
+// Compute curent drag time delta (in ticks).
+long qtractorMidiEditor::timeDelta ( qtractorScrollView *pScrollView )
+{
+	qtractorTimeScale::Cursor cursor(m_pTimeScale);
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(m_iOffset);	
+	unsigned long t0 = pNode->tickFromFrame(m_iOffset);
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+
+	int x1, x2;
+	unsigned long t1, t2;
+
+	if (m_pEventDrag) {
+		t1 = t0 + m_pEventDrag->time();
+		pNode = cursor.seekTick(t1);
+		x1 = pNode->pixelFromTick(t1);
+	} else {
+		x1 = x0 + (
+			static_cast<qtractorScrollView *> (m_pEditView) == pScrollView
+			? m_select.rectView().x() : m_select.rectEvent().x());
+		pNode = cursor.seekPixel(x1);
+		t1 = pNode->tickFromPixel(x1);
+	}
+
+	x2 = x1 + m_posDelta.x();
+	pNode = cursor.seekPixel(x2);
+	t2 = pNode->tickFromPixel(x2);
+
+	return long(pNode->tickSnap(t2)) - long(t1);
 }
 
 
@@ -1982,7 +2254,7 @@ void qtractorMidiEditor::updateDragResize ( qtractorScrollView *pScrollView,
 
 	m_posDelta.setX(dx);
 	m_posDelta.setY(dy);
-	
+
 	rectUpdateView = rectUpdateView.unite(
 		m_select.rectView().translated(m_posDelta.x(), 0));
 	m_pEditView->viewport()->update(QRect(
@@ -2006,23 +2278,7 @@ void qtractorMidiEditor::executeDragMove ( qtractorScrollView *pScrollView,
 
 	updateDragMove(pScrollView, pos + m_posStep);
 
-	long iTimeDelta, t2, t1;
-	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
-	if (m_pEventDrag) {
-		if (m_posDelta.x() < 0)
-			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-		else
-			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
-		t1 = long(m_pEventDrag->time());
-		t2 = t1 + iTimeDelta;
-	} else {
-		int x1 = (
-			static_cast<qtractorScrollView *> (m_pEditView) == pScrollView
-			? m_select.rectView().x() : m_select.rectEvent().x());
-		t1 = long(m_pTimeScale->tickFromPixel(x1));
-		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
-	}
-	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
+	long iTimeDelta = timeDelta(pScrollView);
 
 	int h1 = m_pEditList->itemHeight();
 	int iNoteDelta = 0;
@@ -2062,24 +2318,7 @@ void qtractorMidiEditor::executeDragResize ( qtractorScrollView *pScrollView,
 
 	updateDragResize(pScrollView, pos);
 
-	long iTimeDelta, t2, t1;
-	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
-	if (m_pEventDrag) {
-		if (m_posDelta.x() < 0)
-			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-		else
-			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
-		t1 = long(m_pEventDrag->time());
-		if (m_resizeMode == ResizeNoteRight)
-			t1 += long(m_pEventDrag->duration());
-		t2 = t1 + iTimeDelta;
-	} else {
-		int x1 = (m_resizeMode == ResizeNoteRight
-			? m_rectDrag.right() : m_rectDrag.left());
-		t1 = long(m_pTimeScale->tickFromPixel(x1));
-		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
-	}
-	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
+	long iTimeDelta = timeDelta(pScrollView);
 
 	int h = (m_pEditEvent->viewport())->height();
 	int iValueDelta = 0;
@@ -2188,23 +2427,7 @@ void qtractorMidiEditor::executeDragPaste ( qtractorScrollView *pScrollView,
 
 	updateDragMove(pScrollView, pos + m_posStep);
 
-	long iTimeDelta, t2, t1;
-	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
-	if (m_pEventDrag) {
-		if (m_posDelta.x() < 0)
-			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-		else
-			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
-		t1 = long(m_pEventDrag->time());
-		t2 = t1 + iTimeDelta;
-	} else {
-		int x1 = (
-			static_cast<qtractorScrollView *> (m_pEditView) == pScrollView
-			? m_select.rectView().x() : m_select.rectEvent().x());
-		t1 = long(m_pTimeScale->tickFromPixel(x1));
-		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
-	}
-	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
+	long iTimeDelta = timeDelta(pScrollView);
 
 	int h1 = m_pEditList->itemHeight();
 	int iNoteDelta = 0;
@@ -2357,6 +2580,7 @@ void qtractorMidiEditor::resetDragState ( qtractorScrollView *pScrollView )
 			m_dragState == DragResize ||
 			m_dragState == DragPaste  ||
 			m_dragState == DragStep)
+			m_select.clear();
 			updateContents();
 	}
 
@@ -2389,6 +2613,7 @@ bool qtractorMidiEditor::adjustEditCommand (
 	while (pEvent) {
 		unsigned long iTime = pEvent->time();
 		unsigned long iTimeEnd = iTime + pEvent->duration();
+		qtractorMidiEvent *pNextEvent = pEvent->next();
 		// NOTEON: Find previous note event and check overlaps...
 		if (pEvent->type() == qtractorMidiEvent::NOTEON) {
 			// Already there?
@@ -2398,8 +2623,9 @@ bool qtractorMidiEditor::adjustEditCommand (
 				qtractorMidiEvent *pPrevEvent = *iter;
 				unsigned long iPrevTime = pPrevEvent->time();
 				unsigned long iPrevTimeEnd = iPrevTime + pPrevEvent->duration();
-				// Left-side event...
+				// Inner operlap...
 				if (iTime > iPrevTime && iTime < iPrevTimeEnd) {
+					// Left-side outer event...
 					unsigned long iDuration = pPrevEvent->duration();
 					pPrevEvent->setDuration(iTime - iPrevTime);
 					if (!pEditCommand->findEvent(pPrevEvent,
@@ -2407,7 +2633,7 @@ bool qtractorMidiEditor::adjustEditCommand (
 						pEditCommand->resizeEventTime(
 							pPrevEvent, iPrevTime, iDuration);
 					}
-					// Right-side event...
+					// Right-side outer event...
 					if (iTimeEnd < iPrevTimeEnd) {
 						qtractorMidiEvent *pNewEvent
 							= new qtractorMidiEvent(*pPrevEvent);
@@ -2419,18 +2645,57 @@ bool qtractorMidiEditor::adjustEditCommand (
 				}
 				else
 				// Loose overlap?...
-				if (iTime == iPrevTime || iTimeEnd == iPrevTimeEnd) {
-					pSeq->unlinkEvent(pPrevEvent);
-					if (!pEditCommand->findEvent(pPrevEvent,
-							qtractorMidiEditCommand::RemoveEvent))
-						pEditCommand->removeEvent(pPrevEvent);
+				if (iTime == iPrevTime) {
+					// Exact overlap...
+					if (iTimeEnd == iPrevTimeEnd) {
+						pSeq->unlinkEvent(pPrevEvent);
+						if (!pEditCommand->findEvent(pPrevEvent,
+								qtractorMidiEditCommand::RemoveEvent))
+							pEditCommand->removeEvent(pPrevEvent);
+					} else {
+						// Partial overlap...
+						if (iTimeEnd < iPrevTimeEnd) {
+							// Short over large...
+							unsigned long iDuration = pPrevEvent->duration();
+							pPrevEvent->setDuration(pEvent->duration());
+							if (!pEditCommand->findEvent(pPrevEvent,
+									qtractorMidiEditCommand::ResizeEventTime)) {
+								pEditCommand->resizeEventTime(
+									pPrevEvent, iPrevTime, iDuration);
+							}
+							iDuration = pEvent->duration();
+							pSeq->unlinkEvent(pEvent);
+							pEvent->setTime(iTimeEnd);
+							pEvent->setDuration(iPrevTimeEnd - iTimeEnd);
+							pSeq->insertEvent(pEvent);
+							if (!pEditCommand->findEvent(pEvent,
+									qtractorMidiEditCommand::ResizeEventTime)) {
+								pEditCommand->resizeEventTime(
+									pEvent, iTime, iDuration);
+							}
+						} else {
+							// Large over short...
+							unsigned long iDuration = pEvent->duration();
+							pSeq->unlinkEvent(pEvent);
+							pEvent->setTime(iPrevTimeEnd);
+							pEvent->setDuration(iTimeEnd - iPrevTimeEnd);
+							pSeq->insertEvent(pEvent);
+							if (!pEditCommand->findEvent(pEvent,
+									qtractorMidiEditCommand::ResizeEventTime)) {
+								pEditCommand->resizeEventTime(
+									pEvent, iTime, iDuration);
+							}
+						}
+						// We've move it ahead...
+						pEvent = pPrevEvent;
+					}					
 				}
 			}
 			// Set as last note...
 			notes[note] = pEvent;
 		}
 		// Iterate next...
-		pEvent = pEvent->next();
+		pEvent = pNextEvent;
 	}
 
 	return true;
@@ -2581,9 +2846,9 @@ void qtractorMidiEditor::sendNote ( int iNote, int iVelocity )
 // MIDI event tool tip helper.
 QString qtractorMidiEditor::eventToolTip ( qtractorMidiEvent *pEvent ) const
 {
+	unsigned long t0 = m_pTimeScale->tickFromFrame(m_iOffset);
 	QString sToolTip = tr("Time:\t%1\nType:\t")
-		.arg(m_pTimeScale->textFromTick(pEvent->time()
-			+ m_pTimeScale->tickFromFrame(m_iOffset)));
+		.arg(m_pTimeScale->textFromTick(t0 + pEvent->time()));
 
 	switch (pEvent->type()) {
 //	case qtractorMidiEvent::NOTEOFF:
@@ -2594,7 +2859,7 @@ QString qtractorMidiEditor::eventToolTip ( qtractorMidiEvent *pEvent ) const
 			.arg(int(pEvent->note()))
 			.arg(noteName(pEvent->note()))
 			.arg(int(pEvent->velocity()))
-			.arg(m_pTimeScale->textFromTick(pEvent->duration(), true));
+			.arg(m_pTimeScale->textFromTick(t0, true, pEvent->duration()));
 		break;
 	case qtractorMidiEvent::KEYPRESS:
 		sToolTip += tr("Key Press (%1) %2\nValue:\t%3")
@@ -2650,7 +2915,7 @@ bool qtractorMidiEditor::keyPress ( qtractorScrollView *pScrollView,
 	case Qt::Key_Insert: // Aha, joking :)
 	case Qt::Key_Return:
 		if (m_dragState == DragStep) {
-			executeDragMove(pScrollView, m_posDrag);
+			executeDragMove(m_pEditView, m_posDrag);
 		} else {
 			const QPoint& pos = pScrollView->viewportToContents(
 				pScrollView->viewport()->mapFromGlobal(QCursor::pos()));
@@ -2781,32 +3046,34 @@ bool qtractorMidiEditor::keyStep ( int iKey )
 		m_dragState != DragPaste)
 		return false;
 
-	int iVerticalStep = m_pEditList->itemHeight();
-	unsigned short iSnapPerBeat = m_pTimeScale->snapPerBeat();
-	if (iSnapPerBeat < 1)
-		iSnapPerBeat = 1;
-	int iHorizontalStep
-		= (m_pTimeScale->horizontalZoom() * m_pTimeScale->pixelsPerBeat())
-			/ (iSnapPerBeat * 100);
-
-	// Now determine which step...
-	switch (iKey) {
-	case Qt::Key_Left:
-		m_posStep.setX(m_posStep.x() - iHorizontalStep);
-		break;
-	case Qt::Key_Right:
-		m_posStep.setX(m_posStep.x() + iHorizontalStep);
-		break;
-	case Qt::Key_Up:
-		m_posStep.setY(m_posStep.y() - iVerticalStep);
-		break;
-	case Qt::Key_Down:
-		m_posStep.setY(m_posStep.y() + iVerticalStep);
-		break;
-	default:
-		return false;
+	// Determine vertical step...
+	if (iKey == Qt::Key_Up || iKey == Qt::Key_Down)  {
+		int iVerticalStep = m_pEditList->itemHeight();
+		int y0 = m_posDrag.y();
+		int y1 = y0 + m_posStep.y();
+		if (iKey == Qt::Key_Up)
+			y1 -= iVerticalStep;
+		else
+			y1 += iVerticalStep;
+		m_posStep.setY((y1 < 0 ? 0 : y1) - y0);
+	}
+	else
+	// Determine horizontal step...
+	if (iKey == Qt::Key_Left || iKey == Qt::Key_Right)  {
+		unsigned short iSnapPerBeat = m_pTimeScale->snapPerBeat();
+		if (iSnapPerBeat < 1)
+			iSnapPerBeat = 1;
+		int x0 = m_posDrag.x() + m_pTimeScale->pixelFromFrame(m_iOffset);
+		int x1 = x0 + m_posStep.x();
+		qtractorTimeScale::Cursor cursor(m_pTimeScale);
+		qtractorTimeScale::Node *pNode = cursor.seekPixel(x1);
+		int iHorizontalStep = pNode->pixelsPerBeat() / iSnapPerBeat;
+		if (iKey == Qt::Key_Left)
+			x1 -= iHorizontalStep;
+		else
+			x1 += iHorizontalStep;
+		m_posStep.setX(m_pTimeScale->pixelSnap(x1 < 0 ? 0 : x1) - x0);
 	}
-
 
 	// Early sanity check...
 	const QRect& rect = m_select.rectView();
