@@ -1,7 +1,7 @@
 // qtractorMidiEditor.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -87,12 +87,34 @@ public:
 	void setBackground(const QColor& back);
 	const QColor& background() const;
 
+	// Snap-to-beat grid mode.
+	void setSnapGrid(bool bSnapGrid);
+	bool isSnapGrid() const;
+
 	// Edit (creational) mode.
 	void setEditMode(bool bEditMode);
 	bool isEditMode() const;
 
+	// Zoom (view) modes.
+	enum { ZoomNone = 0, ZoomHorizontal = 1, ZoomVertical = 2, ZoomAll = 3 };
+
+	void setZoomMode(int iZoomMode);
+	int zoomMode() const;
+
+	// Zoom ratio accessors.
+	void setHorizontalZoom(unsigned short iHorizontalZoom);
+	unsigned short horizontalZoom() const;
+
+	void setVerticalZoom(unsigned short iVerticalZoom);
+	unsigned short verticalZoom() const;
+
 	// Local time scale accessors.
 	qtractorTimeScale *timeScale() const;
+	unsigned long timeOffset() const;
+
+	// The original clip time-scale length/time.
+	void setClipLength(unsigned long iClipLength);
+	unsigned long clipLength() const;
 
 	// Time-scale offset (in frames) accessors.
 	void setOffset(unsigned long iOffset);
@@ -125,6 +147,9 @@ public:
 	unsigned long playHead() const;
 	int playHeadX() const;
 
+	// Update time-scale to master session.
+	void updateTimeScale();
+
 	// Play-head follow-ness.
 	void setSyncView(bool bSyncView);
 	bool isSyncView() const;
@@ -189,6 +214,9 @@ public:
 	// Try to center vertically the edit-view...
 	void centerContents();
 
+	// Reset event cursors.
+	void reset();
+
 	// To optimize and keep track of current (re)draw
 	// position, mostly like an sequence cursor/iterator.
 	qtractorMidiEvent *seekEvent(unsigned long iTime);
@@ -306,10 +334,16 @@ protected:
 		SelectCommit = 4
 	};
 
+	// Update all selection rectangular areas.
+	void updateSelect();
+
 	// Update the event selection list.
 	void updateDragSelect(qtractorScrollView *pScrollView,
 		const QRect& rectSelect, int flags);
 
+	// Compute curent drag time delta (in ticks).
+	long timeDelta(qtractorScrollView *pScrollView);
+
 	// Drag-move current selection.
 	void updateDragMove(qtractorScrollView *pScrollView, const QPoint& pos);
 	// Finalize the event drag-move.
@@ -374,6 +408,9 @@ private:
 	// The local time scale.
 	qtractorTimeScale *m_pTimeScale;
 
+	// The original clip time-scale length/time.
+	unsigned long m_iClipLengthTime;
+
 	// The local time-scale offset/length.
 	unsigned long m_iOffset;
 	unsigned long m_iLength;
@@ -414,10 +451,16 @@ private:
 	// Viewport rubber-banding stuff.
 	qtractorRubberBand *m_pRubberBand;
 
+	// Zoom mode flag.
+	int m_iZoomMode;
+
 	// Edit mode flag.
 	bool m_bEditMode;
 	bool m_bEventDragEdit;
 
+	// Snap-to-beat grid mode.
+	bool m_bSnapGrid;
+
 	// Last useful editing values.
 	struct {
 		unsigned char  note;
