@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -61,7 +61,6 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	// Setup UI struct...
 	m_ui.setupUi(this);
 
-	m_iClipLength = 0;
 	m_iDirtyCount = 0;
 
 	// Set our central widget.
@@ -296,6 +295,18 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.viewZoomResetAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewZoomReset()));
+	QObject::connect(m_ui.viewZoomHorizontalAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewZoomHorizontal()));
+	QObject::connect(m_ui.viewZoomVerticalAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewZoomVertical()));
+	QObject::connect(m_ui.viewZoomAllAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewZoomAll()));
+	QObject::connect(m_ui.viewSnapGridAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewSnapGrid(bool)));
 	QObject::connect(m_ui.viewRefreshAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewRefresh()));
@@ -310,6 +321,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
+	QObject::connect(m_ui.viewZoomMenu,
+		SIGNAL(aboutToShow()),
+		SLOT(updateZoomMenu()));
 	QObject::connect(m_ui.viewSnapMenu,
 		SIGNAL(aboutToShow()),
 		SLOT(updateSnapMenu()));
@@ -351,6 +365,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		m_ui.viewValueColorAction->setChecked(pOptions->bMidiValueColor);
 		m_ui.viewPreviewAction->setChecked(pOptions->bMidiPreview);
 		m_ui.viewFollowAction->setChecked(pOptions->bMidiFollow);
+		m_ui.viewSnapGridAction->setChecked(pOptions->bMidiSnapGrid);
 		if (pOptions->bMidiEditMode)
 			m_ui.editModeOnAction->setChecked(true);
 		else
@@ -362,6 +377,10 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		viewToolbarEdit(pOptions->bMidiEditToolbar);
 		viewToolbarView(pOptions->bMidiViewToolbar);
 		viewToolbarTransport(pOptions->bMidiTransportToolbar);
+		m_pMidiEditor->setZoomMode(pOptions->iMidiZoomMode);
+		m_pMidiEditor->setHorizontalZoom(pOptions->iMidiHorizontalZoom);
+		m_pMidiEditor->setVerticalZoom(pOptions->iMidiVerticalZoom);
+		m_pMidiEditor->setSnapGrid(pOptions->bMidiSnapGrid);
 		m_pMidiEditor->setEditMode(pOptions->bMidiEditMode);
 		m_pMidiEditor->setNoteColor(pOptions->bMidiNoteColor);
 		m_pMidiEditor->setValueColor(pOptions->bMidiValueColor);
@@ -462,11 +481,13 @@ bool qtractorMidiEditorForm::queryClose (void)
 			"\"%1\"\n\n"
 			"Do you want to save the changes?")
 			.arg(filename()),
-			tr("Save"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Save...
+			QMessageBox::Save |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Save:
 			bQueryClose = saveClipFile(false);
 			// Fall thru....
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bQueryClose = false;
@@ -485,7 +506,11 @@ bool qtractorMidiEditorForm::queryClose (void)
 			pOptions->bMidiEditToolbar = m_ui.editToolbar->isVisible();
 			pOptions->bMidiViewToolbar = m_ui.viewToolbar->isVisible();
 			pOptions->bMidiTransportToolbar = m_ui.transportToolbar->isVisible();
-			pOptions->bMidiEditMode = m_ui.editModeOnAction->isChecked();
+			pOptions->iMidiZoomMode = m_pMidiEditor->zoomMode();
+			pOptions->iMidiHorizontalZoom = m_pMidiEditor->horizontalZoom();
+			pOptions->iMidiVerticalZoom = m_pMidiEditor->verticalZoom();
+			pOptions->bMidiSnapGrid = m_pMidiEditor->isSnapGrid();
+			pOptions->bMidiEditMode = m_pMidiEditor->isEditMode();
 			pOptions->bMidiNoteDuration = m_ui.viewNoteDurationAction->isChecked();
 			pOptions->bMidiNoteColor = m_ui.viewNoteColorAction->isChecked();
 			pOptions->bMidiValueColor = m_ui.viewValueColorAction->isChecked();
@@ -540,7 +565,7 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
 		if (pMidiClip && pMidiClip->isDirty()) {
 			 // Revert to original clip length...
-			pMidiClip->setClipLength(m_iClipLength);
+			pMidiClip->setClipLength(m_pMidiEditor->clipLength());
 			pMidiClip->openMidiFile(
 				pMidiClip->filename(),
 				pMidiClip->trackChannel());
@@ -583,13 +608,17 @@ qtractorTimeScale *qtractorMidiEditorForm::timeScale (void) const
 	return m_pMidiEditor->timeScale();
 }
 
+unsigned long qtractorMidiEditorForm::timeOffset (void) const
+{
+	return m_pMidiEditor->timeOffset();
+}
+
 
 // Editing MIDI clip accessors.
 void qtractorMidiEditorForm::setMidiClip ( qtractorMidiClip *pMidiClip  )
 {
 	if (queryClose()) {
 		m_pMidiEditor->setMidiClip(pMidiClip);
-		m_iClipLength = pMidiClip->clipLength();
 		m_iDirtyCount = 0;
 	}
 }
@@ -636,9 +665,14 @@ void qtractorMidiEditorForm::setup ( qtractorMidiClip *pMidiClip )
 	if (pMainForm == NULL)
 		return;
 
-	// Get those time-scales in sync...
-	qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
+	// Get those time-scales in sync,
+	// while keeping zoom ratios persistant...
+	unsigned short iHorizontalZoom = m_pMidiEditor->horizontalZoom();
+	unsigned short iVerticalZoom   = m_pMidiEditor->verticalZoom();
+	qtractorTimeScale *pTimeScale  = m_pMidiEditor->timeScale();
 	pTimeScale->copy(*pSession->timeScale());
+	m_pMidiEditor->setHorizontalZoom(iHorizontalZoom);
+	m_pMidiEditor->setVerticalZoom(iVerticalZoom);
 
 	// Default snap-per-beat setting...
 	m_pSnapPerBeatComboBox->setCurrentIndex(
@@ -741,13 +775,14 @@ bool qtractorMidiEditorForm::saveClipFile ( bool bPrompt )
 		if (sFilename.isEmpty())
 			return false;
 		// Enforce .mid extension...
-		if (QFileInfo(sFilename).suffix().isEmpty())
+		if (QFileInfo(sFilename).suffix() != sExt)
 			sFilename += '.' + sExt;
 	}
 
 	// Save it right away...
 	bool bResult = qtractorMidiFile::saveCopyFile(sFilename,
-		filename(), trackChannel(), sequence(), timeScale(), format());
+		filename(), trackChannel(), format(), sequence(),
+		timeScale(), timeOffset());
 
 	// Have we done it right?
 	if (bResult) {
@@ -1104,27 +1139,56 @@ void qtractorMidiEditorForm::viewFollow ( bool bOn )
 }
 
 
-// Horizontal and vertical zoom-in.
+// Horizontal and/or vertical zoom-in.
 void qtractorMidiEditorForm::viewZoomIn (void)
 {
 	m_pMidiEditor->zoomIn();
 }
 
 
-// Horizontal and vertical zoom-out.
+// Horizontal and/or vertical zoom-out.
 void qtractorMidiEditorForm::viewZoomOut (void)
 {
 	m_pMidiEditor->zoomOut();
 }
 
 
-// Reset both zoom levels to default.
+// Reset zoom level to default.
 void qtractorMidiEditorForm::viewZoomReset (void)
 {
 	m_pMidiEditor->zoomReset();
 }
 
 
+// Set horizontal zoom mode
+void qtractorMidiEditorForm::viewZoomHorizontal (void)
+{
+	m_pMidiEditor->setZoomMode(qtractorMidiEditor::ZoomHorizontal);
+}
+
+
+// Set vertical zoom mode
+void qtractorMidiEditorForm::viewZoomVertical (void)
+{
+	m_pMidiEditor->setZoomMode(qtractorMidiEditor::ZoomVertical);
+}
+
+
+// Set all zoom mode
+void qtractorMidiEditorForm::viewZoomAll (void)
+{
+	m_pMidiEditor->setZoomMode(qtractorMidiEditor::ZoomAll);
+}
+
+
+// Set grid mode
+void qtractorMidiEditorForm::viewSnapGrid ( bool bOn )
+{
+	m_pMidiEditor->setSnapGrid(bOn);
+	m_pMidiEditor->updateContents();
+}
+
+
 // Change snap-per-beat setting via menu.
 void qtractorMidiEditorForm::viewSnap (void)
 {
@@ -1280,8 +1344,9 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	else
 		m_pStatusModLabel->clear();
 
-	m_pDurationLabel->setText(
-		(m_pMidiEditor->timeScale())->textFromTick(pSeq->duration(), true));
+	qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
+	m_pDurationLabel->setText(pTimeScale->textFromTick(
+		pTimeScale->tickFromFrame(m_pMidiEditor->offset()), true, pSeq->duration()));
 
 	qtractorSession  *pSession  = qtractorSession::getInstance();
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
@@ -1308,10 +1373,8 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 			!bRolling && bSelectable);
 		m_ui.transportRecordAction->setEnabled(
 			(!bLooping || !bPunching) && pSession->recordTracks() > 0);
-		m_ui.transportPunchAction->setEnabled(
-			!bLooping && (bPunching || bSelectable));
-		m_ui.transportPunchSetAction->setEnabled(
-			!bLooping && bSelectable);
+		m_ui.transportPunchAction->setEnabled(bPunching || bSelectable);
+		m_ui.transportPunchSetAction->setEnabled(bSelectable);
 		m_ui.transportRewindAction->setChecked(iRolling < 0);
 		m_ui.transportFastForwardAction->setChecked(iRolling > 0);
 		m_ui.transportLoopAction->setChecked(bLooping);
@@ -1342,6 +1405,20 @@ void qtractorMidiEditorForm::updateInstrumentNames (void)
 //-------------------------------------------------------------------------
 // qtractorMidiEditorForm -- Selection widget slots.
 
+// Zoom view menu stabilizer.
+void qtractorMidiEditorForm::updateZoomMenu (void)
+{
+	int iZoomMode = m_pMidiEditor->zoomMode();
+	
+	m_ui.viewZoomHorizontalAction->setChecked(
+		iZoomMode == qtractorMidiEditor::ZoomHorizontal);
+	m_ui.viewZoomVerticalAction->setChecked(
+		iZoomMode == qtractorMidiEditor::ZoomVertical);
+	m_ui.viewZoomAllAction->setChecked(
+		iZoomMode == qtractorMidiEditor::ZoomAll);
+}
+ 
+ 
 // Snap-per-beat view menu builder.
 void qtractorMidiEditorForm::updateSnapMenu (void)
 {
@@ -1363,6 +1440,9 @@ void qtractorMidiEditorForm::updateSnapMenu (void)
 		pAction->setChecked(iSnap == iSnapCurrent);
 		pAction->setData(iSnap++);
 	}
+
+	m_ui.viewSnapMenu->addSeparator();
+	m_ui.viewSnapMenu->addAction(m_ui.viewSnapGridAction);
 }
 
 
@@ -1380,6 +1460,11 @@ void qtractorMidiEditorForm::snapPerBeatChanged ( int iSnap )
 
 	// No need to express the change as a undoable command?
 	pTimeScale->setSnapPerBeat(iSnapPerBeat);
+
+	// If showing grid, it changed a bit for sure...
+	if (m_pMidiEditor->isSnapGrid())
+		m_pMidiEditor->updateContents();
+
 	m_pMidiEditor->editView()->setFocus();
 }
 
