@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@ public:
 
 	// Local time-scale accessor.
 	qtractorTimeScale *timeScale() const;
+	unsigned long timeOffset() const;
 
 	// MIDI clip sequence accessors.
 	void setMidiClip(qtractorMidiClip *pMidiClip);
@@ -126,13 +127,18 @@ public slots:
 	void viewZoomIn();
 	void viewZoomOut();
 	void viewZoomReset();
+	void viewZoomHorizontal();
+	void viewZoomVertical();
+	void viewZoomAll();
 	void viewSnap();
+	void viewSnapGrid(bool bOn);
 	void viewRefresh();
 
 	void helpShortcuts();
 	void helpAbout();
 	void helpAboutQt();
 
+	void updateZoomMenu();
 	void updateSnapMenu();
 
 	void sendNote(int iNote, int iVelocity);
@@ -171,8 +177,6 @@ private:
 	// Instance variables...
 	qtractorMidiEditor *m_pMidiEditor;
 
-	unsigned long m_iClipLength;
-
 	int m_iDirtyCount;
 
 	// Edit-mode action group up.
