@@ -1,7 +1,7 @@
 // qtractorMidiEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@
 #include "qtractorMidiSequence.h"
 #include "qtractorMidiClip.h"
 #include "qtractorMidiBuffer.h"
+#include "qtractorMidiControl.h"
 
 #include "qtractorPlugin.h"
 
@@ -336,9 +337,9 @@ void qtractorMidiOutputThread::run (void)
 #endif
 		// Only if playing, the output process cycle.
 		if (m_pSession->isPlaying()) {
-			m_mutex.unlock();
+			//m_mutex.unlock();
 			process();
-			m_mutex.lock();
+			//m_mutex.lock();
 		}
 	}
 	m_mutex.unlock();
@@ -362,6 +363,9 @@ void qtractorMidiOutputThread::process (void)
 	if (pMidiCursor == NULL)
 		return;
 
+	// Always do the queue drift stats ahead of the pack...
+	pMidiEngine->drift();
+
 	// Now for the next readahead bunch...
 	unsigned long iFrameStart = pMidiCursor->frame();
 	unsigned long iFrameEnd   = iFrameStart + m_iReadAhead;
@@ -375,10 +379,10 @@ void qtractorMidiOutputThread::process (void)
 	if (m_pSession->isLooping() && iFrameStart < m_pSession->loopEnd()) {
 		// Loop-length might be shorter than the read-ahead...
 		while (iFrameEnd >= m_pSession->loopEnd()) {
-			// Process the remaining until end-of-loop...
-			m_pSession->process(pMidiCursor, iFrameStart, m_pSession->loopEnd());
 			// Process metronome clicks...
 			pMidiEngine->processMetro(iFrameStart, m_pSession->loopEnd());
+			// Process the remaining until end-of-loop...
+			m_pSession->process(pMidiCursor, iFrameStart, m_pSession->loopEnd());
 			// Reset to start-of-loop...
 			iFrameStart = m_pSession->loopStart();
 			iFrameEnd   = iFrameStart + (iFrameEnd - m_pSession->loopEnd());
@@ -388,10 +392,10 @@ void qtractorMidiOutputThread::process (void)
 		}
 	}
 
-	// Regular range...
-	m_pSession->process(pMidiCursor, iFrameStart, iFrameEnd);
 	// Process metronome clicks...
 	pMidiEngine->processMetro(iFrameStart, iFrameEnd);
+	// Regular range...
+	m_pSession->process(pMidiCursor, iFrameStart, iFrameEnd);
 
 	// Sync with loop boundaries (unlikely?)...
 	if (m_pSession->isLooping() && iFrameStart < m_pSession->loopEnd()
@@ -449,8 +453,9 @@ void qtractorMidiOutputThread::trackSync ( qtractorTrack *pTrack,
 		unsigned long le = m_pSession->loopEnd();
 		if (iFrameStart < le) {
 			long iTimeStart = pMidiEngine->timeStart();
-			pMidiEngine->setTimeStart(
-				iTimeStart + m_pSession->tickFromFrame(le - ls));
+			pMidiEngine->setTimeStart(iTimeStart
+				+ m_pSession->tickFromFrame(le)
+				- m_pSession->tickFromFrame(ls));
 			trackClipSync(pTrack, iFrameStart, le);
 			pMidiEngine->setTimeStart(iTimeStart);
 			iFrameStart = ls;
@@ -543,12 +548,12 @@ qtractorMidiEngine::qtractorMidiEngine ( qtractorSession *pSession )
 	m_pOutputThread  = NULL;
 
 	m_iTimeStart     = 0;
-	m_iTimeDelta     = 0;
 	m_iTimeDrift     = 0;
 
 	m_pNotifyObject  = NULL;
 	m_eNotifyMmcType = QEvent::None;
 	m_eNotifyCtlType = QEvent::None;
+	m_eNotifySppType = QEvent::None;
 
 	m_bControlBus    = false;
 	m_pIControlBus   = NULL;
@@ -560,13 +565,22 @@ qtractorMidiEngine::qtractorMidiEngine ( qtractorSession *pSession )
 	m_iMetroChannel      = 9;	// GM Drums channel (10)
 	m_iMetroBarNote      = 76;	// GM High-wood stick
 	m_iMetroBarVelocity  = 96;
-	m_iMetroBarDuration  = 24;
+	m_iMetroBarDuration  = 48;
 	m_iMetroBeatNote     = 77;	// GM Low-wood stick
 	m_iMetroBeatVelocity = 64;
-	m_iMetroBeatDuration = 16;
+	m_iMetroBeatDuration = 24;
+
+	// Time-scale cursor (tempo/time-signature map)
+	m_pMetroCursor = NULL;
+
+	// Track down tempo changes.
+	m_fMetroTempo = 0.0f;
 
 	// No input/capture quantization (default).
 	m_iCaptureQuantize = 0;
+
+	// MIDI controller mapping flagger.
+	m_iResetAllControllers = 0;
 }
 
 
@@ -638,11 +652,22 @@ void qtractorMidiEngine::resetTempo (void)
 	if (!isActivated())
 		return;
 
+	// Needs a valid cursor...
+	if (m_pMetroCursor == NULL)
+		return;
+
+	// Reset tempo cursor.
+	m_pMetroCursor->reset();
+
 	// There must a session reference...
 	qtractorSession *pSession = session();
 	if (pSession == NULL)
 		return;
 
+	// Recache tempo node...
+	qtractorTimeScale::Node *pNode
+		= m_pMetroCursor->seekFrame(pSession->playHead());
+
 	// Set queue tempo...
 	snd_seq_queue_tempo_t *tempo;
 	snd_seq_queue_tempo_alloca(&tempo);
@@ -651,9 +676,12 @@ void qtractorMidiEngine::resetTempo (void)
 	// Set the new intended ones...
 	snd_seq_queue_tempo_set_ppq(tempo, (int) pSession->ticksPerBeat());
 	snd_seq_queue_tempo_set_tempo(tempo,
-		(unsigned int) (60000000.0f / pSession->tempo()));
+		(unsigned int) (60000000.0f / pNode->tempoEx()));
 	// Give tempo struct to the queue.
 	snd_seq_set_queue_tempo(m_pAlsaSeq, m_iAlsaQueue, tempo);
+
+	// Recache tempo value...
+	m_fMetroTempo = pNode->tempo;
 }
 
 
@@ -665,6 +693,9 @@ void qtractorMidiEngine::resetAllMonitors (void)
 	if (pSession == NULL)
 		return;
 
+	// Reset common MIDI monitor stuff...
+	qtractorMidiMonitor::resetTime(pSession);
+
 	// Reset all MIDI bus monitors...
 	for (qtractorBus *pBus = buses().first();
 			pBus; pBus = pBus->next()) {
@@ -691,9 +722,15 @@ void qtractorMidiEngine::resetAllMonitors (void)
 }
 
 
-// Reset all MIDI controllers...
-void qtractorMidiEngine::resetAllControllers (void)
+// Reset all MIDI instrument/controllers...
+void qtractorMidiEngine::resetAllControllers ( bool bForceImmediate )
 {
+	// Deferred processsing?
+	if (!bForceImmediate) {
+		m_iResetAllControllers++;
+		return;
+	}
+
 	// There must a session reference...
 	qtractorSession *pSession = session();
 	if (pSession == NULL)
@@ -720,10 +757,13 @@ void qtractorMidiEngine::resetAllControllers (void)
 		}
 	}
 
-	// Reset all MIDI track channel controllers...
+	// Reset all MIDI tracks channel bank/program and controllers...
 	for (qtractorTrack *pTrack = pSession->tracks().first();
 			pTrack; pTrack = pTrack->next()) {
 		if (pTrack->trackType() == qtractorTrack::Midi) {
+			// MIDI track instrument patching (channel bank/program)...
+			pTrack->setMidiPatch(pSession->instruments());
+			// MIDI track channel controllers...
 			qtractorMidiBus *pMidiBus
 				= static_cast<qtractorMidiBus *> (pTrack->outputBus());
 			qtractorMidiMonitor *pMidiMonitor
@@ -734,6 +774,23 @@ void qtractorMidiEngine::resetAllControllers (void)
 			}
 		}
 	}
+
+	// Re-send all mapped feedback MIDI controllers...
+	qtractorMidiControl *pMidiControl
+		= qtractorMidiControl::getInstance();
+	if (pMidiControl)
+		pMidiControl->sendAllControllers();
+
+	// Done.
+	m_iResetAllControllers = 0;
+}
+
+
+// Whether is actually pending a reset of
+// all the MIDI instrument/controllers...
+bool qtractorMidiEngine::isResetAllControllers (void) const
+{
+	return (m_iResetAllControllers > 0);
 }
 
 
@@ -811,7 +868,7 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 		data2    = pEv->data.control.value;
 		// Trap controller commands...
 		if (m_pIControlBus && m_pIControlBus->alsaPort() == iAlsaPort) {
-			// FIXME: Avoid some estraneous events...
+			// FIXME: Avoid some extraneous events...
 			if (data1 > 0x7f || data2 > 0x7f)
 				return;
 			// Post the stuffed event...
@@ -841,6 +898,21 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 		data1    = (iSysex & 0x007f);
 		data2    = (iSysex & 0x3f80) >> 7;
 		break;
+	case SND_SEQ_EVENT_START:
+	case SND_SEQ_EVENT_STOP:
+	case SND_SEQ_EVENT_CONTINUE:
+	case SND_SEQ_EVENT_SONGPOS:
+		// Trap controller commands...
+		if (m_pIControlBus && m_pIControlBus->alsaPort() == iAlsaPort) {
+			// Post the stuffed event...
+			if (m_pNotifyObject) {
+				QApplication::postEvent(m_pNotifyObject,
+					new qtractorMidiSppEvent(m_eNotifySppType,
+						int(pEv->type), pEv->data.control.value));
+			}
+		}
+		// Not handled any longer.
+		return;
 	case SND_SEQ_EVENT_SYSEX:
 		type     = qtractorMidiEvent::SYSEX;
 		pSysex   = (unsigned char *) pEv->data.ext.ptr;
@@ -864,7 +936,7 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 	}
 
 	// Now check which bus and track we're into...
-	int iDrainOutput = 0;
+//	int iDrainOutput = 0;
 	bool bRecording = (pSession->isRecording() && isPlaying());
 	for (qtractorTrack *pTrack = pSession->tracks().first();
 			pTrack; pTrack = pTrack->next()) {
@@ -908,9 +980,8 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 						snd_seq_ev_set_source(pEv, pMidiBus->alsaPort());
 						snd_seq_ev_set_subs(pEv);
 						snd_seq_ev_set_direct(pEv);
-						snd_seq_event_output(m_pAlsaSeq, pEv);
-					//	snd_seq_drain_output(m_pAlsaSeq);
-						iDrainOutput++;
+						snd_seq_event_output_direct(m_pAlsaSeq, pEv);
+					//	iDrainOutput++;
 						// Done with MIDI-thru.
 						pMidiBus->midiMonitor_out()->enqueue(type, data2);
 						// Do it for the MIDI plugins too...
@@ -947,9 +1018,8 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 					snd_seq_ev_set_source(pEv, pMidiBus->alsaPort());
 					snd_seq_ev_set_subs(pEv);
 					snd_seq_ev_set_direct(pEv);
-					snd_seq_event_output(m_pAlsaSeq, pEv);
-				//	snd_seq_drain_output(m_pAlsaSeq);
-					iDrainOutput++;
+					snd_seq_event_output_direct(m_pAlsaSeq, pEv);
+				//	iDrainOutput++;
 					// Done with MIDI-thru.
 					pMidiBus->midiMonitor_out()->enqueue(type, data2);
 				}
@@ -957,8 +1027,8 @@ void qtractorMidiEngine::capture ( snd_seq_event_t *pEv )
 		}
 	}
 
-	if (iDrainOutput > 0)
-		snd_seq_drain_output(m_pAlsaSeq);
+//	if (iDrainOutput > 0)
+//		snd_seq_drain_output(m_pAlsaSeq);
 }
 
 
@@ -1084,44 +1154,65 @@ void qtractorMidiEngine::enqueue ( qtractorTrack *pTrack,
 }
 
 
-// Flush ouput queue (if necessary)...
-void qtractorMidiEngine::flush (void)
+// Do ouput queue status (audio vs. MIDI)...
+void qtractorMidiEngine::drift (void)
 {
-	// Really flush MIDI output...
-	snd_seq_drain_output(m_pAlsaSeq);
-
 	qtractorSession *pSession = session();
 	if (pSession == NULL)
 		return;
 //	if (pSession->isRecording())
 //		return;
 
+	if (m_pMetroCursor == NULL)
+		return;
+
 	// Time to have some corrective approach...?
 	snd_seq_queue_status_t *pQueueStatus;
 	snd_seq_queue_status_alloca(&pQueueStatus);
 	if (snd_seq_get_queue_status(
 			m_pAlsaSeq, m_iAlsaQueue, pQueueStatus) >= 0) {
-		unsigned long iMidiTime
-			= snd_seq_queue_status_get_tick_time(pQueueStatus);
-		unsigned long iAudioTime = pSession->tickFromFrame(
-			pSession->audioEngine()->sessionCursor()->frameTime());
-		m_iTimeDelta = (iAudioTime - iMidiTime) - m_iTimeDrift;
-		if (m_iTimeDelta && iAudioTime > 0 && iMidiTime > 0) {
-		//	m_iTimeStart += m_iTimeDelta;
-			m_iTimeDrift += m_iTimeDelta;
-#ifdef CONFIG_DEBUG
-			qDebug("qtractorMidiEngine::flush(): "
-				"iAudioTime=%lu iMidiTime=%lu iTimeDelta=%ld (%ld)",
-				iAudioTime, iMidiTime, m_iTimeDelta, m_iTimeDrift);
-#endif
+		unsigned long iAudioFrame = pSession->playHead();
+		qtractorTimeScale::Node *pNode = m_pMetroCursor->seekFrame(iAudioFrame);
+		long iAudioTime = long(pNode->tickFromFrame(iAudioFrame));
+		long iMidiTime = m_iTimeStart
+			+ long(snd_seq_queue_status_get_tick_time(pQueueStatus));
+		iAudioFrame += readAhead();
+		long iDeltaMax = long(pNode->tickFromFrame(iAudioFrame)) - iAudioTime;
+		long iDeltaTime = (iAudioTime - iMidiTime) - m_iTimeDrift;
+		if (iDeltaTime && iDeltaTime > -iDeltaMax && iDeltaTime < +iDeltaMax) {
+		//	m_iTimeStart += iDeltaTime;
+			m_iTimeDrift += iDeltaTime;
+		//	m_iTimeDrift >>= 1; // Damp fast-average drift.
+		#ifdef CONFIG_DEBUG
+			qDebug("qtractorMidiEngine::drift(): "
+				"iAudioTime=%ld iMidiTime=%ld (%ld) iTimeDrift=%ld",
+				iAudioTime, iMidiTime, iDeltaTime, m_iTimeDrift);
+		#endif
 		}
+		// HACK: Make the least significant correction now!
+		if (m_iTimeDrift > 0) { --m_iTimeDrift;	++m_iTimeStart; }
+		else
+		if (m_iTimeDrift < 0) { ++m_iTimeDrift;	--m_iTimeStart; }
 	}
 }
 
 
+// Flush ouput queue (if necessary)...
+void qtractorMidiEngine::flush (void)
+{
+	// Really flush MIDI output...
+	snd_seq_drain_output(m_pAlsaSeq);
+}
+
+
 // Device engine initialization method.
 bool qtractorMidiEngine::init ( const QString& sClientName )
 {
+	// There must a session reference...
+	qtractorSession *pSession = session();
+	if (pSession == NULL)
+		return false;
+
 	// Try open a new client...
 	if (snd_seq_open(&m_pAlsaSeq, "hw", SND_SEQ_OPEN_DUPLEX, 0) < 0)
 		return false;
@@ -1158,6 +1249,9 @@ bool qtractorMidiEngine::init ( const QString& sClientName )
 		}
 	}
 
+	// Time-scale cursor (tempo/time-signature map)
+	m_pMetroCursor = new qtractorTimeScale::Cursor(pSession->timeScale());
+
 	// Open control/metronome buses, at least try...
 	openControlBus();
 	openMetroBus();
@@ -1182,8 +1276,8 @@ bool qtractorMidiEngine::activate (void)
 	m_pOutputThread = new qtractorMidiOutputThread(pSession);
 	m_pOutputThread->start(QThread::HighPriority);
 
+	// Reset/zero tickers...
 	m_iTimeStart = 0;
-	m_iTimeDelta = 0;
 	m_iTimeDrift = 0;
 
 	// Reset all dependable monitoring...
@@ -1220,8 +1314,7 @@ bool qtractorMidiEngine::start (void)
 	resetAllMonitors();
 
 	// Start queue timer...
-	m_iTimeStart = (long) pSession->tickFromFrame(pMidiCursor->frame());
-	m_iTimeDelta = 0;
+	m_iTimeStart = long(pSession->tickFromFrame(pMidiCursor->frame()));
 	m_iTimeDrift = 0;
 
 	// Effectively start sequencer queue timer...
@@ -1288,7 +1381,6 @@ void qtractorMidiEngine::clean (void)
 		delete m_pOutputThread;
 		m_pOutputThread = NULL;
 		m_iTimeStart = 0;
-		m_iTimeDelta = 0;
 		m_iTimeDrift = 0;
 	}
 
@@ -1303,6 +1395,12 @@ void qtractorMidiEngine::clean (void)
 		m_pInputThread = NULL;
 	}
 
+	// Time-scale cursor (tempo/time-signature map)
+	if (m_pMetroCursor) {
+		delete m_pMetroCursor;
+		m_pMetroCursor = NULL;
+	}
+
 	// Drop subscription stuff.
 	if (m_pAlsaSubsSeq) {
 		if (m_pAlsaNotifier) {
@@ -1334,10 +1432,10 @@ void qtractorMidiEngine::restartLoop (void)
 {
 	qtractorSession *pSession = session();
 	if (pSession && pSession->isLooping()) {
-		m_iTimeStart -= (long) pSession->tickFromFrame(
-			pSession->loopEnd() - pSession->loopStart());
-		m_iTimeStart += m_iTimeDelta; // Drift correction...
-		m_iTimeDelta  = 0;
+		m_iTimeStart -= long(pSession->tickFromFrame(pSession->loopEnd())
+			- pSession->tickFromFrame(pSession->loopStart()));
+		m_iTimeStart += m_iTimeDrift; // Drift correction...
+		m_iTimeDrift  = 0;
 	}
 }
 
@@ -1389,11 +1487,11 @@ void qtractorMidiEngine::trackMute ( qtractorTrack *pTrack, bool bMute )
 			= static_cast<qtractorMidiBus *> (pTrack->outputBus());
 		if (pMidiBus)
 			pMidiBus->setController(pTrack, ALL_NOTES_OFF);
-		// Reset track monitor...
+		// Clear/reset track monitor...
 		qtractorMidiMonitor *pMidiMonitor
 			= static_cast<qtractorMidiMonitor *> (pTrack->monitor());
 		if (pMidiMonitor)
-			pMidiMonitor->reset();
+			pMidiMonitor->clear();
 		// Reset track plugin buffers...
 		if ((pTrack->pluginList())->midiManager())
 			(pTrack->pluginList())->midiManager()->reset();
@@ -1463,6 +1561,11 @@ void qtractorMidiEngine::setNotifyCtlType ( QEvent::Type eNotifyCtlType )
 	m_eNotifyCtlType = eNotifyCtlType;
 }
 
+void qtractorMidiEngine::setNotifySppType ( QEvent::Type eNotifySppType )
+{
+	m_eNotifySppType = eNotifySppType;
+}
+
 
 QObject *qtractorMidiEngine::notifyObject (void) const
 {
@@ -1479,6 +1582,11 @@ QEvent::Type qtractorMidiEngine::notifyCtlType (void) const
 	return m_eNotifyCtlType;
 }
 
+QEvent::Type qtractorMidiEngine::notifySppType (void) const
+{
+	return m_eNotifySppType;
+}
+
 
 // Control bus accessors.
 void qtractorMidiEngine::setControlBus ( bool bControlBus )
@@ -1651,6 +1759,37 @@ void qtractorMidiEngine::sendMmcCommand ( qtractorMmcEvent::Command cmd,
 }
 
 
+// SPP dispatch special command.
+void qtractorMidiEngine::sendSppCommand ( int iCmdType, unsigned short iSongPos ) const
+{
+	// We surely need a output control bus...
+	if (m_pOControlBus == NULL)
+		return;
+
+	// Initialize sequencer event...
+	snd_seq_event_t ev;
+	snd_seq_ev_clear(&ev);
+
+	// Addressing...
+	snd_seq_ev_set_source(&ev, m_pOControlBus->alsaPort());
+	snd_seq_ev_set_subs(&ev);
+
+	// The event will be direct...
+	snd_seq_ev_set_direct(&ev);
+
+	// Set command parameters...
+	// - SND_SEQ_EVENT_START
+	// - SND_SEQ_EVENT_STOP
+	// - SND_SEQ_EVENT_CONTINUE
+	// - SND_SEQ_EVENT_SONGPOS
+	ev.type = snd_seq_event_type(iCmdType);
+	ev.data.control.value = iSongPos;
+
+	// Bail out...
+	snd_seq_event_output_direct(m_pAlsaSeq, &ev);
+}
+
+
 // Metronome switching.
 void qtractorMidiEngine::setMetronome ( bool bMetronome )
 {
@@ -1819,23 +1958,52 @@ unsigned long qtractorMidiEngine::metroBeatDuration (void) const
 void qtractorMidiEngine::processMetro (
 	unsigned long iFrameStart, unsigned long iFrameEnd )
 {
+	if (m_pMetroCursor == NULL)
+		return;
+
+	qtractorTimeScale::Node *pNode = m_pMetroCursor->seekFrame(iFrameEnd);
+
+	// Take this moment to check for tempo changes...
+	if (pNode->tempo != m_fMetroTempo) {
+		// New tempo node...
+		unsigned long iTime = (pNode->frame < iFrameStart
+			? pNode->tickFromFrame(iFrameStart) : pNode->tick);
+		// Enqueue tempo event...
+		snd_seq_event_t ev;
+		snd_seq_ev_clear(&ev);
+		// Scheduled delivery: take into account
+		// the time playback/queue started...
+		unsigned long tick
+			= ((long) iTime > m_iTimeStart ? iTime - m_iTimeStart : 0);
+		snd_seq_ev_schedule_tick(&ev, m_iAlsaQueue, 0, tick);
+		ev.type = SND_SEQ_EVENT_TEMPO;
+		ev.data.queue.queue = m_iAlsaQueue;
+		ev.data.queue.param.value
+			= (unsigned int) (60000000.0f / pNode->tempoEx());
+		ev.dest.client = SND_SEQ_CLIENT_SYSTEM;
+		ev.dest.port = SND_SEQ_PORT_SYSTEM_TIMER;
+		// Pump it into the queue.
+		snd_seq_event_output(m_pAlsaSeq, &ev);
+		// Save for next change.
+		m_fMetroTempo = pNode->tempo;
+		// Update MIDI monitor slot stuff...
+		qtractorMidiMonitor::splitTime(session(), pNode->frame, tick);
+	}
+
+	// Get on with the actual metronome stuff...
 	if (!m_bMetronome)
 		return;
 
 	if (m_pMetroBus == NULL)
 		return;
 
-	qtractorSession *pSession = session();
-	if (pSession == NULL)
-		return;
+	// Register the next metronome beat slot.
+	unsigned long iTimeEnd = pNode->tickFromFrame(iFrameEnd);
 
-	// Register the next metronome slot.
-	unsigned long  iTimeStart = pSession->tickFromFrame(iFrameStart);
-	unsigned long  iTimeEnd   = pSession->tickFromFrame(iFrameEnd);
-
-	unsigned short iTicksPerBeat = pSession->ticksPerBeat();
-	unsigned int   iBeat = (iTimeStart + (iTicksPerBeat >> 1)) / iTicksPerBeat;
-	unsigned long  iTime = iBeat * iTicksPerBeat;
+	pNode = m_pMetroCursor->seekFrame(iFrameStart);
+	unsigned long iTimeStart = pNode->tickFromFrame(iFrameStart);
+	unsigned int  iBeat = pNode->beatFromTick(iTimeStart);
+	unsigned long iTime = pNode->tickFromBeat(iBeat);
 
 	// Intialize outbound event...
 	snd_seq_event_t ev;
@@ -1851,33 +2019,42 @@ void qtractorMidiEngine::processMetro (
 	while (iTime < iTimeEnd) {
 		// Scheduled delivery: take into account
 		// the time playback/queue started...
-		unsigned long tick
-			= ((long) iTime > m_iTimeStart ? iTime - m_iTimeStart : 0);
-		snd_seq_ev_schedule_tick(&ev, m_iAlsaQueue, 0, tick);
-		// Set proper event data...
-		if (pSession->beatIsBar(iBeat)) {
-			ev.data.note.note     = m_iMetroBarNote;
-			ev.data.note.velocity = m_iMetroBarVelocity;
-			ev.data.note.duration = m_iMetroBarDuration;
-		} else {
-			ev.data.note.note     = m_iMetroBeatNote;
-			ev.data.note.velocity = m_iMetroBeatVelocity;
-			ev.data.note.duration = m_iMetroBeatDuration;
-		}
-		// Pump it into the queue.
-		snd_seq_event_output(m_pAlsaSeq, &ev);
-		// MIDI track monitoring...
-		if (m_pOControlBus->midiMonitor_out()) {
-			m_pOControlBus->midiMonitor_out()->enqueue(
-				qtractorMidiEvent::NOTEON, ev.data.note.velocity, tick);
+		if (iTime >= iTimeStart) {
+			unsigned long tick
+				= ((long) iTime > m_iTimeStart ? iTime - m_iTimeStart : 0);
+			snd_seq_ev_schedule_tick(&ev, m_iAlsaQueue, 0, tick);
+			// Set proper event data...
+			if (pNode->beatIsBar(iBeat)) {
+				ev.data.note.note     = m_iMetroBarNote;
+				ev.data.note.velocity = m_iMetroBarVelocity;
+				ev.data.note.duration = m_iMetroBarDuration;
+			} else {
+				ev.data.note.note     = m_iMetroBeatNote;
+				ev.data.note.velocity = m_iMetroBeatVelocity;
+				ev.data.note.duration = m_iMetroBeatDuration;
+			}
+			// Pump it into the queue.
+			snd_seq_event_output(m_pAlsaSeq, &ev);
+			// MIDI track monitoring...
+			if (m_pOControlBus->midiMonitor_out()) {
+				m_pOControlBus->midiMonitor_out()->enqueue(
+					qtractorMidiEvent::NOTEON, ev.data.note.velocity, tick);
+			}
 		}
 		// Go for next beat...
-		iTime += iTicksPerBeat;
-		iBeat++;
+		iTime += pNode->ticksPerBeat;
+		pNode = m_pMetroCursor->seekBeat(++iBeat);
 	}
 }
 
 
+// Access to current tempo/time-signature cursor.
+qtractorTimeScale::Cursor *qtractorMidiEngine::metroCursor (void) const
+{
+	return m_pMetroCursor;
+}
+
+
 // Document element methods.
 bool qtractorMidiEngine::loadElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
@@ -1948,7 +2125,7 @@ bool qtractorMidiEngine::saveElement ( qtractorSessionDocument *pDocument,
 		}
 	}
 
-	// Contrrol bus (input) connects...
+	// Control bus (input) connects...
 	if (m_bControlBus && m_pIControlBus) {
 		QDomElement eInputs
 			= pDocument->document()->createElement("control-inputs");
@@ -1958,7 +2135,7 @@ bool qtractorMidiEngine::saveElement ( qtractorSessionDocument *pDocument,
 		pElement->appendChild(eInputs);
 	}
 
-	// Contrrol bus (output) connects...
+	// Control bus (output) connects...
 	if (m_bControlBus && m_pOControlBus) {
 		QDomElement eOutputs
 			= pDocument->document()->createElement("control-outputs");
@@ -1988,7 +2165,7 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 	qtractorMidiBus *pExportBus )
 {
 	// No simultaneous or foul exports...
-	if (!isActivated() || isPlaying())
+	if (isPlaying())
 		return false;
 
 	// Make sure we have an actual session cursor...
@@ -2080,19 +2257,21 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 				// For each event...
 				qtractorMidiEvent *pEvent
 					= pMidiClip->sequence()->events().first();
-				while (pEvent && iTimeClip
-					+ pEvent->time() < iTimeStart)
+				while (pEvent && iTimeClip + pEvent->time() < iTimeStart)
 					pEvent = pEvent->next();
-				while (pEvent && iTimeClip
-					+ pEvent->time() + pEvent->duration() < iTimeEnd) {
+				while (pEvent && iTimeClip + pEvent->time() < iTimeEnd) {
 					qtractorMidiEvent *pNewEvent
 						= new qtractorMidiEvent(*pEvent);
 					pNewEvent->setTime(iTimeOffset + pEvent->time());
 					if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
+						unsigned long iTimeEvent = iTimeClip + pEvent->time();
 						float fGain = pMidiClip->gain(
-							pSession->frameFromTick(pEvent->time()));
+							pSession->frameFromTick(iTimeEvent)
+							- pClip->clipStart());
 						pNewEvent->setVelocity((unsigned char)
 							(fGain * float(pEvent->velocity())) & 0x7f);
+						if (iTimeEvent + pEvent->duration() > iTimeEnd)
+							pNewEvent->setDuration(iTimeEnd - iTimeEvent);
 					}
 					pSeq->insertEvent(pNewEvent);
 					pEvent = pEvent->next();
@@ -2128,10 +2307,13 @@ bool qtractorMidiEngine::fileExport ( const QString& sExportPath,
 	// File ready for export?
 	bool bResult = file.open(sExportPath, qtractorMidiFile::Write);
 	if (bResult) {
-		file.setTempo(pSession->tempo());
-		file.setBeatsPerBar(pSession->beatsPerBar());
-		file.writeHeader(iFormat, iTracks, iTicksPerBeat);
-		file.writeTracks(ppSeqs, iSeqs);
+		if (file.writeHeader(iFormat, iTracks, iTicksPerBeat)) {
+			if (file.tempoMap()) {
+				file.tempoMap()->fromTimeScale(
+					pSession->timeScale(), iTimeStart);
+			}
+			file.writeTracks(ppSeqs, iSeqs);
+		}
 		file.close();
 	}
 
@@ -2159,21 +2341,25 @@ int qtractorMidiEngine::updateConnects (void)
 	// Control bus inputs...
 	if (m_bControlBus && m_pIControlBus) {
 		iUpdate += m_pIControlBus->updateConnects(
-				qtractorBus::Input, m_pIControlBus->inputs(), true);
+			qtractorBus::Input, m_pIControlBus->inputs(), true);
 	}
 
 	// Control bus outputs...
 	if (m_bControlBus && m_pOControlBus) {
 		iUpdate += m_pOControlBus->updateConnects(
-				qtractorBus::Output, m_pOControlBus->outputs(), true);
+			qtractorBus::Output, m_pOControlBus->outputs(), true);
 	}
 
 	// Metronome bus outputs...
 	if (m_bMetroBus && m_pMetroBus) {
 		iUpdate += m_pMetroBus->updateConnects(
-				qtractorBus::Output, m_pMetroBus->outputs(), true);
+			qtractorBus::Output, m_pMetroBus->outputs(), true);
 	}
 
+	// Reset all pending controllers, if any...
+	if (m_iResetAllControllers > 0)
+		resetAllControllers(true); // Force immediate!
+
 	// Done.
 	return iUpdate;
 }
@@ -2204,7 +2390,7 @@ qtractorMidiBus::qtractorMidiBus ( qtractorMidiEngine *pMidiEngine,
 	m_iAlsaPort = -1;
 
 	if (busMode & qtractorBus::Input) {
-		m_pIMidiMonitor = new qtractorMidiMonitor(pMidiEngine->session());
+		m_pIMidiMonitor = new qtractorMidiMonitor();
 		m_pIPluginList  = createPluginList();
 	} else {
 		m_pIMidiMonitor = NULL;
@@ -2212,7 +2398,7 @@ qtractorMidiBus::qtractorMidiBus ( qtractorMidiEngine *pMidiEngine,
 	}
 
 	if (busMode & qtractorBus::Output) {
-		m_pOMidiMonitor = new qtractorMidiMonitor(pMidiEngine->session());
+		m_pOMidiMonitor = new qtractorMidiMonitor();
 		m_pOPluginList  = createPluginList();
 	} else {
 		m_pOMidiMonitor = NULL;
@@ -2324,7 +2510,7 @@ void qtractorMidiBus::updateBusMode (void)
 	// Have a new/old input monitor?
 	if (busMode() & qtractorBus::Input) {
 		if (m_pIMidiMonitor == NULL)
-			m_pIMidiMonitor = new qtractorMidiMonitor(engine()->session());
+			m_pIMidiMonitor = new qtractorMidiMonitor();
 		if (m_pIPluginList == NULL)
 			m_pIPluginList = createPluginList();
 	} else {
@@ -2341,7 +2527,7 @@ void qtractorMidiBus::updateBusMode (void)
 	// Have a new/old output monitor?
 	if (busMode() & qtractorBus::Output) {
 		if (m_pOMidiMonitor == NULL)
-			m_pOMidiMonitor = new qtractorMidiMonitor(engine()->session());
+			m_pOMidiMonitor = new qtractorMidiMonitor();
 		if (m_pOPluginList == NULL)
 			m_pOPluginList = createPluginList();
 	} else {
@@ -2367,7 +2553,7 @@ void qtractorMidiBus::shutOff ( bool bClose ) const
 	if (pMidiEngine->alsaSeq() == NULL)
 		return;
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("qtractorMidiBus[%p]::shutOff(%d)", this, int(bClose));
 #endif
 
@@ -2446,7 +2632,7 @@ void qtractorMidiBus::setPatch ( unsigned short iChannel,
 			ev.data.control.value = (iBank & 0x3f80) >> 7;
 		else
 			ev.data.control.value = (iBank & 0x007f);
-		snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+		snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 		if (pTrackMidiManager)
 			pTrackMidiManager->direct(&ev);
 		if (pBusMidiManager)
@@ -2459,7 +2645,7 @@ void qtractorMidiBus::setPatch ( unsigned short iChannel,
 		ev.data.control.channel = iChannel;
 		ev.data.control.param   = BANK_SELECT_LSB;
 		ev.data.control.value   = (iBank & 0x007f);
-		snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+		snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 		if (pTrackMidiManager)
 			pTrackMidiManager->direct(&ev);
 		if (pBusMidiManager)
@@ -2470,13 +2656,13 @@ void qtractorMidiBus::setPatch ( unsigned short iChannel,
 	ev.type = SND_SEQ_EVENT_PGMCHANGE;
 	ev.data.control.channel = iChannel;
 	ev.data.control.value   = iProg;
-	snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+	snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 	if (pTrackMidiManager)
 		pTrackMidiManager->direct(&ev);
 	if (pBusMidiManager)
 		pBusMidiManager->direct(&ev);
 
-	pMidiEngine->flush();
+//	pMidiEngine->flush();
 }
 
 
@@ -2487,6 +2673,12 @@ void qtractorMidiBus::setController ( qtractorTrack *pTrack,
 	setControllerEx(pTrack->midiChannel(), iController, iValue, pTrack);
 }
 
+void qtractorMidiBus::setController ( unsigned short iChannel,
+	int iController, int iValue ) const
+{
+	setControllerEx(iChannel, iController, iValue, NULL);
+}
+
 
 // Direct MIDI controller common helper.
 void qtractorMidiBus::setControllerEx ( unsigned short iChannel,
@@ -2524,7 +2716,7 @@ void qtractorMidiBus::setControllerEx ( unsigned short iChannel,
 	ev.data.control.channel = iChannel;
 	ev.data.control.param   = iController;
 	ev.data.control.value   = iValue;
-	snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+	snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 
 	// Do it for the MIDI plugins too...
 	if (pTrack && (pTrack->pluginList())->midiManager())
@@ -2532,7 +2724,7 @@ void qtractorMidiBus::setControllerEx ( unsigned short iChannel,
 	if (pluginList_out() && pluginList_out()->midiManager())
 		(pluginList_out()->midiManager())->direct(&ev);
 
-	pMidiEngine->flush();
+//	pMidiEngine->flush();
 }
 
 
@@ -2573,7 +2765,7 @@ void qtractorMidiBus::sendNote ( qtractorTrack *pTrack,
 	ev.data.note.channel  = iChannel;
 	ev.data.note.note     = iNote;
 	ev.data.note.velocity = iVelocity;
-	snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+	snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 
 	// Do it for the MIDI plugins too...
 	if ((pTrack->pluginList())->midiManager())
@@ -2581,7 +2773,7 @@ void qtractorMidiBus::sendNote ( qtractorTrack *pTrack,
 	if (pluginList_out() && pluginList_out()->midiManager())
 		(pluginList_out()->midiManager())->direct(&ev);
 
-	pMidiEngine->flush();
+//	pMidiEngine->flush();
 
 	// Bus/track output monitoring...
 	if (iVelocity > 0) {
@@ -2633,9 +2825,9 @@ void qtractorMidiBus::sendSysex ( unsigned char *pSysex, unsigned int iSysex ) c
 	// Just set SYSEX stuff and send it out..
 	ev.type = SND_SEQ_EVENT_SYSEX;
 	snd_seq_ev_set_sysex(&ev, iSysex, pSysex);
-	snd_seq_event_output(pMidiEngine->alsaSeq(), &ev);
+	snd_seq_event_output_direct(pMidiEngine->alsaSeq(), &ev);
 
-	pMidiEngine->flush();
+//	pMidiEngine->flush();
 }
 
 
@@ -2794,23 +2986,21 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 	while (snd_seq_query_next_client(
 			pMidiEngine->alsaSeq(), pClientInfo) >= 0) {
 		item.client = snd_seq_client_info_get_client(pClientInfo);
-		if (item.client > 0) {
-			item.clientName = snd_seq_client_info_get_name(pClientInfo);
-			snd_seq_port_info_set_client(pPortInfo, item.client);
-			snd_seq_port_info_set_port(pPortInfo, -1);
-			while (snd_seq_query_next_port(
-					pMidiEngine->alsaSeq(), pPortInfo) >= 0) {
-				unsigned int iPortCapability
-					= snd_seq_port_info_get_capability(pPortInfo);
-				if (((iPortCapability & iPortFlags) == iPortFlags) &&
-					((iPortCapability & SND_SEQ_PORT_CAP_NO_EXPORT) == 0)) {
-					item.port = snd_seq_port_info_get_port(pPortInfo);
-					item.portName = snd_seq_port_info_get_name(pPortInfo);
-					pItem = connects.findItem(item);
-					if (pItem) {
-						pItem->port = item.port;
-						pItem->client = item.client;
-					}
+		item.clientName = snd_seq_client_info_get_name(pClientInfo);
+		snd_seq_port_info_set_client(pPortInfo, item.client);
+		snd_seq_port_info_set_port(pPortInfo, -1);
+		while (snd_seq_query_next_port(
+				pMidiEngine->alsaSeq(), pPortInfo) >= 0) {
+			unsigned int iPortCapability
+				= snd_seq_port_info_get_capability(pPortInfo);
+			if (((iPortCapability & iPortFlags) == iPortFlags) &&
+				((iPortCapability & SND_SEQ_PORT_CAP_NO_EXPORT) == 0)) {
+				item.port = snd_seq_port_info_get_port(pPortInfo);
+				item.portName = snd_seq_port_info_get_name(pPortInfo);
+				pItem = connects.findItem(item);
+				if (pItem) {
+					pItem->port = item.port;
+					pItem->client = item.client;
 				}
 			}
 		}
@@ -2860,6 +3050,9 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 	QListIterator<ConnectItem *> iter(connects);
 	while (iter.hasNext()) {
 		ConnectItem *pItem = iter.next();
+		// Don't care of non-valid client/ports...
+		if (pItem->client < 0 || pItem->port < 0)
+			continue;
 		// Mangle which is output and input...
 		if (busMode == qtractorBus::Input) {
 			seq_addr.client = pItem->client;
@@ -2893,6 +3086,11 @@ int qtractorMidiBus::updateConnects ( qtractorBus::BusMode busMode,
 		}
 	}
 
+	// Remember to resend all session/tracks control stuff,
+	// iif we've changed any of the intended MIDI connections...
+	if (iUpdate)
+		pMidiEngine->resetAllControllers(false); // Deferred++
+
 	// Done.
 	return iUpdate;
 }
@@ -2941,7 +3139,7 @@ void qtractorMidiBus::setVolume ( qtractorTrack *pTrack, float fVolume )
 // MIDI channel stereo panning.
 void qtractorMidiBus::setPanning ( qtractorTrack *pTrack, float fPanning )
 {
-	unsigned char pan = (int(63.0f * (1.0f + fPanning)) + 1) & 0x7f;
+	unsigned char pan = (unsigned char) ((0x40 + int(63.0f * fPanning)) & 0x7f);
 	setController(pTrack, CHANNEL_PANNING, pan);
 }
 
