@@ -1,7 +1,7 @@
 // qtractorMidiEngine.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -74,7 +74,8 @@ public:
 	void resetAllMonitors();
 
 	// Reset all MIDI controllers...
-	void resetAllControllers();
+	void resetAllControllers(bool bForceImmediate);
+	bool isResetAllControllers() const;
 
 	// MIDI event capture method.
 	void capture(snd_seq_event_t *pEv);
@@ -83,6 +84,9 @@ public:
 	void enqueue(qtractorTrack *pTrack, qtractorMidiEvent *pEvent,
 		unsigned long iTime, float fGain = 1.0f);
 
+	// Do ouput queue drift stats (audio vs. MIDI)...
+	void drift();
+
 	// Flush ouput queue (if necessary)...
 	void flush();
 
@@ -125,14 +129,19 @@ public:
 	// Process metronome clicks.
 	void processMetro(unsigned long iFrameStart, unsigned long iFrameEnd);
 
+	// Access to current tempo/time-signature cursor.
+	qtractorTimeScale::Cursor *metroCursor() const;
+
 	// Event notifier widget settings.
 	void setNotifyObject  (QObject *pNotifyObject);
 	void setNotifyMmcType (QEvent::Type eNotifyMmcType);
 	void setNotifyCtlType (QEvent::Type eNotifyCtlType);
+	void setNotifySppType (QEvent::Type eNotifySppType);
 
 	QObject     *notifyObject() const;
 	QEvent::Type notifyMmcType() const;
 	QEvent::Type notifyCtlType() const;
+	QEvent::Type notifySppType() const;
 
 	// Control bus accessors.
 	void setControlBus(bool bControlBus);
@@ -150,6 +159,9 @@ public:
 	void sendMmcCommand(qtractorMmcEvent::Command cmd,
 		unsigned char *pMmcData = NULL, unsigned short iMmcData = 0) const;
 
+	// SPP dispatch special command.
+	void sendSppCommand(int iCmdType, unsigned short iSongPos = 0) const;
+
 	// Document element methods.
 	bool loadElement(qtractorSessionDocument *pDocument,
 		QDomElement *pElement);
@@ -209,13 +221,13 @@ private:
 
 	// The delta-time when playback started .
 	long m_iTimeStart;
-	long m_iTimeDelta;
 	long m_iTimeDrift;
 
 	// The event notifier widget.
 	QObject      *m_pNotifyObject;
 	QEvent::Type  m_eNotifyMmcType;
 	QEvent::Type  m_eNotifyCtlType;
+	QEvent::Type  m_eNotifySppType;
 
 	// The assigned control buses.
 	bool             m_bControlBus;
@@ -234,8 +246,17 @@ private:
 	int              m_iMetroBeatVelocity;
 	unsigned long    m_iMetroBeatDuration;
 
+	// Time-scale cursor (tempo/time-signature map)
+	qtractorTimeScale::Cursor *m_pMetroCursor;
+
+	// Track down tempo changes.
+	float m_fMetroTempo;
+
 	// Input quantization (aka. record snap-per-beat).
-	unsigned short   m_iCaptureQuantize;
+	unsigned short m_iCaptureQuantize;
+
+	// Controller update pending flagger.
+	int m_iResetAllControllers;
 };
 
 
@@ -290,6 +311,8 @@ public:
 	// Direct MIDI controller helpers.
 	void setController(qtractorTrack *pTrack,
 		int iController, int iValue = 0) const;
+	void setController(unsigned short iChannel,
+		int iController, int iValue = 0) const;
 
 	// Direct MIDI note helper.
 	void sendNote(qtractorTrack *pTrack,
@@ -371,30 +394,26 @@ private:
 
 
 //----------------------------------------------------------------------
-// qtractorMidiControlEvent - MIDI Control custom event.
+// qtractorMidiSppEvent - MIDI SPP custom event.
 //
 
-class qtractorMidiControlEvent : public QEvent
+class qtractorMidiSppEvent : public QEvent
 {
 public:
 
 	// Contructor.
-	qtractorMidiControlEvent(QEvent::Type eType, unsigned short iChannel,
-		unsigned char controller, unsigned char value)
-		: QEvent(eType), m_channel(iChannel),
-			m_controller(controller), m_value(value) {}
+	qtractorMidiSppEvent(QEvent::Type eType, int iCmdType, unsigned short iSongPos)
+		: QEvent(eType), m_iCmdType(iCmdType), m_iSongPos(iSongPos) {}
 
 	// Accessors.
-	unsigned short channel()    const { return m_channel; }
-	unsigned short controller() const { return m_controller; }
-	unsigned short value()      const { return m_value; }
+	int            cmdType() const { return m_iCmdType; }
+	unsigned short songPos() const { return m_iSongPos; }
 
 private:
 
 	// Instance variables.
-	unsigned short m_channel;
-	unsigned short m_controller;
-	unsigned short m_value;
+	int            m_iCmdType;
+	unsigned short m_iSongPos;
 };
 
 
