@@ -1,7 +1,7 @@
 // qtractorMidiFile.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,10 +19,9 @@
 
 *****************************************************************************/
 
-#include "qtractorAbout.h"
 #include "qtractorMidiFile.h"
+
 #include "qtractorTimeScale.h"
-#include "qtractorSession.h"
 
 #include <QFileInfo>
 #include <QDir>
@@ -50,10 +49,10 @@ qtractorMidiFile::qtractorMidiFile (void)
 	m_iTracks       = 0;
 	m_iTicksPerBeat = 0;
 
-	m_fTempo        = 120.0f;
-	m_iBeatsPerBar  = 4;
-
 	m_pTrackInfo    = NULL;
+
+	// Special tempo/time-signature map.
+	m_pTempoMap     = NULL;
 }
 
 
@@ -142,6 +141,9 @@ bool qtractorMidiFile::open ( const QString& sFilename, int iMode )
 		}
 	}
 
+	// Special tempo/time-signature map.
+	m_pTempoMap = new qtractorMidiFileTempo(this);
+
 	// We're in business...
 	return true;
 }
@@ -159,6 +161,11 @@ void qtractorMidiFile::close (void)
 		delete [] m_pTrackInfo;
 		m_pTrackInfo = NULL;
 	}
+
+	if (m_pTempoMap) {
+		delete m_pTempoMap;
+		m_pTempoMap = NULL;
+	}
 }
 
 
@@ -168,6 +175,8 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 {
 	if (m_pFile == NULL)
 		return false;
+	if (m_pTempoMap == NULL)
+		return false;
 	if (m_iMode != Read)
 		return false;
 
@@ -345,7 +354,9 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				len = readInt();
 				if (len > 0) {
 					if (meta == qtractorMidiEvent::TEMPO) {
-						m_fTempo = (60000000.0f / float(readInt(len)));
+						m_pTempoMap->addNodeTempo(iTime,
+							qtractorTimeScale::uroundf(
+								60000000.0f / float(readInt(len))));
 					} else {
 						data = new unsigned char [len + 1];
 						if (readData(data, len) < (int) len) {
@@ -360,7 +371,9 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 							break;
 						case qtractorMidiEvent::TIME:
 							// Beats per bar is the numerator of time signature...
-							m_iBeatsPerBar = (unsigned short) data[0];
+							m_pTempoMap->addNodeTime(iTime,
+								(unsigned short) data[0],
+								(unsigned short) data[1]);
 							break;
 						default:
 							// Ignore all others...
@@ -409,6 +422,8 @@ bool qtractorMidiFile::writeHeader ( unsigned short iFormat,
 {
 	if (m_pFile == NULL)
 		return false;
+	if (m_pTempoMap)
+		return false;
 	if (m_iMode != Write)
 		return false;
 
@@ -432,7 +447,10 @@ bool qtractorMidiFile::writeHeader ( unsigned short iFormat,
 	writeInt(m_iFormat = iFormat, 2);
 	writeInt(m_iTracks = iTracks, 2);
 	writeInt(m_iTicksPerBeat = iTicksPerBeat, 2);
-	
+
+	// Special tempo/time-signature map.
+	m_pTempoMap = new qtractorMidiFileTempo(this);
+
 	// Assume all is fine.
 	return true;
 }
@@ -444,6 +462,8 @@ bool qtractorMidiFile::writeTracks ( qtractorMidiSequence **ppSeqs,
 {
 	if (m_pFile == NULL)
 		return false;
+	if (m_pTempoMap == NULL)
+		return false;
 	if (m_iMode != Write)
 		return false;
 
@@ -492,26 +512,21 @@ bool qtractorMidiFile::writeTracks ( qtractorMidiSequence **ppSeqs,
 		QByteArray aTrackName = sTrackName.toUtf8();
 		writeData((unsigned char *) aTrackName.constData(), aTrackName.length());
 
-		// Write basic META data...
-		if (m_iFormat == 0 || pSeq == NULL || (iSeqs > 1 && iTrack == 0)) {
-			// Tempo...
-			writeInt(0); // delta-delta=0
-			writeInt(qtractorMidiEvent::META, 1);
-			writeInt(qtractorMidiEvent::TEMPO, 1);
-			writeInt(3);
-			writeInt(int(60000000.0f / m_fTempo), 3);
-			// Time signature...
-			writeInt(0); // delta-time=0
-			writeInt(qtractorMidiEvent::META, 1);
-			writeInt(qtractorMidiEvent::TIME, 1);
-			writeInt(4);
-			writeInt(m_iBeatsPerBar, 1);    // Numerator.
-			writeInt(2, 1);                 // Denominator.
-			writeInt(32, 1);                // MIDI clocks per metronome click.
-			writeInt(4, 1);                 // 32nd notes per quarter.
+		// Tempo/time-signature map...
+		qtractorMidiFileTempo::Node *pNode = m_pTempoMap->nodes().first();
+
+		// Tempo/time-signature map (track 0)
+		// - applicable to SMF format 1 files...
+		if (pSeq == NULL) {
+			unsigned long iNodeTime = 0;
+			while (pNode) {
+				writeNode(pNode, iNodeTime);
+				iNodeTime = pNode->tick;
+				pNode = pNode->next();
+			}
 		}
 
-		// Now's time for proper event beeing written down...
+		// Now's time for proper events being written down...
 		//
 		if (pSeq) {
 
@@ -556,6 +571,25 @@ bool qtractorMidiFile::writeTracks ( qtractorMidiSequence **ppSeqs,
 
 			unsigned long iEventTime = 0;
 
+			// Track/channel bank-select...
+			if (pSeq->bank() >= 0) {
+				writeInt(0); // delta-time=0
+				writeInt(qtractorMidiEvent::CONTROLLER, 1);
+				writeInt(0x00, 1);	// Bank MSB.
+				writeInt((pSeq->bank() & 0x3f80) >> 7, 1);
+				writeInt(0); // delta-time=0
+				writeInt(qtractorMidiEvent::CONTROLLER, 1);
+				writeInt(0x20, 1);	// Bank LSB.
+				writeInt((pSeq->bank() & 0x007f), 1);
+			}
+
+			// Track/channel program change...
+			if (pSeq->program() >= 0) {
+				writeInt(0); // delta-time=0
+				writeInt(qtractorMidiEvent::PGMCHANGE, 1);
+				writeInt((pSeq->program() & 0x007f), 1);
+			}
+
 			// Lets-a go, down to event merge loop...
 			//
 			for (;;) {
@@ -632,6 +666,17 @@ bool qtractorMidiFile::writeTracks ( qtractorMidiSequence **ppSeqs,
 					pNoteOffItem->notesOff.remove(pNoteOff);
 				}
 
+				// Tempo/time-signature map (interleaved)
+				// - applicable to SMF format 0 files...
+				if (iSeqs > 1 && iTrack == 0) {
+					while (pNode && iTime >= pNode->tick) {
+						writeNode(pNode, iLastTime);
+						iLastTime = pNode->tick;
+						iLastStatus = 0;
+						pNode = pNode->next();
+					}
+				}
+
 				// OK. Let's get back to actual event...
 
 				// - Delta time...
@@ -867,6 +912,49 @@ int qtractorMidiFile::writeData ( unsigned char *pData, unsigned short n )
 }
 
 
+// Write tempo-time-signature node.
+void qtractorMidiFile::writeNode (
+	qtractorMidiFileTempo::Node *pNode, unsigned long iLastTime )
+{
+	unsigned long iDeltaTime
+		= (pNode->tick > iLastTime ? pNode->tick - iLastTime : 0);
+	qtractorMidiFileTempo::Node *pPrev = pNode->prev();
+
+	if (pPrev == NULL || (pPrev->tempo != pNode->tempo)) {
+	#ifdef CONFIG_DEBUG_0
+		qDebug("qtractorMidiFile::writeNode(%lu) time=%lu TEMPO (%g)",
+			iLastTime, iDeltaTime, pNode->tempo);
+	#endif
+		// Tempo change...
+		writeInt(iDeltaTime);
+		writeInt(qtractorMidiEvent::META, 1);
+		writeInt(qtractorMidiEvent::TEMPO, 1);
+		writeInt(3);
+		writeInt(qtractorTimeScale::uroundf(60000000.0f / pNode->tempo), 3);
+		iDeltaTime = 0;
+	}
+
+	if (pPrev == NULL ||
+		pPrev->beatsPerBar != pNode->beatsPerBar ||
+		pPrev->beatDivisor != pNode->beatDivisor) {
+	#ifdef CONFIG_DEBUG_0
+		qDebug("qtractorMidiFile::writeNode(%lu) time=%lu TIME (%u/%u)",
+			iLastTime, iDeltaTime, pNode->beatsPerBar, 1 << pNode->beatDivisor);
+	#endif
+		// Time signature change...
+		writeInt(iDeltaTime);
+		writeInt(qtractorMidiEvent::META, 1);
+		writeInt(qtractorMidiEvent::TIME, 1);
+		writeInt(4);
+		writeInt(pNode->beatsPerBar, 1);    // Numerator.
+		writeInt(pNode->beatDivisor, 1);    // Denominator.
+		writeInt(32, 1);                    // MIDI clocks per metronome click.
+		writeInt(4, 1);                     // 32nd notes per quarter.
+	//	iDeltaTime = 0;
+	}
+}
+
+
 // Create filename revision (name says it all).
 QString qtractorMidiFile::createFilePathRevision (
 	const QString& sFilename, int iRevision )
@@ -897,13 +985,12 @@ QString qtractorMidiFile::createFilePathRevision (
 // All-in-one SMF file writer/creator method.
 bool qtractorMidiFile::saveCopyFile ( const QString& sNewFilename,
 	const QString& sOldFilename, unsigned short iTrackChannel,
-	qtractorMidiSequence *pSeq, qtractorTimeScale *pTimeScale,
-	unsigned short iFormat )
+	unsigned short iFormat, qtractorMidiSequence *pSeq,
+	qtractorTimeScale *pTimeScale, unsigned long iTimeOffset )
 {
 	qtractorMidiFile file;
-	float fTempo = (pTimeScale ? pTimeScale->tempo() : 120.0f);
-	unsigned short iTicksPerBeat = (pTimeScale ? pTimeScale->ticksPerBeat() : 96);
-	unsigned short iBeatsPerBar  = (pTimeScale ? pTimeScale->beatsPerBar()  : 4);
+	qtractorTimeScale ts;
+	unsigned short iTracks;
 	unsigned short iSeq, iSeqs = 0;
 	qtractorMidiSequence **ppSeqs = NULL;
 	const QString sTrackName = QObject::tr("Track %1");
@@ -911,20 +998,21 @@ bool qtractorMidiFile::saveCopyFile ( const QString& sNewFilename,
 	if (pSeq == NULL)
 		return false;
 
+	if (pTimeScale)
+		ts.copy(*pTimeScale);
+
 	// Open and load the whole source file...
 	if (file.open(sOldFilename)) {
-		iTicksPerBeat = file.ticksPerBeat();
+		ts.setTicksPerBeat(file.ticksPerBeat());
 		iFormat = file.format();
 		iSeqs = (iFormat == 1 ? file.tracks() : 16);
 		ppSeqs = new qtractorMidiSequence * [iSeqs];
 		for (iSeq = 0; iSeq < iSeqs; ++iSeq) {
 			ppSeqs[iSeq] = new qtractorMidiSequence(
-				sTrackName.arg(iSeq + 1), iSeq, iTicksPerBeat);
-		}
-		if (file.readTracks(ppSeqs, iSeqs)) {
-			fTempo = file.tempo();
-			iBeatsPerBar = file.beatsPerBar();
+				sTrackName.arg(iSeq + 1), iSeq, ts.ticksPerBeat());
 		}
+		if (file.readTracks(ppSeqs, iSeqs) && file.tempoMap())
+			file.tempoMap()->intoTimeScale(&ts, iTimeOffset);
 		file.close();
 	}
 
@@ -932,14 +1020,19 @@ bool qtractorMidiFile::saveCopyFile ( const QString& sNewFilename,
 	if (!file.open(sNewFilename, qtractorMidiFile::Write))
 		return false;
 
-	file.setTempo(fTempo);
-	file.setBeatsPerBar(iBeatsPerBar);
-
 	if (ppSeqs == NULL)
 		iSeqs = (iFormat == 0 ? 1 : 2);
 
 	// Write SMF header...
-	file.writeHeader(iFormat, (iFormat == 0 ? 1 : iSeqs), iTicksPerBeat);
+	iTracks = (iFormat == 0 ? 1 : iSeqs);
+	if (!file.writeHeader(iFormat, iTracks, ts.ticksPerBeat())) {
+		file.close();
+		return false;
+	}
+
+	// Set (initial) tempo/time-signature node.
+	if (file.tempoMap())
+		file.tempoMap()->fromTimeScale(&ts, iTimeOffset);
 
 	// Write SMF tracks(s)...
 	if (ppSeqs) {
