@@ -1,7 +1,7 @@
 // qtractorMidiListView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -76,7 +76,7 @@ qtractorMidiFileItem::qtractorMidiFileItem (
 // Tooltip renderer.
 QString qtractorMidiFileItem::toolTip (void) const
 {
-	return QObject::tr("%1 (format %2)\n%3 tracks, %4 tpb\n%5")
+	return QObject::tr("%1 (format %2)\n%3 tracks, %4 tpqn\n%5")
 		.arg(QTreeWidgetItem::text(qtractorMidiListView::Name))
 		.arg(QTreeWidgetItem::text(qtractorMidiListView::Format))
 		.arg(QTreeWidgetItem::text(qtractorMidiListView::Tracks))
