@@ -1,7 +1,7 @@
 // qtractorOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -97,6 +97,7 @@ qtractorOptions::qtractorOptions (void)
 	bTransportToolbar = m_settings.value("/TransportToolbar", true).toBool();
 	bTimeToolbar    = m_settings.value("/TimeToolbar", true).toBool();
 	bThumbToolbar   = m_settings.value("/ThumbToolbar", true).toBool();
+	iZoomMode       = m_settings.value("/ZoomMode", 1).toInt();
 	m_settings.endGroup();
 
 	// Transport options group.
@@ -140,10 +141,10 @@ qtractorOptions::qtractorOptions (void)
 	iMetroChannel      = m_settings.value("/Channel", 9).toInt();
 	iMetroBarNote      = m_settings.value("/BarNote", 76).toInt();
 	iMetroBarVelocity  = m_settings.value("/BarVelocity", 96).toInt();
-	iMetroBarDuration  = m_settings.value("/BarDuration", 24).toInt();
+	iMetroBarDuration  = m_settings.value("/BarDuration", 48).toInt();
 	iMetroBeatNote     = m_settings.value("/BeatNote", 77).toInt();
 	iMetroBeatVelocity = m_settings.value("/BeatVelocity", 64).toInt();
-	iMetroBeatDuration = m_settings.value("/BeatDuration", 16).toInt();
+	iMetroBeatDuration = m_settings.value("/BeatDuration", 24).toInt();
 	m_settings.endGroup();
 
 	m_settings.endGroup(); // Options group.
@@ -157,9 +158,12 @@ qtractorOptions::qtractorOptions (void)
 	sMidiDir       = m_settings.value("/MidiDir").toString();
 	sPresetDir     = m_settings.value("/PresetDir").toString();
 	sInstrumentDir = m_settings.value("/InstrumentDir").toString();
+	sMidiControlDir = m_settings.value("/MidiControlDir").toString();
 	bAutoMonitor   = m_settings.value("/AutoMonitor", true).toBool();
 	iSnapPerBeat   = m_settings.value("/SnapPerBeat", 4).toInt();
 	fTempo   = float(m_settings.value("/Tempo", 120.0).toDouble());
+	iBeatsPerBar   = m_settings.value("/BeatsPerBar", 4).toInt();
+	iBeatDivisor   = m_settings.value("/BeatDivisor", 2).toInt();
 	iPasteRepeatCount = m_settings.value("/PasteRepeatCount", 2).toInt();
 	bPasteRepeatPeriod = m_settings.value("/PasteRepeatPeriod", false).toInt();
 	sPluginSearch  = m_settings.value("/PluginSearch").toString();
@@ -190,6 +194,19 @@ qtractorOptions::qtractorOptions (void)
 	}
 	m_settings.endGroup();
 
+	// MIDI controller file list.
+	iFile = 0;
+	midiControlFiles.clear();
+	m_settings.beginGroup("/MidiControlFiles");
+	for (;;) {
+		QString sFilename = m_settings.value(
+			sFilePrefix.arg(++iFile)).toString();
+		if (sFilename.isEmpty())
+			break;
+		midiControlFiles.append(sFilename);
+	}
+	m_settings.endGroup();
+
 	// Recent file list.
 	iFile = 0;
 	recentFiles.clear();
@@ -197,8 +214,9 @@ qtractorOptions::qtractorOptions (void)
 	while (iFile < iMaxRecentFiles) {
 		QString sFilename = m_settings.value(
 			sFilePrefix.arg(++iFile)).toString();
-		if (!sFilename.isEmpty())
-			recentFiles.append(sFilename);
+		if (sFilename.isEmpty())
+			break;
+		recentFiles.append(sFilename);
 	}
 	m_settings.endGroup();
 
@@ -224,6 +242,10 @@ qtractorOptions::qtractorOptions (void)
 	bMidiPreview     = m_settings.value("/Preview", true).toBool();
 	bMidiFollow      = m_settings.value("/Follow", false).toBool();
 	bMidiEditMode    = m_settings.value("/EditMode", false).toBool();
+	iMidiZoomMode    = m_settings.value("/ZoomMode", 3).toInt();
+	iMidiHorizontalZoom = m_settings.value("/HorizontalZoom", 100).toInt();
+	iMidiVerticalZoom = m_settings.value("/VerticalZoom", 100).toInt();
+	bMidiSnapGrid    = m_settings.value("/SnapGrid", false).toBool();
 	m_settings.endGroup();
 
 	// Meter colors.
@@ -280,6 +302,7 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/TransportToolbar", bTransportToolbar);
 	m_settings.setValue("/TimeToolbar", bTimeToolbar);
 	m_settings.setValue("/ThumbToolbar", bThumbToolbar);
+	m_settings.setValue("/ZoomMode", iZoomMode);
 	m_settings.endGroup();
 
 	// Transport options group.
@@ -340,9 +363,12 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/MidiDir", sMidiDir);
 	m_settings.setValue("/PresetDir", sPresetDir);
 	m_settings.setValue("/InstrumentDir", sInstrumentDir);
+	m_settings.setValue("/MidiControlDir", sMidiControlDir);
 	m_settings.setValue("/AutoMonitor", bAutoMonitor);
 	m_settings.setValue("/SnapPerBeat", iSnapPerBeat);
 	m_settings.setValue("/Tempo", double(fTempo));
+	m_settings.setValue("/BeatsPerBar", iBeatsPerBar);
+	m_settings.setValue("/BeatDivisor", iBeatDivisor);
 	m_settings.setValue("/PasteRepeatCount", iPasteRepeatCount);
 	m_settings.setValue("/PasteRepeatPeriod", bPasteRepeatPeriod);
 	m_settings.setValue("/PluginSearch", sPluginSearch);
@@ -371,12 +397,23 @@ qtractorOptions::~qtractorOptions (void)
         m_settings.remove(sFilePrefix.arg(iFile));
 	m_settings.endGroup();
 
+	// MIDI controller file list.
+	iFile = 0;
+	m_settings.beginGroup("/MidiControlFiles");
+	QStringListIterator iter2(midiControlFiles);
+    while (iter2.hasNext())
+		m_settings.setValue(sFilePrefix.arg(++iFile), iter2.next());
+    // Cleanup old entries, if any...
+    while (!m_settings.value(sFilePrefix.arg(++iFile)).isNull())
+        m_settings.remove(sFilePrefix.arg(iFile));
+	m_settings.endGroup();
+
 	// Recent file list.
 	iFile = 0;
 	m_settings.beginGroup("/RecentFiles");
-	QStringListIterator iter2(recentFiles);
-    while (iter2.hasNext())
-		m_settings.setValue(sFilePrefix.arg(++iFile), iter2.next());
+	QStringListIterator iter3(recentFiles);
+    while (iter3.hasNext())
+		m_settings.setValue(sFilePrefix.arg(++iFile), iter3.next());
 	m_settings.endGroup();
 
 	// Tracks widget settings.
@@ -401,6 +438,10 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/Preview", bMidiPreview);
 	m_settings.setValue("/Follow", bMidiFollow);
 	m_settings.setValue("/EditMode", bMidiEditMode);
+	m_settings.setValue("/ZoomMode", iMidiZoomMode);
+	m_settings.setValue("/HorizontalZoom", iMidiHorizontalZoom);
+	m_settings.setValue("/VerticalZoom", iMidiVerticalZoom);
+	m_settings.setValue("/SnapGrid", bMidiSnapGrid);
 	m_settings.endGroup();
 
 	// Meter colors.
@@ -431,47 +472,40 @@ QSettings& qtractorOptions::settings (void)
 //
 
 // Help about command line options.
-void qtractorOptions::print_usage ( const char *arg0 )
+void qtractorOptions::print_usage ( const QString& arg0 )
 {
 	QTextStream out(stderr);
 	out << QObject::tr(
 		"Usage: %1 [options] [session-file]\n\n"
 		QTRACTOR_TITLE " - " QTRACTOR_SUBTITLE "\n\n"
 		"Options:\n\n"
-		"  -?, --help\n\tShow help about command line options\n\n"
+		"  -h, --help\n\tShow help about command line options\n\n"
 		"  -v, --version\n\tShow version information\n\n")
 		.arg(arg0);
 }
 
 
 // Parse command line arguments into m_settings.
-bool qtractorOptions::parse_args ( int argc, char **argv )
+bool qtractorOptions::parse_args ( const QStringList& args )
 {
 	QTextStream out(stderr);
 	const QString sEol = "\n\n";
 	int iCmdArgs = 0;
+	int argc = args.count();
 
-	for (int i = 1; i < argc; i++) {
+	for (int i = 1; i < argc; ++i) {
 
 		if (iCmdArgs > 0) {
 			sSessionFile += ' ';
-			sSessionFile += argv[i];
+			sSessionFile += args.at(i);
 			iCmdArgs++;
 			continue;
 		}
 
-		QString sVal;
-		QString sArg = argv[i];
-		int iEqual = sArg.indexOf('=');
-		if (iEqual >= 0) {
-			sVal = sArg.right(sArg.length() - iEqual - 1);
-			sArg = sArg.left(iEqual);
-		}
-		else if (i < argc)
-			sVal = argv[i + 1];
+		QString sArg = args.at(i);
 
-		if (sArg == "-?" || sArg == "--help") {
-			print_usage(argv[0]);
+		if (sArg == "-h" || sArg == "--help") {
+			print_usage(args.at(0));
 			return false;
 		}
 		else if (sArg == "-v" || sArg == "--version") {
@@ -481,7 +515,7 @@ bool qtractorOptions::parse_args ( int argc, char **argv )
 		}
 		else {
 			// If we don't have one by now,
-			// this will be the startup sesion file...
+			// this will be the startup session file...
 			sSessionFile += sArg;
 			iCmdArgs++;
 		}
