@@ -1,7 +1,7 @@
 // qtractorOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -48,9 +48,9 @@ public:
 	QSettings& settings();
 
 	// Command line arguments parser.
-	bool parse_args(int argc, char **argv);
+	bool parse_args(const QStringList& args);
 	// Command line usage helper.
-	void print_usage(const char *arg0);
+	void print_usage(const QString& arg0);
 
 	// Startup supplied session file.
 	QString sSessionFile;
@@ -82,6 +82,7 @@ public:
 	bool    bTransportToolbar;
 	bool    bTimeToolbar;
 	bool    bThumbToolbar;
+	int     iZoomMode;
 
 	// Transport options...
 	bool    bMetronome;
@@ -128,6 +129,7 @@ public:
 	QString sMidiDir;
 	QString sPresetDir;
 	QString sInstrumentDir;
+	QString sMidiControlDir;
 
 	// Session options.
 	bool    bSessionTemplate;
@@ -135,6 +137,8 @@ public:
 	bool	bAutoMonitor;
 	int     iSnapPerBeat;
 	float   fTempo;
+	int     iBeatsPerBar;
+	int     iBeatDivisor;
 
 	// Paste-repeat convenient defaults.
 	int     iPasteRepeatCount;
@@ -159,6 +163,9 @@ public:
 	// The instrument file list.
 	QStringList instrumentFiles;
 
+	// The MIDI controllers file list.
+	QStringList midiControlFiles;
+
 	// Recent file list.
 	int iMaxRecentFiles;
 	QStringList recentFiles;
@@ -180,6 +187,10 @@ public:
 	bool bMidiPreview;
 	bool bMidiFollow;
 	bool bMidiEditMode;
+	int  iMidiZoomMode;
+	int  iMidiHorizontalZoom;
+	int  iMidiVerticalZoom;
+	bool bMidiSnapGrid;
 
 	// Meter colors.
 	QStringList audioMeterColors;
