@@ -1,7 +1,7 @@
 // qtractorPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -625,6 +625,9 @@ qtractorPluginForm *qtractorPlugin::form (void)
 		Qt::WindowFlags wflags = Qt::Window
 		#if QT_VERSION >= 0x040200
 			| Qt::CustomizeWindowHint
+		#if QT_VERSION >= 0x040500
+			| Qt::WindowCloseButtonHint
+		#endif
 		#endif
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
@@ -887,7 +890,7 @@ void qtractorPlugin::saveValues (
 	}
 }
 
-	
+
 //----------------------------------------------------------------------------
 // qtractorPluginList -- Plugin chain list instance.
 //
@@ -985,6 +988,7 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 // Reset and (re)activate all plugin chain.
 void qtractorPluginList::resetBuffer (void)
 {
+#if 0
 	// Save and reset activation count...
 	int iActivated = m_iActivated;
 	m_iActivated = 0;
@@ -995,13 +999,13 @@ void qtractorPluginList::resetBuffer (void)
 		if (pPlugin->isActivated())
 			pPlugin->deactivate();
 	}
-
+#endif
 	// Reset interim buffer, if any...
 	if (m_pppBuffers[1]) {
 		for (unsigned short i = 0; i < m_iChannels; i++)
 			::memset(m_pppBuffers[1][i], 0, m_iBufferSize * sizeof(float));
 	}
-
+#if 0
 	// Restore activation of all previously deactivated plugins...
 	for (qtractorPlugin *pPlugin = first();
 			pPlugin; pPlugin = pPlugin->next()) {
@@ -1011,6 +1015,7 @@ void qtractorPluginList::resetBuffer (void)
 
 	// Restore activation count.
 	m_iActivated = iActivated;
+#endif
 }
 
 
@@ -1329,9 +1334,9 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 					pPlugin->setValueList(vlist);
 				if (!values.isEmpty())
 					pPlugin->setValues(values);
-				pPlugin->setActivated(bActivated);
 				addPluginRef(pPlugin);
 				append(pPlugin);
+				pPlugin->setActivated(bActivated); // Later's better!
 			}
 		}
 		else
