@@ -25,9 +25,10 @@
 
 #include "qtractorPluginListView.h"
 
-#include "qtractorMainForm.h"
 #include "qtractorSession.h"
 
+#include "qtractorMidiBuffer.h"
+
 
 //----------------------------------------------------------------------
 // class qtractorPluginCommand - implementation
@@ -56,11 +57,7 @@ qtractorPluginCommand::~qtractorPluginCommand (void)
 // Add new plugin(s) command methods.
 bool qtractorPluginCommand::addPlugins (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -86,11 +83,7 @@ bool qtractorPluginCommand::addPlugins (void)
 // Remove existing plugin(s) command methods.
 bool qtractorPluginCommand::removePlugins (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -181,11 +174,7 @@ bool qtractorInsertPluginCommand::redo (void)
 	if (pPlugin == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -218,11 +207,7 @@ bool qtractorInsertPluginCommand::undo (void)
 	if (pPlugin == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -275,11 +260,7 @@ bool qtractorMovePluginCommand::redo (void)
 	if (m_pPluginList == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -364,9 +345,10 @@ bool qtractorPresetPluginCommand::redo (void)
 	if (pPlugin == NULL)
 		return false;
 
-	// Save the currtoggled state alright...
-	QStringList vlist = pPlugin->values();
-	pPlugin->setValues(m_vlist);
+	// Save the current toggled state alright...
+	QStringList vlist = pPlugin->valueList();
+	pPlugin->setValueList(m_vlist);
+	pPlugin->realizeValues();
 	// Swap it nice, finally.
 	m_vlist = vlist;
 
@@ -406,10 +388,11 @@ bool qtractorResetPluginCommand::redo (void)
 	// Toggle/swap it nice...
 	m_bReset = !m_bReset;
 	if (m_bReset) {
-		m_vlist = pPlugin->values();
+		m_vlist = pPlugin->valueList();
 		pPlugin->reset();
 	} else {
-		pPlugin->setValues(m_vlist);
+		pPlugin->setValueList(m_vlist);
+		pPlugin->realizeValues();
 	}
 
 	// Update the form, showing it up as necessary...
@@ -443,10 +426,10 @@ qtractorPluginParamCommand::qtractorPluginParamCommand (
 
 	// Try replacing an previously equivalent command...
 	static qtractorPluginParamCommand *s_pPrevParamCommand = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (s_pPrevParamCommand && pMainForm) {
+	if (s_pPrevParamCommand) {
+		qtractorSession *pSession = qtractorSession::getInstance();
 		qtractorCommand *pLastCommand
-			= pMainForm->commands()->lastCommand();
+			= (pSession->commands())->lastCommand();
 		qtractorCommand *pPrevCommand
 			= static_cast<qtractorCommand *> (s_pPrevParamCommand);
 		if (pPrevCommand == pLastCommand
@@ -462,7 +445,7 @@ qtractorPluginParamCommand::qtractorPluginParamCommand (
 				if (iPrevSign == iCurrSign) {
 					m_fPrevValue = fLastValue;
 					m_bPrevValue = true;
-					pMainForm->commands()->removeLastCommand();
+					(pSession->commands())->removeLastCommand();
 				}
 			}
 		}
@@ -489,6 +472,8 @@ bool qtractorPluginParamCommand::redo (void)
 
 	// Update the form, showing it up as necessary...
 	(pPlugin->form())->updateParamWidget(m_pParam->index());
+	// Update any GUI editor...
+	pPlugin->idleEditor();
 
 	return true;
 }
@@ -501,4 +486,37 @@ bool qtractorPluginParamCommand::undo (void)
 }
 
 
+//----------------------------------------------------------------------
+// class qtractorAudioOutputBusCommand - declaration.
+//
+
+// Constructor.
+qtractorAudioOutputBusCommand::qtractorAudioOutputBusCommand (
+	qtractorMidiManager *pMidiManager, bool bAudioOutputBus )
+	: qtractorCommand(QObject::tr("dedicated audio outputs")),
+		m_pMidiManager(pMidiManager),
+		m_bAudioOutputBus(bAudioOutputBus)
+{
+}
+
+
+// Plugin audio ouput bus command methods.
+bool qtractorAudioOutputBusCommand::redo (void)
+{
+	if (m_pMidiManager == NULL)
+		return false;
+
+	bool bAudioOutputBus = m_pMidiManager->isAudioOutputBus();
+	m_pMidiManager->setAudioOutputBus(m_bAudioOutputBus);
+	m_bAudioOutputBus = bAudioOutputBus;
+
+	return true;
+}
+
+bool qtractorAudioOutputBusCommand::undo (void)
+{
+	return redo();
+}
+
+
 // end of qtractorPluginCommand.cpp
