@@ -29,6 +29,10 @@
 
 #include "qtractorMainForm.h"
 
+#include "qtractorAudioEngine.h"
+#include "qtractorMidiBuffer.h"
+#include "qtractorConnections.h"
+
 #include <QItemDelegate>
 #include <QPainter>
 #include <QMenu>
@@ -42,6 +46,11 @@
 #include <QDropEvent>
 #include <QContextMenuEvent>
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------------
 // qtractorTinyScrollBarStyle -- Custom style to have some tiny scrollbars
@@ -110,11 +119,11 @@ public:
 			pPainter->drawText(rect,
 				Qt::AlignLeft | Qt::AlignVCenter, pItem->text());
 			// Draw frame lines...
-			pPainter->setPen(rgbBack.light(150));
+			pPainter->setPen(rgbBack.lighter(150));
 			pPainter->drawLine(
 				option.rect.left(),  option.rect.top(),
 				option.rect.right(), option.rect.top());
-			pPainter->setPen(rgbBack.dark(150));
+			pPainter->setPen(rgbBack.darker(150));
 			pPainter->drawLine(
 				option.rect.left(),  option.rect.bottom(),
 				option.rect.right(), option.rect.bottom());
@@ -197,10 +206,6 @@ qtractorPluginListView::qtractorPluginListView ( QWidget *pParent )
 	QListWidget::setDropIndicatorShown(true);
 	QListWidget::setAutoScroll(true);
 
-	QListWidget::setFont(QFont(QListWidget::font().family(), 6));
-	QListWidget::verticalScrollBar()->setStyle(&g_tinyScrollBarStyle);
-	QListWidget::horizontalScrollBar()->setStyle(&g_tinyScrollBarStyle);
-
 	QListWidget::setIconSize(QSize(10, 10));
 	QListWidget::setItemDelegate(new qtractorPluginListItemDelegate(this));
 	QListWidget::setSelectionMode(QAbstractItemView::SingleSelection);
@@ -261,6 +266,26 @@ qtractorPluginList *qtractorPluginListView::pluginList (void) const
 	return m_pPluginList;
 }
 
+
+// Special scrollbar style accessors.
+void qtractorPluginListView::setTinyScrollBar ( bool bTinyScrollBar )
+{
+	if (bTinyScrollBar) {
+		QListWidget::verticalScrollBar()->setStyle(&g_tinyScrollBarStyle);
+		QListWidget::horizontalScrollBar()->setStyle(&g_tinyScrollBarStyle);
+	} else {
+		QListWidget::verticalScrollBar()->setStyle(NULL);
+		QListWidget::horizontalScrollBar()->setStyle(NULL);
+	}
+}
+
+bool qtractorPluginListView::isTinyScrollBar (void) const
+{
+	return QListWidget::verticalScrollBar()->style() == &g_tinyScrollBarStyle
+		&& QListWidget::horizontalScrollBar()->style() == &g_tinyScrollBarStyle;
+}
+
+
 // Plugin list refreshner
 void qtractorPluginListView::refresh (void)
 {
@@ -322,10 +347,12 @@ void qtractorPluginListView::moveItem (
 		pNextPlugin = pNextItem->plugin();
 
 	// Make it an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
-			new qtractorMovePluginCommand(pPlugin, pNextPlugin, m_pPluginList));
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorMovePluginCommand(pPlugin, pNextPlugin, m_pPluginList));
 }
 
 
@@ -353,12 +380,13 @@ void qtractorPluginListView::copyItem (
 		pNextPlugin = pNextItem->plugin();
 
 	// Make it an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		pMainForm->commands()->exec(
-			new qtractorInsertPluginCommand(
-				tr("copy plugin"), pPlugin, pNextPlugin));
-	}
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorInsertPluginCommand(
+			tr("copy plugin"), pPlugin, pNextPlugin));
 }
 
 
@@ -369,12 +397,13 @@ void qtractorPluginListView::addPlugin (void)
 		return;
 
 	qtractorPluginSelectForm selectForm(this);
-	selectForm.setChannels(m_pPluginList->channels(), m_pPluginList->isMidi());
+	selectForm.setChannels(m_pPluginList->channels(),
+		(m_pPluginList->flags() & qtractorPluginList::Midi));
 	if (!selectForm.exec())
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Tell the world we'll take some time...
@@ -391,12 +420,13 @@ void qtractorPluginListView::addPlugin (void)
 				selectForm.pluginFilename(i),
 				selectForm.pluginIndex(i),
 				selectForm.pluginTypeHint(i));
+		pPlugin->setActivated(selectForm.isPluginActivated());
 		pAddPluginCommand->addPlugin(pPlugin);
 		// Show the plugin form right away...
 		(pPlugin->form())->activateForm();
 	}
 
-	pMainForm->commands()->exec(pAddPluginCommand);
+	pSession->execute(pAddPluginCommand);
 
 	// We're formerly done.
 	QApplication::restoreOverrideCursor();
@@ -419,10 +449,12 @@ void qtractorPluginListView::removePlugin (void)
 		return;
 
 	// Make it a undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
-			new qtractorRemovePluginCommand(pPlugin));
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorRemovePluginCommand(pPlugin));
 }
 
 
@@ -439,12 +471,12 @@ void qtractorPluginListView::activatePlugin (void)
 		return;
 
 	// Make it a undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		pMainForm->commands()->exec(
-			new qtractorActivatePluginCommand(pPlugin,
-				!pPlugin->isActivated()));
-	}
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorActivatePluginCommand(pPlugin, !pPlugin->isActivated()));
 }
 
 
@@ -459,8 +491,8 @@ void qtractorPluginListView::activateAllPlugins (void)
 		return;
 
 	// Make it an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	qtractorActivatePluginCommand *pActivateAllCommand
@@ -473,7 +505,7 @@ void qtractorPluginListView::activateAllPlugins (void)
 			pActivateAllCommand->addPlugin(pPlugin);
 	}
 
-	pMainForm->commands()->exec(pActivateAllCommand);
+	pSession->execute(pActivateAllCommand);
 }
 
 
@@ -488,8 +520,8 @@ void qtractorPluginListView::deactivateAllPlugins (void)
 		return;
 
 	// Make it an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	qtractorActivatePluginCommand *pDeactivateAllCommand
@@ -502,7 +534,7 @@ void qtractorPluginListView::deactivateAllPlugins (void)
 			pDeactivateAllCommand->addPlugin(pPlugin);
 	}
 
-	pMainForm->commands()->exec(pDeactivateAllCommand);
+	pSession->execute(pDeactivateAllCommand);
 }
 
 
@@ -517,8 +549,8 @@ void qtractorPluginListView::removeAllPlugins (void)
 		return;
 
 	// Make it an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	qtractorRemovePluginCommand *pRemoveAllCommand
@@ -530,7 +562,7 @@ void qtractorPluginListView::removeAllPlugins (void)
 		pRemoveAllCommand->addPlugin(pPlugin);
 	}
 
-	pMainForm->commands()->exec(pRemoveAllCommand);
+	pSession->execute(pRemoveAllCommand);
 }
 
 
@@ -589,6 +621,37 @@ void qtractorPluginListView::editPlugin (void)
 }
 
 
+// Audio specific slots.
+void qtractorPluginListView::audioOutputs (void)
+{
+	qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
+	if (pMidiManager && pMidiManager->audioOutputBus()) {
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm && pMainForm->connections()) {
+			(pMainForm->connections())->showBus(
+				pMidiManager->audioOutputBus(), qtractorBus::Output);
+		}
+	}
+}
+
+
+void qtractorPluginListView::audioOutputBus (void)
+{
+	qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
+	if (pMidiManager == NULL)
+		return;
+
+	// Make it an undoable command...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorAudioOutputBusCommand(pMidiManager,
+			!pMidiManager->isAudioOutputBus()));
+}
+
+
 // Show an existing plugin form slot.
 void qtractorPluginListView::itemDoubleClickedSlot ( QListWidgetItem *item )
 {
@@ -696,9 +759,9 @@ void qtractorPluginListView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		qtractorPlugin *pPlugin = pItem->plugin();
 		if (pPlugin) {
 			// Make it a undoable command...
-			qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-			if (pMainForm) {
-				pMainForm->commands()->exec(
+			qtractorSession *pSession = qtractorSession::getInstance();
+			if (pSession) {
+				pSession->execute(
 					new qtractorActivatePluginCommand(pPlugin,
 						!pPlugin->isActivated()));
 			}
@@ -741,9 +804,9 @@ bool qtractorPluginListView::canDropEvent ( QDropEvent *pDropEvent )
 	} else if (m_pPluginList->last() == pPlugin)
 		return false;
 
-	// All that to check whether it will get properly instantiated...
-	if ((pPlugin->type())->instances(
-		m_pPluginList->channels(), m_pPluginList->isMidi()) < 1)
+	// All that to check whether it will get properly instantiated.
+	// FIXME: We don't care whether it's a MIDI chain at this time...
+	if ((pPlugin->type())->instances(m_pPluginList->channels(), false) < 1)
 		return false;
 
 	// This is the place we'll drop something
@@ -977,7 +1040,7 @@ void qtractorPluginListView::contextMenuEvent (
 	menu.addSeparator();
 
 	pAction = menu.addAction(
-		tr("Rem&ove All"), this, SLOT(removeAllPlugins()));
+		tr("Re&move All"), this, SLOT(removeAllPlugins()));
 	pAction->setEnabled(bEnabled);
 
 	menu.addSeparator();
@@ -998,9 +1061,23 @@ void qtractorPluginListView::contextMenuEvent (
 		QIcon(":/icons/formEdit.png"),
 		tr("&Edit Plugin..."), this, SLOT(editPlugin()));
 	pAction->setCheckable(true);
-	pAction->setChecked(pPlugin && pPlugin->isVisible());
+	pAction->setChecked(pPlugin && pPlugin->isFormVisible());
 	pAction->setEnabled(pItem != NULL);
 
+	qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
+	if (pMidiManager) {
+		menu.addSeparator();
+		QMenu *pAudioMenu = menu.addMenu(
+			QIcon(":/icons/trackAudio.png"), "Audi&o");
+		pAction = pAudioMenu->addAction(
+			tr("&Outputs"), this, SLOT(audioOutputs()));
+		pAction->setEnabled(pMidiManager->audioOutputBus() != NULL);
+		pAction = pAudioMenu->addAction(
+			tr("&Dedicated"), this, SLOT(audioOutputBus()));
+		pAction->setCheckable(true);
+		pAction->setChecked(pMidiManager->isAudioOutputBus());
+	}
+
 	menu.exec(pContextMenuEvent->globalPos());
 }
 
