@@ -1,7 +1,7 @@
 // qtractorSession.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -145,6 +145,9 @@ qtractorSession::~qtractorSession (void)
 // Open session engine(s).
 bool qtractorSession::open ( const QString& sClientName )
 {
+	// Lock it up...
+	lock();
+
 	// A default MIDI master bus is always in order...
 	if (m_pMidiEngine->buses().count() == 0)
 		m_pMidiEngine->addBus(new qtractorMidiBus(m_pMidiEngine, "Master"));
@@ -156,6 +159,7 @@ bool qtractorSession::open ( const QString& sClientName )
 	//  Actually open session device engines...
 	if (!m_pAudioEngine->open(sClientName) ||
 		!m_pMidiEngine->open(sClientName)) {
+		unlock();
 		close();
 		return false;
 	}
@@ -164,12 +168,15 @@ bool qtractorSession::open ( const QString& sClientName )
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
 		if (!pTrack->open()) {
+			unlock();
 			close();
 			return false;
 		}
 	}
 
 	// Done.
+	unlock();
+
 	return true;
 }
 
@@ -177,6 +184,9 @@ bool qtractorSession::open ( const QString& sClientName )
 // Close session engine(s).
 void qtractorSession::close (void)
 {
+	// Lock it up...
+	lock();
+
 	m_pAudioEngine->close();
 	m_pMidiEngine->close();
 
@@ -186,6 +196,7 @@ void qtractorSession::close (void)
 		pTrack->close();
 	}
 
+	unlock();
 //	clear();
 }
 
@@ -241,13 +252,21 @@ void qtractorSession::clear (void)
 
 	updateTimeScale();
 
-	m_pAudioEngine->sessionCursor()->reset();
-	m_pAudioEngine->sessionCursor()->seek(0);
-	m_cursors.append(m_pAudioEngine->sessionCursor());
+	qtractorSessionCursor *pAudioCursor = m_pAudioEngine->sessionCursor(); 
+	if (pAudioCursor) {
+		pAudioCursor->resetClips();
+		pAudioCursor->reset();
+		pAudioCursor->seek(0);
+		m_cursors.append(pAudioCursor);
+	}
 
-	m_pMidiEngine->sessionCursor()->reset();
-	m_pMidiEngine->sessionCursor()->seek(0);
-	m_cursors.append(m_pMidiEngine->sessionCursor());
+	qtractorSessionCursor *pMidiCursor = m_pMidiEngine->sessionCursor(); 
+	if (pMidiCursor) {
+		pMidiCursor->resetClips();
+		pMidiCursor->reset();
+		pMidiCursor->seek(0);
+		m_cursors.append(pMidiCursor);
+	}
 }
 
 
@@ -374,6 +393,19 @@ float qtractorSession::tempo (void) const
 }
 
 
+// Tempo beat type accessors.
+void qtractorSession::setBeatType ( unsigned short iBeatType )
+{
+	m_props.timeScale.setBeatType(iBeatType);
+}
+
+
+unsigned short qtractorSession::beatType (void) const
+{
+	return m_props.timeScale.beatType();
+}
+
+
 // Resolution accessors.
 void qtractorSession::setTicksPerBeat ( unsigned short iTicksPerBeat )
 {
@@ -399,9 +431,16 @@ unsigned short qtractorSession::beatsPerBar (void) const
 }
 
 
-bool qtractorSession::beatIsBar ( unsigned int iBeat ) const
+// Time signature (denominator) accessors.
+void qtractorSession::setBeatDivisor ( unsigned short iBeatDivisor )
 {
-	return ((iBeat % m_props.timeScale.beatsPerBar()) == 0);
+	m_props.timeScale.setBeatDivisor(iBeatDivisor);
+}
+
+
+unsigned short qtractorSession::beatDivisor (void) const
+{
+	return m_props.timeScale.beatDivisor();
 }
 
 
@@ -478,25 +517,13 @@ unsigned long qtractorSession::editTail (void) const
 }
 
 
-// Pixel/Beat number conversion.
-unsigned int qtractorSession::beatFromPixel ( unsigned int x ) const
-{
-	return m_props.timeScale.beatFromPixel(x);
-}
-
-unsigned int qtractorSession::pixelFromBeat ( unsigned int iBeat ) const
-{ 
-	return m_props.timeScale.pixelFromBeat(iBeat);
-}
-
-
 // Pixel/Tick number conversion.
-unsigned long qtractorSession::tickFromPixel ( unsigned int x ) const
+unsigned long qtractorSession::tickFromPixel ( unsigned int x )
 {
 	return m_props.timeScale.tickFromPixel(x);
 }
 
-unsigned int qtractorSession::pixelFromTick ( unsigned long iTick ) const
+unsigned int qtractorSession::pixelFromTick ( unsigned long iTick )
 {
 	return m_props.timeScale.pixelFromTick(iTick);
 }
@@ -515,41 +542,41 @@ unsigned int qtractorSession::pixelFromFrame ( unsigned long iFrame ) const
 
 
 // Beat/frame conversion.
-unsigned long qtractorSession::frameFromBeat ( unsigned int iBeat ) const
+unsigned long qtractorSession::frameFromBeat ( unsigned int iBeat )
 {
 	return m_props.timeScale.frameFromBeat(iBeat);
 }
 
-unsigned int qtractorSession::beatFromFrame ( unsigned long iFrame ) const
+unsigned int qtractorSession::beatFromFrame ( unsigned long iFrame )
 {
 	return m_props.timeScale.beatFromFrame(iFrame);
 }
 
 
 // Tick/Frame number conversion.
-unsigned long qtractorSession::frameFromTick ( unsigned long iTick ) const
+unsigned long qtractorSession::frameFromTick ( unsigned long iTick )
 {
 	return m_props.timeScale.frameFromTick(iTick);
 }
 
-unsigned long qtractorSession::tickFromFrame ( unsigned long iFrame ) const
+unsigned long qtractorSession::tickFromFrame ( unsigned long iFrame )
 {
 	return m_props.timeScale.tickFromFrame(iFrame);
 }
 
 
 // Beat/frame snap filters.
-unsigned long qtractorSession::tickSnap ( unsigned long iTick ) const
+unsigned long qtractorSession::tickSnap ( unsigned long iTick )
 {
 	return m_props.timeScale.tickSnap(iTick);
 }
 
-unsigned long qtractorSession::frameSnap ( unsigned long iFrame ) const
+unsigned long qtractorSession::frameSnap ( unsigned long iFrame )
 {
 	return m_props.timeScale.frameSnap(iFrame);
 }
 
-unsigned int qtractorSession::pixelSnap ( unsigned int x ) const
+unsigned int qtractorSession::pixelSnap ( unsigned int x )
 {
 	return m_props.timeScale.pixelSnap(x);
 }
@@ -567,6 +594,19 @@ unsigned long qtractorSession::locateFromFrame ( unsigned long iFrame ) const
 }
 
 
+
+// Song position pointer (SPP=MIDI beats) to frame converters.
+unsigned long qtractorSession::frameFromSongPos ( unsigned short iSongPos )
+{
+	return frameFromTick((iSongPos * ticksPerBeat()) >> 2);
+}
+
+unsigned short qtractorSession::songPosFromFrame ( unsigned long iFrame )
+{
+	return ((tickFromFrame(iFrame) << 2) / ticksPerBeat());
+}
+
+
 // Update scale divisor factors.
 void qtractorSession::updateTimeScale (void)
 {
@@ -758,7 +798,11 @@ void qtractorSession::moveTrack ( qtractorTrack *pTrack,
 	else
 		m_tracks.append(pTrack);
 
-	reset();
+	qtractorSessionCursor *pSessionCursor = m_cursors.first();
+	while (pSessionCursor) {
+		pSessionCursor->resetClips();
+		pSessionCursor = pSessionCursor->next();
+	}
 
 //	unlock();
 }
@@ -866,7 +910,7 @@ unsigned int qtractorSession::soloTracks (void) const
 // Temporary current track accessors.
 void qtractorSession::setCurrentTrack ( qtractorTrack *pTrack )
 {
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("qtractorSession::setCurrentTrack(%p)", pTrack);
 #endif
 	m_pCurrentTrack = pTrack;
@@ -913,17 +957,6 @@ void qtractorSession::unlinkSessionCursor (
 }
 
 
-// Reset all linked session cursors.
-void qtractorSession::reset (void)
-{
-	qtractorSessionCursor *pSessionCursor = m_cursors.first();
-	while (pSessionCursor) {
-		pSessionCursor->reset();
-		pSessionCursor = pSessionCursor->next();
-	}
-}
-
-
 // Reset (reactivate) all plugin chains...
 void qtractorSession::resetAllPlugins (void)
 {
@@ -1197,7 +1230,7 @@ unsigned long qtractorSession::punchOut (void) const
 
 bool qtractorSession::isPunching (void) const
 {
-	return (m_iPunchIn < m_iPunchOut) && !isLooping();
+	return (m_iPunchIn < m_iPunchOut);
 }
 
 
@@ -1446,16 +1479,11 @@ void qtractorSession::releaseMidiTag ( qtractorTrack *pTrack )
 }
 
 
-// MIDI session/tracks instrument patching.
-void qtractorSession::setMidiPatch (void)
+// MIDI session/tracks instrument/controller patching (conditional).
+void qtractorSession::setMidiPatch ( bool bForceImmediate )
 {
-	for (qtractorTrack *pTrack = m_tracks.first();
-			pTrack; pTrack = pTrack->next()) {
-		if (pTrack->trackType() == qtractorTrack::Midi)
-			pTrack->setMidiPatch(m_pInstruments);
-	}
-
-	m_pMidiEngine->resetAllControllers();
+	if (!bForceImmediate || m_pMidiEngine->isResetAllControllers())
+		m_pMidiEngine->resetAllControllers(bForceImmediate);
 }
 
 
@@ -1524,6 +1552,7 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
 {
 	qtractorSession::clear();
+	qtractorSession::lock();
 
 	// Templates have no session name...
 	if (!pDocument->isTemplate())
@@ -1567,6 +1596,8 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 					qtractorSession::setTicksPerBeat(eProp.text().toUShort());
 				else if (eProp.tagName() == "beats-per-bar")
 					qtractorSession::setBeatsPerBar(eProp.text().toUShort());
+				else if (eProp.tagName() == "beat-divisor")
+					qtractorSession::setBeatDivisor(eProp.text().toUShort());
 			}
 			// We need to make this permanent, right now.
 			qtractorSession::updateTimeScale();
@@ -1651,6 +1682,46 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 			stabilize();
 		}
 		else
+		// Load tempo/time-signature map...
+		if (eChild.tagName() == "tempo-map") {
+			for (QDomNode nNode = eChild.firstChild();
+					!nNode.isNull();
+						nNode = nNode.nextSibling()) {
+				// Convert tempo node to element...
+				QDomElement eNode = nNode.toElement();
+				if (eNode.isNull())
+					continue;
+				// Load time-scale...
+				if (eNode.tagName() == "tempo-node") {
+					unsigned long iFrame = eNode.attribute("frame").toULong();
+					float fTempo = 120.0f;
+					unsigned short iBeatType = 2;
+					unsigned short iBeatsPerBar = 4;
+					unsigned short iBeatDivisor = 2;
+					for (QDomNode nTime = eNode.firstChild();
+							!nTime.isNull();
+								nTime = nTime.nextSibling()) {
+						// Convert node to element...
+						QDomElement eTime = nTime.toElement();
+						if (eTime.isNull())
+							continue;
+						if (eTime.tagName() == "tempo")
+							fTempo = eTime.text().toFloat();
+						else if (eTime.tagName() == "beat-type")
+							iBeatType = eTime.text().toUShort();
+						else if (eTime.tagName() == "beats-per-bar")
+							iBeatsPerBar = eTime.text().toUShort();
+						else if (eTime.tagName() == "beat-divisor")
+							iBeatDivisor = eTime.text().toUShort();
+					}
+					// Add new node to tempo/time-signature map...
+					qtractorSession::timeScale()->addNode(iFrame,
+						fTempo, iBeatType, iBeatsPerBar, iBeatDivisor);
+				}
+			}
+			// Again, make view/time scaling factors permanent.
+			qtractorSession::updateTimeScale();
+		}
 		// Load tracks...
 		if (eChild.tagName() == "tracks") {
 			for (QDomNode nTrack = eChild.firstChild();
@@ -1708,6 +1779,8 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 	if (iPunchIn < iPunchOut)
 		qtractorSession::setPunch(iPunchIn, iPunchOut);
 
+	qtractorSession::unlock();
+
 	return true;
 }
 
@@ -1735,6 +1808,8 @@ bool qtractorSession::saveElement ( qtractorSessionDocument *pDocument,
 		QString::number(qtractorSession::ticksPerBeat()), &eProps);
 	pDocument->saveTextElement("beats-per-bar",
 		QString::number(qtractorSession::beatsPerBar()), &eProps);
+	pDocument->saveTextElement("beat-divisor",
+		QString::number(qtractorSession::beatDivisor()), &eProps);
 	pElement->appendChild(eProps);
 
 	// Save session state...
@@ -1794,6 +1869,30 @@ bool qtractorSession::saveElement ( qtractorSessionDocument *pDocument,
 	eDevices.appendChild(eMidiEngine);
 	pElement->appendChild(eDevices);
 
+	// Save tempo/time-signature, if any...
+	qtractorTimeScale::Node *pNode
+		= qtractorSession::timeScale()->nodes().first();
+	if (pNode) pNode = pNode->next(); // Skip first anchor node.
+	if (pNode) {
+		QDomElement eTimeScale = pDocument->document()->createElement("tempo-map");
+		while (pNode) {
+			QDomElement eNode = pDocument->document()->createElement("tempo-node");
+			eNode.setAttribute("bar", QString::number(pNode->bar));
+			eNode.setAttribute("frame", QString::number(pNode->frame));
+			pDocument->saveTextElement("tempo",
+				QString::number(pNode->tempo), &eNode);
+			pDocument->saveTextElement("beat-type",
+				QString::number(pNode->beatType), &eNode);
+			pDocument->saveTextElement("beats-per-bar",
+				QString::number(pNode->beatsPerBar), &eNode);
+			pDocument->saveTextElement("beat-divisor",
+				QString::number(pNode->beatDivisor), &eNode);
+			eTimeScale.appendChild(eNode);
+			pNode = pNode->next();
+		}
+		pElement->appendChild(eTimeScale);
+	}
+
 	// Save track view state...
 	QDomElement eTracks = pDocument->document()->createElement("tracks");
 	QDomElement eView   = pDocument->document()->createElement("view");
