@@ -1,7 +1,7 @@
 // qtractorSession.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -88,6 +88,10 @@ public:
 	void setTempo(float fTempo);
 	float tempo() const;
 
+	// Tempo beat type accessors.
+	void setBeatType(unsigned short iBeatType);
+	unsigned short beatType() const;
+
 	// Resolution accessors.
 	void setTicksPerBeat(unsigned short iTicksPerBeat);
 	unsigned short ticksPerBeat() const;
@@ -95,7 +99,10 @@ public:
 	// Beats/Bar(measure) accessors.
 	void setBeatsPerBar(unsigned short iBeatsPerBar);
 	unsigned short beatsPerBar() const;
-	bool beatIsBar(unsigned int iBeat) const;
+
+	// Time signature (denominator) accessors.
+	void setBeatDivisor(unsigned short iBeatDivisor);
+	unsigned short beatDivisor() const;
 
 	// Horizontal zoom factor.
 	void setHorizontalZoom(unsigned short iHorizontalZoom);
@@ -113,35 +120,35 @@ public:
 	void setSnapPerBeat(unsigned short iSnapPerBeat);
 	unsigned short snapPerBeat(void) const;
 
-	// Pixel/Beat number conversion.
-	unsigned int beatFromPixel(unsigned int x) const;
-	unsigned int pixelFromBeat(unsigned int iBeat) const;
-
 	// Pixel/Tick number conversion.
-	unsigned long tickFromPixel(unsigned int x) const;
-	unsigned int pixelFromTick(unsigned long iTick) const;
+	unsigned long tickFromPixel(unsigned int x);
+	unsigned int pixelFromTick(unsigned long iTick);
 
 	// Pixel/Frame number conversion.
 	unsigned long frameFromPixel(unsigned int x) const;
 	unsigned int pixelFromFrame(unsigned long iFrame) const;
 
 	// Beat/frame conversion.
-	unsigned long frameFromBeat(unsigned int iBeat) const;
-	unsigned int beatFromFrame(unsigned long iFrame) const;
+	unsigned long frameFromBeat(unsigned int iBeat);
+	unsigned int beatFromFrame(unsigned long iFrame);
 
 	// Tick/Frame number conversion.
-	unsigned long frameFromTick(unsigned long iTick) const;
-	unsigned long tickFromFrame(unsigned long iFrame) const;
+	unsigned long frameFromTick(unsigned long iTick);
+	unsigned long tickFromFrame(unsigned long iFrame);
 
 	// Beat/frame snap filters.
-	unsigned long tickSnap(unsigned long iTick) const;
-	unsigned long frameSnap(unsigned long iFrame) const;
-	unsigned int pixelSnap(unsigned int x) const;
+	unsigned long tickSnap(unsigned long iTick);
+	unsigned long frameSnap(unsigned long iFrame);
+	unsigned int pixelSnap(unsigned int x);
 
 	// Frame/locate (SMPTE) conversion.
 	unsigned long frameFromLocate(unsigned long iLocate) const;
 	unsigned long locateFromFrame(unsigned long iFrame) const;
 
+	// Song position pointer (SPP=MIDI beats) to frame converters.
+	unsigned long  frameFromSongPos(unsigned short iSongPos);
+	unsigned short songPosFromFrame(unsigned long iFrame);
+
 	// Update time scale divisor factors.
 	void updateTimeScale();
 
@@ -188,9 +195,6 @@ public:
 		qtractorTrack::TrackType syncType = qtractorTrack::None);
 	void unlinkSessionCursor(qtractorSessionCursor *pSessionCursor);
 
-	// Reset all linked session cursors.
-	void reset();
-
 	// Reset (reactivate) all plugin chains...
 	void resetAllPlugins();
 
@@ -276,8 +280,8 @@ public:
 	void acquireMidiTag(qtractorTrack *pTrack);
 	void releaseMidiTag(qtractorTrack *pTrack);
 
-	// MIDI session/tracks instrument patching.
-	void setMidiPatch();
+	// MIDI session/tracks instrument/controller patching (conditional).
+	void setMidiPatch(bool bForceImmediate);
 
 	// MIDI managers factory methods.
 	qtractorMidiManager *createMidiManager(qtractorPluginList *pPluginList);
