@@ -1,7 +1,7 @@
 // qtractorSessionCommand.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -51,20 +51,31 @@ qtractorSessionCommand::~qtractorSessionCommand (void)
 
 // Constructor.
 qtractorSessionTempoCommand::qtractorSessionTempoCommand (
-	qtractorSession *pSession, float fTempo, unsigned short iTicksPerBeat )
+	qtractorSession *pSession, float fTempo, unsigned short iBeatType,
+	unsigned short iBeatsPerBar, unsigned short iBeatDivisor,
+	unsigned short iTicksPerBeat )
 	: qtractorSessionCommand(QObject::tr("session tempo"), pSession),
-		m_fTempo(0.0f), m_iTicksPerBeat(0), m_pClipCommand(NULL)
+		m_fTempo(0.0f), m_iBeatType(0), m_iBeatsPerBar(0), m_iBeatDivisor(0),
+		m_iTicksPerBeat(0), m_pClipCommand(NULL)
 {
 	// Tempo changes...
 	if (fTempo > 0.0f && fTempo != pSession->tempo())
 		m_fTempo = fTempo;
+	if (iBeatType > 0 && iBeatType != pSession->beatType())
+		m_iBeatType = iBeatType;
 
-	// Time resolution changes...
+	// Time signature changes...
+	if (iBeatsPerBar > 0 && iBeatsPerBar != pSession->beatsPerBar())
+		m_iBeatsPerBar = iBeatsPerBar;
+	if (iBeatDivisor > 0 && iBeatDivisor != pSession->beatDivisor())
+		m_iBeatDivisor = iBeatDivisor;
+
+	// Resolution changes...
 	if (iTicksPerBeat > 0 && iTicksPerBeat != pSession->ticksPerBeat())
 		m_iTicksPerBeat = iTicksPerBeat;
 
 	// Take care of time-stretching of all audio-clips...
-	if (m_fTempo > 0.0f && pSession->isAutoTimeStretch()) {
+	if (m_fTempo > 0.0f) {
 		for (qtractorTrack *pTrack = pSession->tracks().first();
 				pTrack; pTrack = pTrack->next()) {
 			if (pTrack->trackType() == qtractorTrack::Audio) {
@@ -75,9 +86,13 @@ qtractorSessionTempoCommand::qtractorSessionTempoCommand (
 					if (pAudioClip) {
 						if (m_pClipCommand == NULL)
 							m_pClipCommand = new qtractorClipCommand(name());
-						float fTimeStretch = (pSession->tempo()
-							* pAudioClip->timeStretch()) / m_fTempo;
-						m_pClipCommand->timeStretchClip(pClip, fTimeStretch);
+						if (pSession->isAutoTimeStretch()) {
+							float fTimeStretch = (pSession->tempo()
+								* pAudioClip->timeStretch()) / m_fTempo;
+							m_pClipCommand->timeStretchClip(pClip, fTimeStretch);
+						} else {
+							m_pClipCommand->resetClip(pClip, pClip->clipLength());
+						}
 					}
 				}
 			}
@@ -107,19 +122,38 @@ bool qtractorSessionTempoCommand::redo (void)
 
 	// Tempo changes...
 	float fTempo = 0.0f;
+	unsigned short iBeatType = 0;
 	if (m_fTempo > 0.0f) {
 		fTempo = pSession->tempo();
 		pSession->setTempo(m_fTempo);
 	}
+	if (m_iBeatType > 0) {
+		iBeatType = pSession->beatType();
+		pSession->setBeatType(m_iBeatType);
+	}
+
+	// Time signature changes...
+	unsigned short iBeatsPerBar = 0;
+	unsigned short iBeatDivisor = 0;
+	if (m_iBeatsPerBar > 0) {
+		iBeatsPerBar = pSession->beatsPerBar();
+		pSession->setBeatsPerBar(m_iBeatsPerBar);
+	}
+	if (m_iBeatDivisor > 0) {
+		iBeatDivisor = pSession->beatDivisor();
+		pSession->setBeatDivisor(m_iBeatDivisor);
+	}
 
 	// Time resolution changes...
 	unsigned short iTicksPerBeat = 0;
 	if (m_iTicksPerBeat > 0) {
 		iTicksPerBeat = pSession->ticksPerBeat();
 		pSession->setTicksPerBeat(m_iTicksPerBeat);
-		pSession->updateTimeResolution();
 	}
 
+	if (m_iBeatType > 0 || m_iTicksPerBeat > 0 || m_iBeatDivisor > 0)
+		pSession->updateTimeResolution();
+
 	// In case we have audio clips around...
 	if (m_pClipCommand)
 		m_pClipCommand->redo();
@@ -138,6 +172,12 @@ bool qtractorSessionTempoCommand::redo (void)
 	// Swap it nice, finally.
 	if (fTempo > 0.0f)
 		m_fTempo = fTempo;
+	if (iBeatType > 0)
+		m_iBeatType = iBeatType;
+	if (iBeatsPerBar > 0)
+		m_iBeatsPerBar = iBeatsPerBar;
+	if (iBeatDivisor > 0)
+		m_iBeatDivisor = iBeatDivisor;
 	if (iTicksPerBeat > 0)
 		m_iTicksPerBeat = iTicksPerBeat;
 
@@ -158,12 +198,21 @@ bool qtractorSessionTempoCommand::undo (void)
 // Constructor.
 qtractorSessionLoopCommand::qtractorSessionLoopCommand (
 	qtractorSession *pSession, unsigned long iLoopStart, unsigned long iLoopEnd )
-	: qtractorSessionCommand(QObject::tr("session loop"), pSession)
+	: qtractorSessionCommand(QObject::tr("session loop"), pSession),
+		m_iLoopStart(iLoopStart), m_iLoopEnd(iLoopEnd), m_pPunchCommand(NULL)
 {
-	m_iLoopStart = iLoopStart;
-	m_iLoopEnd   = iLoopEnd;
+	// Cannot loop and punch at the same time...
+	if (pSession->isPunching() && iLoopStart < iLoopEnd)
+		m_pPunchCommand = new qtractorSessionPunchCommand(pSession, 0, 0);
 }
 
+// Destructor.
+qtractorSessionLoopCommand::~qtractorSessionLoopCommand (void)
+{
+	if (m_pPunchCommand) delete m_pPunchCommand;
+}
+
+
 
 // Session-loop command methods.
 bool qtractorSessionLoopCommand::redo (void)
@@ -176,6 +225,10 @@ bool qtractorSessionLoopCommand::redo (void)
 	unsigned long iLoopStart = pSession->loopStart();
 	unsigned long iLoopEnd   = pSession->loopEnd();
 
+	// Do cross-command, if any.
+	if (m_pPunchCommand)
+		m_pPunchCommand->redo();
+
 	// Just set new bounds...
 	pSession->setLoop(m_iLoopStart, m_iLoopEnd);
 
@@ -193,6 +246,66 @@ bool qtractorSessionLoopCommand::redo (void)
 }
 
 bool qtractorSessionLoopCommand::undo (void)
+{
+	// As we swap the prev/loop this is non-idempotent.
+	return redo();
+}
+
+
+//----------------------------------------------------------------------
+// class qtractorSessionPunchCommand - implementation
+//
+
+// Constructor.
+qtractorSessionPunchCommand::qtractorSessionPunchCommand (
+	qtractorSession *pSession, unsigned long iPunchIn, unsigned long iPunchOut )
+	: qtractorSessionCommand(QObject::tr("session punch"), pSession),
+		m_iPunchIn(iPunchIn), m_iPunchOut(iPunchOut), m_pLoopCommand(NULL)
+{
+	// Cannot punch and loop at the same time...
+	if (pSession->isLooping() && iPunchIn < iPunchOut)
+		m_pLoopCommand = new qtractorSessionLoopCommand(pSession, 0, 0);
+}
+
+// Destructor.
+qtractorSessionPunchCommand::~qtractorSessionPunchCommand (void)
+{
+	if (m_pLoopCommand) delete m_pLoopCommand;
+}
+
+
+// Session-punch command methods.
+bool qtractorSessionPunchCommand::redo (void)
+{
+	qtractorSession *pSession = session();
+	if (pSession == NULL)
+		return false;
+
+	// Save the previous session punch state alright...
+	unsigned long iPunchIn  = pSession->punchIn();
+	unsigned long iPunchOut = pSession->punchOut();
+
+	// Do cross-command, if any.
+	if (m_pLoopCommand)
+		m_pLoopCommand->redo();
+
+	// Just set new bounds...
+	pSession->setPunch(m_iPunchIn, m_iPunchOut);
+
+	// Restore edit cursors too...
+	if (m_iPunchIn < m_iPunchOut) {
+		pSession->setEditHead(m_iPunchIn);
+		pSession->setEditTail(m_iPunchOut);
+	}
+
+	// Swap it nice, finally.
+	m_iPunchIn  = iPunchIn;
+	m_iPunchOut = iPunchOut;
+
+	return true;
+}
+
+bool qtractorSessionPunchCommand::undo (void)
 {
 	// As we swap the prev/tempo this is non-idempotent.
 	return redo();
@@ -212,11 +325,17 @@ qtractorSessionEditCommand::qtractorSessionEditCommand (
 {
 	// Append tempo/resolution changes too...
 	float fTempo = properties.timeScale.tempo();
+	unsigned short iBeatType = properties.timeScale.beatType();
+	unsigned short iBeatsPerBar = properties.timeScale.beatsPerBar();
+	unsigned short iBeatDivisor = properties.timeScale.beatDivisor();
 	unsigned short iTicksPerBeat = properties.timeScale.ticksPerBeat();
 	if (pSession->tempo() != fTempo ||
+		pSession->beatType() != iBeatType ||
+		pSession->beatsPerBar()  != iBeatsPerBar ||
+		pSession->beatDivisor()  != iBeatDivisor ||
 		pSession->ticksPerBeat() != iTicksPerBeat) {
 		m_pTempoCommand = new qtractorSessionTempoCommand(pSession,
-			fTempo, iTicksPerBeat);
+			fTempo, iBeatType, iBeatsPerBar, iBeatDivisor, iTicksPerBeat);
 	}
 
 }
