@@ -1,7 +1,7 @@
 // qtractorSessionForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -83,10 +83,7 @@ qtractorSessionForm::qtractorSessionForm (
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(changed()));
 	QObject::connect(m_ui.TempoSpinBox,
-		SIGNAL(valueChanged(double)),
-		SLOT(changed()));
-	QObject::connect(m_ui.BeatsPerBarSpinBox,
-		SIGNAL(valueChanged(int)),
+		SIGNAL(valueChanged(float, unsigned short, unsigned short)),
 		SLOT(changed()));
 	QObject::connect(m_ui.TicksPerBeatSpinBox,
 		SIGNAL(valueChanged(int)),
@@ -133,8 +130,9 @@ void qtractorSessionForm::setSession ( qtractorSession *pSession )
 		QString::number(m_props.timeScale.sampleRate()));
 	m_ui.SampleRateTextLabel->setEnabled(!pSession->isActivated());
 	m_ui.SampleRateComboBox->setEnabled(!pSession->isActivated());
-	m_ui.TempoSpinBox->setValue(m_props.timeScale.tempo());
-	m_ui.BeatsPerBarSpinBox->setValue(int(m_props.timeScale.beatsPerBar()));
+	m_ui.TempoSpinBox->setTempo(m_props.timeScale.tempo(), false);
+	m_ui.TempoSpinBox->setBeatsPerBar(m_props.timeScale.beatsPerBar(), false);
+	m_ui.TempoSpinBox->setBeatDivisor(m_props.timeScale.beatDivisor(), false);
 	m_ui.TicksPerBeatSpinBox->setValue(int(m_props.timeScale.ticksPerBeat()));
 	// View properties...
 	m_ui.SnapPerBeatComboBox->setCurrentIndex(
@@ -174,8 +172,10 @@ void qtractorSessionForm::accept (void)
 		// Time properties...
 		m_props.timeScale.setSampleRate(
 			m_ui.SampleRateComboBox->currentText().toUInt());
-		m_props.timeScale.setTempo(m_ui.TempoSpinBox->value());
-		m_props.timeScale.setBeatsPerBar(m_ui.BeatsPerBarSpinBox->value());
+		m_props.timeScale.setTempo(m_ui.TempoSpinBox->tempo());
+		m_props.timeScale.setBeatType(2);
+		m_props.timeScale.setBeatsPerBar(m_ui.TempoSpinBox->beatsPerBar());
+		m_props.timeScale.setBeatDivisor(m_ui.TempoSpinBox->beatDivisor());
 		m_props.timeScale.setTicksPerBeat(m_ui.TicksPerBeatSpinBox->value());
 		// View properties...
 		m_props.timeScale.setSnapPerBeat(qtractorTimeScale::snapFromIndex(
@@ -208,11 +208,13 @@ void qtractorSessionForm::reject (void)
 			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
-			tr("Apply"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Apply...
+			QMessageBox::Apply |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Apply:
 			accept();
 			return;
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bReject = false;
