@@ -1,7 +1,7 @@
 // qtractorTimeStretch.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    Adapted and refactored from the SoundTouch library (L)GPL,
    Copyright (C) 2001-2006, Olli Parviainen.
@@ -35,19 +35,26 @@
 // SSE detection.
 static inline bool sse_enabled (void)
 {
-	bool bSSE = false;
 #if defined(__GNUC__)
-#if defined(__i386__) || defined(__x86_64__)
 	unsigned int eax, ebx, ecx, edx;
+#if defined(__x86_64__) || (!defined(PIC) && !defined(__PIC__))
 	__asm__ __volatile__ (
-		"movl %%ebx, %%esi\n\t" \
 		"cpuid\n\t" \
-		"xchgl %%ebx, %%esi" \
-		: "=a" (eax), "=S" (ebx), "=c" (ecx), "=d" (edx) \
-		: "0" (1));
+		: "=a" (eax), "=b" (ebx), "=c" (ecx), "=d" (edx) \
+		: "a" (1) : "cc");
+#else
+	__asm__ __volatile__ (
+		"push %%ebx\n\t" \
+		"cpuid\n\t" \
+		"movl %%ebx,%1\n\t" \
+		"pop %%ebx\n\t" \
+		: "=a" (eax), "=r" (ebx), "=c" (ecx), "=d" (edx) \
+		: "a" (1) : "cc");
 #endif
+	return (edx & (1 << 25));
+#else
+	return false;
 #endif
-	return bSSE;
 }
 
 
