@@ -1,7 +1,7 @@
 // qtractorTrackView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -219,7 +219,7 @@ void qtractorTrackView::updateContentsHeight (void)
 		return;
 
 	// Allways give some room to drop something at the bottom...
-	int iContentsHeight = qtractorTrackList::ItemHeightBase << 1;
+	int iContentsHeight = qtractorTrack::HeightBase << 1;
 	// Compute total track height...
 	qtractorTrack *pTrack = pSession->tracks().first();
 	while (pTrack) {
@@ -232,12 +232,12 @@ void qtractorTrackView::updateContentsHeight (void)
 		" => iContentsHeight=%d", iContentsHeight);
 #endif
 
-	// No selection anymore (we'll update all contents anyway)...
-	m_pClipSelect->clear();
-
 	// Do the contents resize thing...
 	qtractorScrollView::resizeContents(
 		qtractorScrollView::contentsWidth(), iContentsHeight);
+
+	// Keep selection (we'll update all contents anyway)...
+	updateClipSelect();
 }
 
 
@@ -249,7 +249,10 @@ void qtractorTrackView::updateContentsWidth ( int iContentsWidth )
 		int iSessionWidth = pSession->pixelFromFrame(pSession->sessionLength());
 		if (iContentsWidth < iSessionWidth)
 			iContentsWidth = iSessionWidth;
-		iContentsWidth += pSession->pixelFromBeat(2 * pSession->beatsPerBar());
+		qtractorTimeScale::Cursor cursor(pSession->timeScale());
+		qtractorTimeScale::Node *pNode = cursor.seekPixel(iContentsWidth);
+		iContentsWidth += pNode->pixelFromBeat(
+			pNode->beat + 2 * pNode->beatsPerBar) - pNode->pixel;
 		if (iContentsWidth < qtractorScrollView::width())
 			iContentsWidth += qtractorScrollView::width();
 		m_iEditHeadX = pSession->pixelFromFrame(pSession->editHead());
@@ -261,13 +264,13 @@ void qtractorTrackView::updateContentsWidth ( int iContentsWidth )
 		" => iContentsWidth=%d", iContentsWidth);
 #endif
 
-	// No selection anymore (we'll update all contents anyway)...
-	m_pClipSelect->clear();
-
 	// Do the contents resize thing...
 	qtractorScrollView::resizeContents(
 		iContentsWidth, qtractorScrollView::contentsHeight());
 
+	// Keep selection (we'll update all contents anyway)...
+	updateClipSelect();
+
 	// Force an update on the track time line too...
 	m_pTracks->trackTime()->resizeContents(
 		iContentsWidth + 100, m_pTracks->trackTime()->contentsHeight());
@@ -565,26 +568,28 @@ void qtractorTrackView::updatePixmap ( int cx, int cy )
 
 	// Draw vertical grid lines...
 	if (cy < y2) {
-		unsigned short iBeat = pSession->beatFromPixel(cx);
-		unsigned long iFrameFromBeat = pSession->frameFromBeat(iBeat);
-		unsigned long iFramesPerBeat = pSession->frameFromBeat(1);
-		unsigned int  iPixelsPerBeat = pSession->pixelFromBeat(1);
-		x = pSession->pixelFromFrame(iFrameFromBeat) - cx;
+		qtractorTimeScale::Cursor cursor(pSession->timeScale());
+		qtractorTimeScale::Node *pNode = cursor.seekPixel(cx);
+		unsigned short iPixelsPerBeat = pNode->pixelsPerBeat();
+		unsigned int iBeat = pNode->beatFromPixel(cx);
+		int x0 = x = pNode->pixelFromBeat(iBeat) - cx;
 		while (x < w) {
 			if (x >= 0) {
-				bool bBeatIsBar = pSession->beatIsBar(iBeat);
+				bool bBeatIsBar = pNode->beatIsBar(iBeat) && (x >= x0);
 				if (bBeatIsBar) {
 					painter.setPen(rgbLight);
 					painter.drawLine(x, 0, x, y2 - cy - 2);
+					x0 = x + 16;
+					if (iBeat == pNode->beat)
+						iPixelsPerBeat = pNode->pixelsPerBeat();
 				}
 				if (bBeatIsBar || iPixelsPerBeat > 16) {
 					painter.setPen(rgbDark);
 					painter.drawLine(x - 1, 0, x - 1, y2 - cy - 2);
 				}
 			}
-			iFrameFromBeat += iFramesPerBeat;
-			x = pSession->pixelFromFrame(iFrameFromBeat) - cx;
-			iBeat++;
+			pNode = cursor.seekBeat(++iBeat);
+			x = pNode->pixelFromBeat(iBeat) - cx;
 		}
 	}
 
@@ -656,7 +661,7 @@ qtractorTrack *qtractorTrackView::trackAt ( const QPoint& pos,
 		int w = qtractorScrollView::width();// View width, not contents.
 		if (pTrack == NULL) {				// Below all tracks.
 			y1 = y2;
-			y2 = y1 + (qtractorTrackList::ItemHeightBase
+			y2 = y1 + (qtractorTrack::HeightBase
 				* pSession->verticalZoom()) / 100;
 		}
 		pTrackViewInfo->trackIndex = iTrack;
@@ -777,7 +782,7 @@ qtractorTrack *qtractorTrackView::dragMoveTrack ( const QPoint& pos,
 	qtractorTrack *pSingleTrack = m_pClipSelect->singleTrack();
 	if (pSingleTrack &&
 		(pTrack == NULL || pSingleTrack->trackType() == pTrack->trackType()))
-		updateClipSelect(tvi.trackRect.y() + 1, tvi.trackRect.height() - 2);
+		updateSingleTrack(tvi.trackRect.y() + 1, tvi.trackRect.height() - 2);
 	// Special update on keyboard vertical drag-stepping...
 	if (bKeyStep)
 		m_posStep.setY(m_posStep.y() - pos.y() + tvi.trackRect.y()
@@ -838,7 +843,6 @@ qtractorTrack *qtractorTrackView::dragDropTrack (
 	// Nothing more?
 	if (pMimeData == NULL)
 		return NULL;
-		
 	// Can it be single track channel (MIDI for sure)?
 	if (qtractorFileChannelDrag::canDecode(pMimeData)) {
 		// Let's see how many track-channels are there...
@@ -866,7 +870,7 @@ qtractorTrack *qtractorTrackView::dragDropTrack (
 	}
 
 	// Nice, now we'll try to set a preview selection rectangle set...
-	m_posDrag.setX(pSession->pixelSnap(pos.x() - 8));
+	m_posDrag.setX(pSession->pixelSnap(pos.x() > 8 ? pos.x() - 8 : 0));
 	m_posDrag.setY(tvi.trackRect.y() + 1);
 	m_rectDrag.setRect(
 		m_posDrag.x(), m_posDrag.y(), 0, tvi.trackRect.height() - 2);
@@ -878,6 +882,10 @@ qtractorTrack *qtractorTrackView::dragDropTrack (
 		// First test as a MIDI file...
 		if (m_dropType == qtractorTrack::None
 			|| m_dropType == qtractorTrack::Midi) {
+			int x0 = m_posDrag.x();
+			qtractorTimeScale::Cursor cursor(pSession->timeScale());
+			qtractorTimeScale::Node *pNode = cursor.seekPixel(x0);
+			unsigned long t1, t0 = pNode->tickFromPixel(x0);
 			qtractorMidiFile file;
 			if (file.open(pDropItem->path)) {
 				qtractorMidiSequence seq;
@@ -885,11 +893,12 @@ qtractorTrack *qtractorTrackView::dragDropTrack (
 				if (pDropItem->channel < 0) {
 					int iTracks = (file.format() == 1 ? file.tracks() : 16);
 					for (int iTrackChannel = 0;
-						iTrackChannel < iTracks; iTrackChannel++) {
+							iTrackChannel < iTracks; iTrackChannel++) {
 						if (file.readTrack(&seq, iTrackChannel)
 							&& seq.duration() > 0) {
-							m_rectDrag.setWidth(
-								pSession->pixelFromTick(seq.duration()));
+							t1 = t0 + seq.duration();
+							pNode = cursor.seekTick(t1);
+							m_rectDrag.setWidth(pNode->pixelFromTick(t1) - x0);
 							pDropItem->rect = m_rectDrag;
 							m_rectDrag.translate(0, m_rectDrag.height() + 4);
 						}
@@ -898,8 +907,9 @@ qtractorTrack *qtractorTrackView::dragDropTrack (
 						m_dropType = qtractorTrack::Midi;
 				} else if (file.readTrack(&seq, pDropItem->channel)
 					&& seq.duration() > 0) {
-					m_rectDrag.setWidth(
-						pSession->pixelFromTick(seq.duration()));
+					t1 = t0 + seq.duration();
+					pNode = cursor.seekTick(t1);
+					m_rectDrag.setWidth(pNode->pixelFromTick(t1) - x0);
 					pDropItem->rect = m_rectDrag;
 					m_rectDrag.translate(0, m_rectDrag.height() + 4);
 					if (m_dropType == qtractorTrack::None)
@@ -1762,20 +1772,47 @@ qtractorClip *qtractorTrackView::currentClip (void) const
 }
 
 
-// Draw/hide the whole current clip selection.
-void qtractorTrackView::updateClipSelect ( int y, int h ) const
+// Clip selection accessor.
+qtractorClipSelect *qtractorTrackView::clipSelect (void) const
 {
-	bool bSingleTrack = (m_pClipSelect->singleTrack() != NULL);
-	QListIterator<qtractorClipSelect::Item *> iter(m_pClipSelect->items());
-	while (iter.hasNext()) {
-		qtractorClipSelect::Item *pClipItem	= iter.next();
-		if (bSingleTrack) {
-			pClipItem->rectClip.setY(y);
-			pClipItem->rectClip.setHeight(h);
+	return m_pClipSelect;
+}
+
+
+// Update whole clip selection.
+void qtractorTrackView::updateClipSelect (void)
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	// Reset all selected clips, but don't
+	// clear their own selection state...
+	m_pClipSelect->reset();
+
+	// Now find all the clips/regions
+	// that were currently selected...
+	int y1, y2 = 0;
+	qtractorTrack *pTrack = pSession->tracks().first();
+	while (pTrack) {
+		y1  = y2;
+		y2 += pTrack->zoomHeight();
+		int y = y1 + 1;
+		int h = y2 - y1 - 2;
+		for (qtractorClip *pClip = pTrack->clips().first();
+				pClip; pClip = pClip->next()) {
+			if (pClip->isClipSelected()) {
+				int x = pSession->pixelFromFrame(pClip->clipSelectStart());
+				int w = pSession->pixelFromFrame(pClip->clipSelectEnd()) - x;
+				m_pClipSelect->addClip(pClip, QRect(x, y, w, h));
+			}
 		}
+		pTrack = pTrack->next();
 	}
 }
 
+
+// Draw/hide the whole current clip selection.
 void qtractorTrackView::showClipSelect (void) const
 {
 	QListIterator<qtractorClipSelect::Item *> iter(m_pClipSelect->items());
@@ -1796,6 +1833,18 @@ void qtractorTrackView::hideClipSelect (void) const
 }
 
 
+// Update single track clip selection.
+void qtractorTrackView::updateSingleTrack ( int y, int h ) const
+{
+	QListIterator<qtractorClipSelect::Item *> iter(m_pClipSelect->items());
+	while (iter.hasNext()) {
+		qtractorClipSelect::Item *pClipItem	= iter.next();
+		pClipItem->rectClip.setY(y);
+		pClipItem->rectClip.setHeight(h);
+	}
+}
+
+
 // Draw/hide the whole drop rectagle list
 void qtractorTrackView::updateDropRects ( int y, int h ) const
 {
@@ -2309,33 +2358,36 @@ bool qtractorTrackView::keyStep ( int iKey )
 		m_dragState != DragDropPaste)
 		return false;
 
-	int iVerticalStep = qtractorTrackList::ItemHeightMin;
-	qtractorTrack *pTrack = m_pTracks->currentTrack();
-	if (pTrack)
-		iVerticalStep += (pTrack->zoomHeight() >> 1);
-	unsigned short iSnapPerBeat = pSession->snapPerBeat();
-	if (iSnapPerBeat < 1)
-		iSnapPerBeat = 1;
-	int iHorizontalStep
-		= (pSession->horizontalZoom() * pSession->pixelsPerBeat())
-			/ (iSnapPerBeat * 100);
-
-	// Now determine which step...
-	switch (iKey) {
-	case Qt::Key_Left:
-		m_posStep.setX(m_posStep.x() - iHorizontalStep);
-		break;
-	case Qt::Key_Right:
-		m_posStep.setX(m_posStep.x() + iHorizontalStep);
-		break;
-	case Qt::Key_Up:
-		m_posStep.setY(m_posStep.y() - iVerticalStep);
-		break;
-	case Qt::Key_Down:
-		m_posStep.setY(m_posStep.y() + iVerticalStep);
-		break;
-	default:
-		return false;
+	// Determine vertical step...
+	if (iKey == Qt::Key_Up || iKey == Qt::Key_Down)  {
+		int iVerticalStep = qtractorTrack::HeightMin;
+		qtractorTrack *pTrack = m_pTracks->currentTrack();
+		if (pTrack)
+			iVerticalStep += (pTrack->zoomHeight() >> 1);
+		int y0 = m_posDrag.y();
+		int y1 = y0 + m_posStep.y();
+		if (iKey == Qt::Key_Up)
+			y1 -= iVerticalStep;
+		else
+			y1 += iVerticalStep;
+		m_posStep.setY((y1 < 0 ? 0 : y1) - y0);
+	}
+	else
+	// Determine horizontal step...
+	if (iKey == Qt::Key_Left || iKey == Qt::Key_Right)  {
+		unsigned short iSnapPerBeat = pSession->snapPerBeat();
+		if (iSnapPerBeat < 1)
+			iSnapPerBeat = 1;
+		int x0 = m_posDrag.x();
+		int x1 = x0 + m_posStep.x();
+		qtractorTimeScale::Cursor cursor(pSession->timeScale());
+		qtractorTimeScale::Node *pNode = cursor.seekPixel(x1);
+		int iHorizontalStep = pNode->pixelsPerBeat() / iSnapPerBeat;
+		if (iKey == Qt::Key_Left)
+			x1 -= iHorizontalStep;
+		else
+			x1 += iHorizontalStep;
+		m_posStep.setX(pSession->pixelSnap(x1 < 0 ? 0 : x1) - x0);
 	}
 
 	// Early sanity check...
@@ -2522,6 +2574,13 @@ bool qtractorTrackView::isClipSelected (void) const
 }
 
 
+// Whether there's a single track selection.
+qtractorTrack *qtractorTrackView::singleTrackSelected (void)
+{
+	return m_pClipSelect->singleTrack();
+}
+
+
 // Whether there's any clip on clipboard. (static)
 bool qtractorTrackView::isClipboard (void)
 {
@@ -2821,7 +2880,8 @@ void qtractorTrackView::moveClipSelect ( qtractorTrack *pTrack )
 		= new qtractorClipCommand(tr("move clip"));
 
 	// We can only move clips between tracks of the same type...
-	int iTrackClip = 0;
+	int  iTrackClip = 0;
+	long iClipDelta = 0;
 	bool bAddTrack = (pTrack == NULL);
 	qtractorTrack *pSingleTrack = m_pClipSelect->singleTrack();
 	if (pSingleTrack) {
@@ -2852,7 +2912,6 @@ void qtractorTrackView::moveClipSelect ( qtractorTrack *pTrack )
 		qtractorClip *pClip = pClipItem->clip;
 		if (pSingleTrack == NULL)
 			pTrack = pClip->track();
-		int x = (pClipItem->rectClip.x() + m_iDraggingX);
 		// Clip parameters.
 		unsigned long iClipStart    = pClip->clipStart();
 		unsigned long iClipOffset   = pClip->clipOffset();
@@ -2885,9 +2944,23 @@ void qtractorTrackView::moveClipSelect ( qtractorTrack *pTrack )
 			pClipCommand->addClip(pClipRight, pClipRight->track());
 			// Done, right clip.
 		}
+		// Convert to precise frame positioning,
+		// but only the first clip gets snapped...
+		iClipStart = iSelectStart;
+		if (iTrackClip == 0) {
+			int x = (pClipItem->rectClip.x() + m_iDraggingX);
+			unsigned long iFrameStart = pSession->frameSnap(
+				pSession->frameFromPixel(x > 0 ? x : 0));
+			iClipDelta = long(iFrameStart) - long(iClipStart);
+			iClipStart = iFrameStart;
+		} else if (long(iClipStart) + iClipDelta > 0) {
+			iClipStart += iClipDelta;
+		} else {
+			iClipStart = 0;
+		}
 		// -- Moved clip...
 		pClipCommand->moveClip(pClip, pTrack,
-			pSession->frameSnap(pSession->frameFromPixel(x > 0 ? x : 0)),
+			iClipStart,
 			iClipOffset + iSelectOffset,
 			iSelectLength);
 		// If track's new it will need a name...
@@ -2957,16 +3030,20 @@ void qtractorTrackView::pasteClipSelect ( qtractorTrack *pTrack )
 			qtractorClip *pClip = pClipItem->clip;
 			if (pSingleTrack == NULL)
 				pTrack = pClip->track();
-			int x = (pClipItem->rect.x() + m_iDraggingX);
 			// Convert to precise frame positioning,
 			// but only the first clip gets snapped...
-			unsigned long iClipStart = pSession->frameFromPixel(x > 0 ? x : 0);
+			unsigned long iClipStart = pClipItem->clipStart;
 			if (iTrackClip == 0) {
-				unsigned long iFrameStart = pSession->frameSnap(iClipStart);
+				int x = (pClipItem->rect.x() + m_iDraggingX);
+				unsigned long iFrameStart = pSession->frameSnap(
+					pSession->frameFromPixel(x > 0 ? x : 0));
 				iClipDelta = long(iFrameStart) - long(iClipStart);
 				iClipStart = iFrameStart;
-			} else if (long(iClipStart) + iClipDelta > 0)
+			} else if (long(iClipStart) + iClipDelta > 0) {
 				iClipStart += iClipDelta;
+			} else {
+				iClipStart = 0;
+			}
 			// Now, its imperative to make a proper copy of those clips...
 			qtractorClip *pNewClip = NULL;
 			switch (pTrack->trackType()) {
