@@ -1,7 +1,7 @@
 // qtractorTrackView.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -111,9 +111,15 @@ public:
 	// Whether there's any clip currently editable.
 	qtractorClip *currentClip() const;
 
+	// Clip selection accessor.
+	qtractorClipSelect *clipSelect() const;
+
 	// Whether there's any clip currently selected.
 	bool isClipSelected() const;
 
+	// Whether there's a single track selection.
+	qtractorTrack *singleTrackSelected();
+
 	// Whether there's any clip on clipboard.
 	static bool isClipboard();
 
@@ -217,11 +223,16 @@ protected:
 	// Clip selection sanity check method.
 	bool queryClipSelect();
 
+	// Update whole clip selection.
+	void updateClipSelect();
+
 	// Draw/hide the whole current clip selection.
-	void updateClipSelect(int y, int h) const;
 	void showClipSelect() const;
 	void hideClipSelect() const;
 
+	// Update single track clip selection.
+	void updateSingleTrack(int y, int h) const;
+
 	// Draw/hide the whole drop rectagle list
 	void updateDropRects(int y, int h) const;
 	void showDropRects() const;
