@@ -1,7 +1,7 @@
 // qtractorTracks.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,13 +30,19 @@
 
 #include "qtractorTrackCommand.h"
 #include "qtractorClipCommand.h"
+#include "qtractorMidiEditCommand.h"
+
+#include "qtractorTrackButton.h"
 
 #include "qtractorAudioEngine.h"
+#include "qtractorAudioBuffer.h"
 #include "qtractorAudioClip.h"
 
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiClip.h"
 
+#include "qtractorClipSelect.h"
+
 #include "qtractorOptions.h"
 
 #include "qtractorMainForm.h"
@@ -48,7 +54,6 @@
 #include "qtractorMidiEditorForm.h"
 
 #include <QVBoxLayout>
-#include <QToolButton>
 #include <QProgressBar>
 #include <QMessageBox>
 #include <QFileDialog>
@@ -190,30 +195,39 @@ void qtractorTracks::verticalZoomStep ( int iZoomStep )
 }
 
 
-// Zoom view slots.
-void qtractorTracks::zoomIn (void)
+// Zoom view actuators.
+void qtractorTracks::zoomIn ( int iZoomMode )
 {
-	horizontalZoomStep(+ ZoomStep);
-	verticalZoomStep(+ ZoomStep);
+	if (iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(+ ZoomStep);
+	if (iZoomMode & ZoomVertical)
+		verticalZoomStep(+ ZoomStep);
+
 	centerContents();
 }
 
-void qtractorTracks::zoomOut (void)
+void qtractorTracks::zoomOut ( int iZoomMode )
 {
-	horizontalZoomStep(- ZoomStep);
-	verticalZoomStep(- ZoomStep);
+	if (iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(- ZoomStep);
+	if (iZoomMode & ZoomVertical)
+		verticalZoomStep(- ZoomStep);
+
 	centerContents();
 }
 
 
-void qtractorTracks::zoomReset (void)
+void qtractorTracks::zoomReset ( int iZoomMode )
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
-	horizontalZoomStep(ZoomBase - pSession->horizontalZoom());
-	verticalZoomStep(ZoomBase - pSession->verticalZoom());
+	if (iZoomMode & ZoomHorizontal)
+		horizontalZoomStep(ZoomBase - pSession->horizontalZoom());
+	if (iZoomMode & ZoomVertical)
+		verticalZoomStep(ZoomBase - pSession->verticalZoom());
+
 	centerContents();
 }
 
@@ -335,13 +349,6 @@ qtractorTrack *qtractorTracks::currentTrack (void) const
 }
 
 
-// Retrieves current selected track widget reference.
-qtractorTrackItemWidget *qtractorTracks::currentTrackWidget (void) const
-{
-	return m_pTrackList->currentTrackWidget();
-}
-
-
 // Retrieves current selected clip reference.
 qtractorClip *qtractorTracks::currentClip (void) const
 {
@@ -560,6 +567,38 @@ bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
 	if (pSession == NULL)
 		return false;
 
+	// Make it as an undoable command...
+	qtractorClipCommand *pClipCommand
+		= new qtractorClipCommand(tr("clip normalize"));
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	// Multiple clip selection...
+	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
+	if (pClipSelect->items().count() > 0) {
+		QListIterator<qtractorClipSelect::Item *> iter(pClipSelect->items());
+		while (iter.hasNext())
+			normalizeClipCommand(pClipCommand, iter.next()->clip);
+	}	// Single, current clip instead?
+	else normalizeClipCommand(pClipCommand, pClip);
+
+	QApplication::restoreOverrideCursor();
+
+	// Check if valid...
+	if (pClipCommand->isEmpty()) {
+		delete pClipCommand;
+		return false;
+	}
+
+	// That's it...
+	return pSession->execute(pClipCommand);
+
+}
+
+
+bool qtractorTracks::normalizeClipCommand (
+	qtractorClipCommand *pClipCommand, qtractorClip *pClip )
+{
 	if (pClip == NULL)
 		pClip = m_pTrackView->currentClip();
 	if (pClip == NULL)
@@ -592,7 +631,6 @@ bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
 			= static_cast<qtractorAudioBus *> (pTrack->outputBus());
 		if (pAudioBus == NULL)
 			return false;
-		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 		QProgressBar *pProgressBar = NULL;
 		if (pMainForm)
 			pProgressBar = pMainForm->progressBar();
@@ -607,7 +645,6 @@ bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
 			fGain /= data.max;
 		if (pProgressBar)
 			pProgressBar->hide();
-		QApplication::restoreOverrideCursor();
 	}
 	else
 	if (pTrack->trackType() == qtractorTrack::Midi) {
@@ -616,71 +653,66 @@ bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
 			= static_cast<qtractorMidiClip *> (pClip);
 		if (pMidiClip == NULL)
 			return false;
-		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 		unsigned char max = 0;
 		pMidiClip->clipExport(midiClipNormalize, &max, iOffset, iLength);
 		if (max > 0x0c && max < 0x7f)
 			fGain *= (127.0f / float(max));
-		QApplication::restoreOverrideCursor();
 	}
 
 	// Make it as an undoable command...
-	qtractorClipCommand *pClipCommand
-		= new qtractorClipCommand(tr("clip normalize"));
 	pClipCommand->gainClip(pClip, fGain);
 
 	// That's it...
-	return pSession->execute(pClipCommand);
+	return true;
 }
 
 
-// Audio clip export calback.
-struct audioClipExportData
-{	// Ctor.
-	audioClipExportData(qtractorAudioFile *pFile)
-		: count(0), file(pFile) {};
-	// Members.
-	unsigned int count;
-	qtractorAudioFile *file;
-};
-
-static void audioClipExport (
-	float **ppFrames, unsigned int iFrames, void *pvArg )
-{
-	audioClipExportData *pData
-		= static_cast<audioClipExportData *> (pvArg);
-
-	(pData->file)->write(ppFrames, iFrames);
-
-	if (++(pData->count) > 100) {
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm) {
-			QProgressBar *pProgressBar = pMainForm->progressBar();
-			pProgressBar->setValue(pProgressBar->value() + iFrames);
-		}
-		qtractorSession::stabilize();
-		pData->count = 0;
-	}
-}
-
-
-// MIDI clip export calback.
-static void midiClipExport (
-	qtractorMidiSequence *pSeq, void *pvArg )
-{
-	qtractorMidiFile *pMidiFile
-		= static_cast<qtractorMidiFile *> (pvArg);
-	pMidiFile->writeTrack(pSeq);
-}
-
-
-// Export given(current) clip.
-bool qtractorTracks::exportClip ( qtractorClip *pClip )
+// Quantize given(current) MIDI clip.
+bool qtractorTracks::quantizeClip ( qtractorClip *pClip )
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
+	// Make it as an undoable command...
+	qtractorMidiClipCommand *pMidiClipCommand
+		= new qtractorMidiClipCommand(tr("clip quantize"));
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	// Multiple clip selection...
+	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
+	if (pClipSelect->items().count() > 0) {
+		QListIterator<qtractorClipSelect::Item *> iter(pClipSelect->items());
+		while (iter.hasNext())
+			quantizeClipCommand(pMidiClipCommand, iter.next()->clip);
+	}	// Single, current clip instead?
+	else quantizeClipCommand(pMidiClipCommand, pClip);
+
+	QApplication::restoreOverrideCursor();
+
+	// Check if valid...
+	if (pMidiClipCommand->isEmpty()) {
+		delete pMidiClipCommand;
+		return false;
+	}
+
+	// That's it...
+	return pSession->execute(pMidiClipCommand);
+}
+
+
+bool qtractorTracks::quantizeClipCommand (
+	qtractorMidiClipCommand *pMidiClipCommand, qtractorClip *pClip )
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	unsigned short iQuantize = pSession->snapPerBeat();
+	if (iQuantize < 1)
+		return false;
+
 	if (pClip == NULL)
 		pClip = m_pTrackView->currentClip();
 	if (pClip == NULL)
@@ -689,8 +721,12 @@ bool qtractorTracks::exportClip ( qtractorClip *pClip )
 	qtractorTrack *pTrack = pClip->track();
 	if (pTrack == NULL)
 		return false;
+	if (pTrack->trackType() != qtractorTrack::Midi)
+		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	qtractorMidiClip *pMidiClip = static_cast<qtractorMidiClip *> (pClip);
+	if (pMidiClip == NULL)
+		return false;
 
 	unsigned long iOffset = 0;
 	unsigned long iLength = pClip->clipLength();
@@ -700,170 +736,594 @@ bool qtractorTracks::exportClip ( qtractorClip *pClip )
 		iLength = pClip->clipSelectEnd() - pClip->clipSelectStart();
 	}
 
-	if (pTrack->trackType() == qtractorTrack::Audio) {
-		// Export audio clip...
-		qtractorAudioClip *pAudioClip
-			= static_cast<qtractorAudioClip *> (pClip);
-		if (pAudioClip == NULL)
-			return false;
-		qtractorAudioBus *pAudioBus
-			= static_cast<qtractorAudioBus *> (pTrack->outputBus());
-		if (pAudioBus == NULL)
-			return false;
-		const QString& sExt = qtractorAudioFileFactory::defaultExt();
-		const QString& sTitle  = tr("Export Audio Clip") + " - " QTRACTOR_TITLE;
-		const QString& sFilter = tr("Audio files (*.%1)").arg(sExt); 
-	#if QT_VERSION < 0x040400
-		// Ask for the filename to save...
-		QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
-			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
-	#else
-		// Construct save-file dialog...
-		QFileDialog fileDialog(this, sTitle,
-			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
-		// Set proper open-file modes...
-		fileDialog.setAcceptMode(QFileDialog::AcceptSave);
-		fileDialog.setFileMode(QFileDialog::AnyFile);
-		fileDialog.setDefaultSuffix(sExt);
-		// Stuff sidebar...
-		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		if (pOptions) {
-			QList<QUrl> urls(fileDialog.sidebarUrls());
-			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
-			urls.append(QUrl::fromLocalFile(pOptions->sAudioDir));
-			fileDialog.setSidebarUrls(urls);
-		}
-		// Show dialog...
-		if (!fileDialog.exec())
-			return false;
-		QString sFilename = fileDialog.selectedFiles().first();
-	#endif
-		if (sFilename.isEmpty())
-			return false;
-		if (QFileInfo(sFilename).suffix() != sExt)
-			sFilename += '.' + sExt;
-		qtractorAudioFile *pAudioFile
-			= qtractorAudioFileFactory::createAudioFile(sFilename,
-				pAudioBus->channels(), pSession->sampleRate());
-		if (pAudioFile == NULL)
-			return false;
-		if (pMainForm) {
-			pMainForm->appendMessages(
-				tr("Audio clip export: \"%1\" started...")
-				.arg(sFilename));
-		}
-		bool bResult = false;
-		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-		QProgressBar *pProgressBar = NULL;
-		if (pMainForm)
-			pProgressBar = pMainForm->progressBar();
-		if (pProgressBar) {
-			pProgressBar->setRange(0, iLength / 100);
-			pProgressBar->reset();
-			pProgressBar->show();
-		}
-		if (pAudioFile->open(sFilename, qtractorAudioFile::Write)) {
-			audioClipExportData data(pAudioFile);
-			pAudioClip->clipExport(
-				audioClipExport, &data, iOffset, iLength);
-			pAudioFile->close();
-			bResult = true;
-		}
-		delete pAudioFile;
-		if (pProgressBar)
-			pProgressBar->hide();
-		QApplication::restoreOverrideCursor();
-		if (pMainForm) {
-			if (bResult) {
-				pMainForm->addAudioFile(sFilename);
-				pMainForm->appendMessages(
-					tr("Audio clip export: \"%1\" complete.")
-					.arg(sFilename));
-			} else {
-				pMainForm->appendMessagesError(
-					tr("Audio clip export:\n\n\"%1\"\n\nfailed.")
-					.arg(sFilename));
-			}
+	// Make it as an undoable command...
+	qtractorMidiEditCommand *pEditCommand
+		= new qtractorMidiEditCommand(pMidiClip, tr("clip quantize"));
+
+	qtractorMidiSequence *pSeq = pMidiClip->sequence();
+	unsigned long iTimeOffset = pSeq->timeOffset();
+
+	qtractorTimeScale::Cursor cursor(pSession->timeScale());
+	qtractorTimeScale::Node *pNode = cursor.seekFrame(pClip->clipStart());
+	unsigned long t0 = pNode->tickFromFrame(pClip->clipStart());
+
+	unsigned long f1 = pClip->clipStart() + pClip->clipOffset() + iOffset;
+	pNode = cursor.seekFrame(f1);
+	unsigned long t1 = pNode->tickFromFrame(f1);
+	unsigned long iTimeStart = t1 - t0;
+	iTimeStart = (iTimeStart > iTimeOffset ? iTimeStart - iTimeOffset : 0);
+	pNode = cursor.seekFrame(f1 += iLength);
+	unsigned long iTimeEnd = iTimeStart + pNode->tickFromFrame(f1) - t1;
+
+	for (qtractorMidiEvent *pEvent = pSeq->events().first();
+			pEvent; pEvent = pEvent->next()) {
+		unsigned long iTime = pEvent->time();
+		if (iTime >= iTimeStart && iTime < iTimeEnd) {
+			pNode = cursor.seekTick(iTime);
+			unsigned long q = pNode->ticksPerBeat / iQuantize;
+			iTime = q * ((iTime + (q >> 1)) / q);
+			unsigned long iDuration = pEvent->duration();
+			if (pEvent->type() == qtractorMidiEvent::NOTEON)
+				iDuration = q * ((iDuration + q - 1) / q);
+			pEditCommand->resizeEventTime(pEvent, iTime, iDuration);
 		}
 	}
-	else
-	if (pTrack->trackType() == qtractorTrack::Midi) {
-		// Export MIDI clip...
+
+	// That's it...
+	pMidiClipCommand->addEditCommand(pEditCommand);
+	return true;
+}
+
+
+
+// Export selected clips.
+bool qtractorTracks::exportClips (void)
+{
+	return mergeExportClips(NULL);
+}
+
+
+// Merge selected clips.
+bool qtractorTracks::mergeClips (void)
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	// Make it as an undoable command...
+	qtractorClipCommand *pClipCommand
+		= new qtractorClipCommand(tr("clip merge"));
+
+	bool bResult = mergeExportClips(pClipCommand);
+	
+	// Have we failed the command prospect?
+	if (!bResult || pClipCommand->isEmpty()) {
+		delete pClipCommand;
+		return false;
+	}
+
+	return pSession->execute(pClipCommand);
+}
+
+
+// Merge/export selected clips.
+bool qtractorTracks::mergeExportClips ( qtractorClipCommand *pClipCommand )
+{
+	// Multiple clip selection:
+	// - make sure we have at least 2 clips to merge...
+	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
+	if (pClipSelect->items().count() < 1)
+		return false;
+
+	// Should be one single track...
+	qtractorTrack *pTrack = pClipSelect->singleTrack();
+	if (pTrack == NULL)
+		return false;
+
+	// Dispatch to specialized method...
+	bool bResult = false;
+	switch (pTrack->trackType()) {
+	case qtractorTrack::Audio:
+		bResult = mergeExportAudioClips(pClipCommand);
+		break;
+	case qtractorTrack::Midi:
+		bResult = mergeExportMidiClips(pClipCommand);
+		break;
+	case qtractorTrack::None:
+	default:
+		break;
+	}
+
+	// Done most.
+	return bResult;
+}
+
+
+// Audio clip buffer merge/export item.
+struct audioClipBufferItem
+{
+	// Constructor.
+	audioClipBufferItem(qtractorClip *pClip,
+		unsigned short iChannels,
+		unsigned int iSampleRate)
+		: clip(static_cast<qtractorAudioClip *> (pClip))
+	{
+		buff = new qtractorAudioBuffer(iChannels, iSampleRate);
+		buff->setOffset(clip->clipOffset());
+		buff->setLength(clip->clipLength());
+		buff->setTimeStretch(clip->timeStretch());
+		buff->setPitchShift(clip->pitchShift());
+		buff->open(clip->filename());
+	}
+
+	// Destructor.
+	~audioClipBufferItem()
+		{ if (buff) delete buff; }
+
+	// Members.
+	qtractorAudioClip   *clip;
+	qtractorAudioBuffer *buff;
+};
+
+
+// Merge/export selected(audio) clips.
+bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
+{
+	// Should be one single MIDI track...
+	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
+	qtractorTrack *pTrack = pClipSelect->singleTrack();
+	if (pTrack == NULL)
+		return false;
+	if (pTrack->trackType() != qtractorTrack::Audio)
+		return false;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	qtractorAudioBus *pAudioBus
+		= static_cast<qtractorAudioBus *> (pTrack->outputBus());
+	if (pAudioBus == NULL)
+		return false;
+
+	const QString& sExt = qtractorAudioFileFactory::defaultExt();
+	const QString& sTitle  = tr("Merge/Export Audio Clip") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Audio files (*.%1)").arg(sExt); 
+#if QT_VERSION < 0x040400
+	// Ask for the filename to save...
+	QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
+		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+#else
+	// Construct save-file dialog...
+	QFileDialog fileDialog(this, sTitle,
+		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+	fileDialog.setFileMode(QFileDialog::AnyFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<QUrl> urls(fileDialog.sidebarUrls());
+		urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+		urls.append(QUrl::fromLocalFile(pOptions->sAudioDir));
+		fileDialog.setSidebarUrls(urls);
+	}
+	// Show dialog...
+	if (!fileDialog.exec())
+		return false;
+	QString sFilename = fileDialog.selectedFiles().first();
+#endif
+	if (sFilename.isEmpty())
+		return false;
+	if (QFileInfo(sFilename).suffix() != sExt)
+		sFilename += '.' + sExt;
+
+	// Should take sometime...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	qtractorAudioFile *pAudioFile
+		= qtractorAudioFileFactory::createAudioFile(sFilename,
+			pAudioBus->channels(), pSession->sampleRate());
+	if (pAudioFile == NULL) {
+		QApplication::restoreOverrideCursor();
+		return false;
+	}
+
+	// Open the file for writing...
+	if (!pAudioFile->open(sFilename, qtractorAudioFile::Write)) {
+		QApplication::restoreOverrideCursor();
+		delete pAudioFile;
+		return false;
+	}
+
+	// Start logging...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		pMainForm->appendMessages(
+			tr("Audio clip merge/export: \"%1\" started...")
+			.arg(sFilename));
+	}
+
+	// Multiple clip selection...
+	QListIterator<qtractorClipSelect::Item *> iter(pClipSelect->items());
+
+	// Multi-selection extents (in frames)...
+	QList<audioClipBufferItem *> items;
+	unsigned long iSelectStart = pSession->sessionLength();
+	unsigned long iSelectEnd = 0;
+	while (iter.hasNext()) {
+		qtractorClip *pClip = iter.next()->clip;
+		if (iSelectStart > pClip->clipSelectStart())
+			iSelectStart = pClip->clipSelectStart();
+		if (iSelectEnd < pClip->clipSelectEnd())
+			iSelectEnd = pClip->clipSelectEnd();
+		items.append(new audioClipBufferItem(
+			pClip, pAudioBus->channels(), pSession->sampleRate()));
+	}
+
+	// A progress indication might be friendly...
+	QProgressBar *pProgressBar = NULL;
+	if (pMainForm)
+		pProgressBar = pMainForm->progressBar();
+	if (pProgressBar) {
+		pProgressBar->setRange(0, (iSelectEnd - iSelectStart) / 100);
+		pProgressBar->reset();
+		pProgressBar->show();
+	}
+
+	// Allocate merge audio scratch buffer...
+	unsigned int iBufferSize = pSession->audioEngine()->bufferSize();
+	unsigned short iChannels = pAudioBus-> channels();
+	unsigned short i;
+	float **ppFrames = new float * [iChannels];
+	for (i = 0; i < iChannels; ++i)
+		ppFrames[i] = new float[iBufferSize];
+
+	// Setup clip buffers...
+	QListIterator<audioClipBufferItem *> it(items);
+	while (it.hasNext()) {
+		audioClipBufferItem *pItem = it.next();
+		qtractorAudioClip   *pClip = pItem->clip;
+		qtractorAudioBuffer *pBuff = pItem->buff;
+		// Almost similar to qtractorAudioClip::process(0)...
+		unsigned long iOffset = 0;
+		unsigned long iClipStart = pClip->clipStart();
+		unsigned long iClipEnd   = iClipStart + pClip->clipLength();
+		if (iSelectStart > iClipStart && iSelectStart < iClipEnd)
+			iOffset = iSelectStart - iClipStart;
+		// Make it initially filled...
+		pBuff->seek(iOffset);
+	//	pBuff->syncExport();
+	}
+
+	// Loop-merge audio clips...
+	unsigned long iFrameStart = iSelectStart;
+	unsigned long iFrameEnd = iFrameStart + iBufferSize;
+	int count = 0;
+
+	// Loop until EOF...
+	while (iFrameStart < iSelectEnd && iFrameEnd > iSelectStart) {
+		// Zero-silence on scratch buffers...
+		for (i = 0; i < iChannels; ++i)
+			::memset(ppFrames[i], 0, iBufferSize * sizeof(float));
+		// Merge clips in window...
+		it.toFront();
+		while (it.hasNext()) {
+			audioClipBufferItem *pItem = it.next();
+			qtractorAudioClip   *pClip = pItem->clip;
+			qtractorAudioBuffer *pBuff = pItem->buff;
+			// Should force sync now and then...
+			if ((count % 33) == 0) pBuff->syncExport();
+			// Quite similar to qtractorAudioClip::process()...
+			unsigned long iClipStart = pClip->clipStart();
+			unsigned long iClipEnd   = iClipStart + pClip->clipLength();
+			if (iFrameStart < iClipStart && iFrameEnd > iClipStart) {
+				unsigned long iOffset = iFrameEnd - iClipStart;
+				if (pBuff->inSync(0, iOffset)) {
+					pBuff->readMix(ppFrames, iOffset,
+						iChannels, iClipStart - iFrameStart,
+						pClip->gain(iOffset));
+				}
+			}
+			else
+			if (iFrameStart >= iClipStart && iFrameStart < iClipEnd) {
+				unsigned long iOffset = iFrameEnd - iClipStart;
+				if (pBuff->inSync(iFrameStart - iClipStart, iOffset)) {
+					pBuff->readMix(ppFrames, iFrameEnd - iFrameStart,
+						iChannels, 0, pClip->gain(iOffset));
+				}
+			}
+		}
+		// Actually write to merge audio file;
+		// - check for last incomplete block...
+		if (iFrameEnd > iSelectEnd)
+			pAudioFile->write(ppFrames, iBufferSize - (iFrameEnd - iSelectEnd));
+		else
+			pAudioFile->write(ppFrames, iBufferSize);
+		// Advance to next buffer...
+		iFrameStart = iFrameEnd;
+		iFrameEnd = iFrameStart + iBufferSize;
+		if (++count > 100 && pProgressBar) {
+			pProgressBar->setValue(pProgressBar->value() + iBufferSize);
+			qtractorSession::stabilize();
+			count = 0;
+		}
+	}
+
+	for (i = 0; i < iChannels; ++i)
+		delete ppFrames[i];
+	delete [] ppFrames;
+
+	qDeleteAll(items);
+	items.clear();
+
+	// Close and free it up...
+	pAudioFile->close();
+	delete pAudioFile;
+
+	if (pProgressBar)
+		pProgressBar->hide();
+
+	// Stop logging...
+	if (pMainForm) {
+		pMainForm->addAudioFile(sFilename);
+		pMainForm->appendMessages(
+			tr("Audio clip merge/export: \"%1\" complete.")
+			.arg(sFilename));
+	}
+
+	// The resulting merge comands, if any...
+	if (pClipCommand) {
+		iter.toFront();
+		while (iter.hasNext()) {
+			qtractorClip *pClip = iter.next()->clip;
+			// Clip parameters.
+			unsigned long iClipStart  = pClip->clipStart();
+			unsigned long iClipOffset = pClip->clipOffset();
+			unsigned long iClipLength = pClip->clipLength();
+			unsigned long iClipEnd    = iClipStart + iClipLength;
+			// Determine and keep clip regions...
+			if (iSelectStart > iClipStart) {
+				// -- Left clip...
+				pClipCommand->resizeClip(pClip,
+					iClipStart,
+					iClipOffset,
+					iSelectStart - iClipStart);
+				// Done, left clip.
+			}
+			else
+			if (iSelectEnd < iClipEnd) {
+				// -- Right clip...
+				pClipCommand->resizeClip(pClip,
+					iSelectEnd,
+					iClipOffset + (iSelectEnd - iClipStart),
+					iClipEnd - iSelectEnd);
+				// Done, right clip.
+			} else {
+				// -- Inner clip...
+				pClipCommand->removeClip(pClip);
+				// Done, inner clip.
+			}
+		}
+		// Set the resulting clip command...
+		qtractorAudioClip *pNewClip = new qtractorAudioClip(pTrack);
+		pNewClip->setClipStart(iSelectStart);
+		pNewClip->setClipLength(iSelectEnd - iSelectStart);
+		pNewClip->setFilename(sFilename);
+		pClipCommand->addClip(pNewClip, pTrack);
+	}
+
+	// Almost done with it...
+	QApplication::restoreOverrideCursor();
+
+	// That's it...
+	return true;
+}
+
+
+// Merge/export selected(MIDI) clips.
+bool qtractorTracks::mergeExportMidiClips ( qtractorClipCommand *pClipCommand )
+{
+	// Should be one single MIDI track...
+	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
+	qtractorTrack *pTrack = pClipSelect->singleTrack();
+	if (pTrack == NULL)
+		return false;
+	if (pTrack->trackType() != qtractorTrack::Midi)
+		return false;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	// Merge MIDI Clip filename requester...
+	const QString  sExt("mid");
+	const QString& sTitle  = tr("Merge/Export MIDI Clip") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("MIDI files (*.%1 *.smf *.midi)").arg(sExt); 
+#if QT_VERSION < 0x040400
+	// Ask for the filename to save...
+	QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
+		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+#else
+	// Construct save-file dialog...
+	QFileDialog fileDialog(this, sTitle,
+		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+	fileDialog.setFileMode(QFileDialog::AnyFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<QUrl> urls(fileDialog.sidebarUrls());
+		urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+		urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
+		fileDialog.setSidebarUrls(urls);
+	}
+	// Show dialog...
+	if (!fileDialog.exec())
+		return false;
+	QString sFilename = fileDialog.selectedFiles().first();
+#endif
+	if (sFilename.isEmpty())
+		return false;
+	if (QFileInfo(sFilename).suffix() != sExt)
+		sFilename += '.' + sExt;
+
+	// Should take sometime...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	// Create SMF...
+	qtractorMidiFile file;
+	if (!file.open(sFilename, qtractorMidiFile::Write)) {
+		QApplication::restoreOverrideCursor();
+		return false;
+	}
+
+	// Write SMF header...
+	unsigned short iTicksPerBeat = pSession->ticksPerBeat();
+	unsigned short iFormat = qtractorMidiClip::defaultFormat();
+	unsigned short iTracks = (iFormat == 0 ? 1 : 2);
+	if (!file.writeHeader(iFormat, iTracks, iTicksPerBeat)) {
+		QApplication::restoreOverrideCursor();
+		return false;
+	}
+
+	// Start logging...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		pMainForm->appendMessages(
+			tr("MIDI clip merge/export: \"%1\" started...")
+			.arg(sFilename));
+	}
+
+	// Multiple clip selection...
+	QListIterator<qtractorClipSelect::Item *> iter(pClipSelect->items());
+
+	// Multi-selection extents (in frames)...
+	unsigned long iSelectStart = pSession->sessionLength();
+	unsigned long iSelectEnd = 0;
+	while (iter.hasNext()) {
+		qtractorClip *pClip = iter.next()->clip;
+		if (iSelectStart > pClip->clipSelectStart())
+			iSelectStart = pClip->clipSelectStart();
+		if (iSelectEnd < pClip->clipSelectEnd())
+			iSelectEnd = pClip->clipSelectEnd();
+	}
+
+	// Multi-selection extents (in ticks)...
+	unsigned long iTimeStart = pSession->tickFromFrame(iSelectStart);
+	unsigned long iTimeEnd   = pSession->tickFromFrame(iSelectEnd);
+
+	// Set proper tempo map...
+	if (file.tempoMap()) {
+		file.tempoMap()->fromTimeScale(
+			pSession->timeScale(),
+			pSession->tickFromFrame(iTimeStart));
+	}
+
+	// Setup track (SMF format 1).
+	if (iFormat == 1)
+		file.writeTrack(NULL);
+
+	// Setup merge sequence...
+	qtractorMidiSequence seq(pTrack->trackName(), 1, iTicksPerBeat);
+	seq.setChannel(pTrack->midiChannel());
+	seq.setBank(pTrack->midiBank());
+	seq.setProgram(pTrack->midiProgram());
+
+	// The merge...
+	iter.toFront();
+	while (iter.hasNext()) {
+		qtractorClip *pClip = iter.next()->clip;
+		// Clip parameters.
+		unsigned long iClipStart  = pClip->clipStart();
+		unsigned long iClipOffset = pClip->clipOffset();
+		unsigned long iClipLength = pClip->clipLength();
+		unsigned long iClipEnd    = iClipStart + iClipLength;
+		// Determine and keep clip regions...
+		if (pClipCommand) {
+			if (iSelectStart > iClipStart) {
+				// -- Left clip...
+				pClipCommand->resizeClip(pClip,
+					iClipStart,
+					iClipOffset,
+					iSelectStart - iClipStart);
+				// Done, left clip.
+			}
+			else
+			if (iSelectEnd < iClipEnd) {
+				// -- Right clip...
+				pClipCommand->resizeClip(pClip,
+					iSelectEnd,
+					iClipOffset + (iSelectEnd - iClipStart),
+					iClipEnd - iSelectEnd);
+				// Done, right clip.
+			} else {
+				// -- Inner clip...
+				pClipCommand->removeClip(pClip);
+				// Done, inner clip.
+			}
+		}
+		// Do the MIDI merge, itself...
 		qtractorMidiClip *pMidiClip
 			= static_cast<qtractorMidiClip *> (pClip);
-		if (pMidiClip == NULL)
-			return false;
-		const QString  sExt("mid");
-		const QString& sTitle  = tr("Export MIDI Clip") + " - " QTRACTOR_TITLE;
-		const QString& sFilter = tr("MIDI files (*.%1 *.smf *.midi)").arg(sExt); 
-	#if QT_VERSION < 0x040400
-		// Ask for the filename to save...
-		QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
-			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
-	#else
-		// Construct save-file dialog...
-		QFileDialog fileDialog(this, sTitle,
-			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
-		// Set proper open-file modes...
-		fileDialog.setAcceptMode(QFileDialog::AcceptSave);
-		fileDialog.setFileMode(QFileDialog::AnyFile);
-		fileDialog.setDefaultSuffix(sExt);
-		// Stuff sidebar...
-		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		if (pOptions) {
-			QList<QUrl> urls(fileDialog.sidebarUrls());
-			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
-			urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
-			fileDialog.setSidebarUrls(urls);
-		}
-		// Show dialog...
-		if (!fileDialog.exec())
-			return false;
-		QString sFilename = fileDialog.selectedFiles().first();
-	#endif
-		if (sFilename.isEmpty())
-			return false;
-		if (QFileInfo(sFilename).suffix() != sExt)
-			sFilename += '.' + sExt;
-		if (pMainForm) {
-			pMainForm->appendMessages(
-				tr("MIDI clip export: \"%1\" started...")
-				.arg(sFilename));
-		}
-		bool bResult = true;
-		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-		qtractorMidiFile *pMidiFile = new qtractorMidiFile();
-		if (pMidiFile->open(sFilename, qtractorMidiFile::Write)) {
-			pMidiFile->setTempo(pSession->tempo());
-			pMidiFile->setBeatsPerBar(pSession->beatsPerBar());
-			unsigned short iFormat = qtractorMidiClip::defaultFormat();
-			unsigned short iTracks = (iFormat == 0 ? 1 : 2);
-			pMidiFile->writeHeader(iFormat, iTracks, pSession->ticksPerBeat());
-			if (iFormat == 1)
-				pMidiFile->writeTrack(NULL);  // Setup track (SMF format 1).
-			pMidiClip->clipExport(
-				midiClipExport, pMidiFile, iOffset, iLength);
-			pMidiFile->close();
-			bResult = true;
-		}
-		delete pMidiFile;
-		QApplication::restoreOverrideCursor();
-		if (pMainForm) {
-			if (bResult) {
-				pMainForm->addMidiFile(sFilename);
-				pMainForm->appendMessages(
-					tr("MIDI clip export: \"%1\" complete.")
-					.arg(sFilename));
-			} else {
-				pMainForm->appendMessagesError(
-					tr("MIDI clip export:\n\n\"%1\"\n\nfailed.")
-					.arg(sFilename));
+		if (pMidiClip) {
+			unsigned long iTimeClip
+				= pSession->tickFromFrame(pClip->clipStart());
+			unsigned long iTimeOffset = iTimeClip - iTimeStart;
+			// For each event...
+			qtractorMidiEvent *pEvent
+				= pMidiClip->sequence()->events().first();
+			while (pEvent && iTimeClip + pEvent->time() < iTimeStart)
+				pEvent = pEvent->next();
+			while (pEvent && iTimeClip + pEvent->time() < iTimeEnd) {
+				qtractorMidiEvent *pNewEvent
+					= new qtractorMidiEvent(*pEvent);
+				pNewEvent->setTime(iTimeOffset + pEvent->time());
+				if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
+					unsigned long iTimeEvent = iTimeClip + pEvent->time();
+					float fGain = pMidiClip->gain(
+						pSession->frameFromTick(iTimeEvent)
+						- pClip->clipStart());
+					pNewEvent->setVelocity((unsigned char)
+						(fGain * float(pEvent->velocity())) & 0x7f);
+					if (iTimeEvent + pEvent->duration() > iTimeEnd)
+						pNewEvent->setDuration(iTimeEnd - iTimeEvent);
+				}
+				seq.insertEvent(pNewEvent);
+				pEvent = pEvent->next();
 			}
 		}
 	}
-	else return false; // WTF?
 
-	// Done.
+	// Write the track and close SMF...
+	file.writeTrack(&seq);
+	file.close();
+
+	// Stop logging...
+	if (pMainForm) {
+		pMainForm->addMidiFile(sFilename);
+		pMainForm->appendMessages(
+			tr("MIDI clip merge/export: \"%1\" complete.")
+			.arg(sFilename));
+	}
+
+	// Set the resulting clip command...
+	if (pClipCommand) {
+		qtractorMidiClip *pNewClip = new qtractorMidiClip(pTrack);
+		pNewClip->setClipStart(iSelectStart);
+		pNewClip->setClipLength(iSelectEnd - iSelectStart);
+		pNewClip->setFilename(sFilename);
+		pNewClip->setTrackChannel(1);
+		pClipCommand->addClip(pNewClip, pTrack);
+	}
+
+	// Almost done with it...
+	QApplication::restoreOverrideCursor();
+
+	// That's it...
 	return true;
 }
 
@@ -875,6 +1335,13 @@ bool qtractorTracks::isClipSelected (void) const
 }
 
 
+// Whether there's a single track selection.
+qtractorTrack *qtractorTracks::singleTrackSelected (void)
+{
+	return m_pTrackView->singleTrackSelected();
+}
+
+
 // Clipboard methods.
 void qtractorTracks::cutClipboard (void)
 {
@@ -1017,7 +1484,8 @@ bool qtractorTracks::removeTrack ( qtractorTrack *pTrack )
 			"\"%1\"\n\n"
 			"Are you sure?")
 			.arg(pTrack->trackName()),
-			tr("OK"), tr("Cancel")) > 0)
+			QMessageBox::Ok | QMessageBox::Cancel)
+			== QMessageBox::Cancel)
 			return false;
 	}
 
@@ -1388,7 +1856,8 @@ void qtractorTracks::trackButtonToggledSlot (
 		return;
 
 	pSession->execute(
-		new qtractorTrackButtonCommand(pTrackButton, bOn));
+		new qtractorTrackStateCommand(
+			pTrackButton->track(), pTrackButton->toolType(), bOn));
 }
 
 
