@@ -1,7 +1,7 @@
 // qtractorTracks.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -34,6 +34,8 @@ class qtractorTrackView;
 class qtractorTrack;
 class qtractorSession;
 class qtractorClip;
+class qtractorClipCommand;
+class qtractorMidiClipCommand;
 
 
 //----------------------------------------------------------------------------
@@ -65,8 +67,6 @@ public:
 	bool removeTrack(qtractorTrack *pTrack = NULL);
 	bool editTrack(qtractorTrack *pTrack = NULL);
 
-	qtractorTrackItemWidget *currentTrackWidget() const;
-
 	// Import Audio/MIDI files into new tracks...
 	bool addAudioTracks(QStringList files, unsigned long iClipStart = 0);
 	bool addMidiTracks(QStringList files, unsigned long iClipStart = 0);
@@ -83,11 +83,16 @@ public:
 	bool editClip(qtractorClip *pClip = NULL);
 	bool splitClip(qtractorClip *pClip = NULL);
 	bool normalizeClip(qtractorClip *pClip = NULL);
-	bool exportClip(qtractorClip *pClip = NULL);
+	bool quantizeClip(qtractorClip *pClip = NULL);
+	bool exportClips();
+	bool mergeClips();
 
 	// Whether there's any clip currently selected.
 	bool isClipSelected() const;
 
+	// Whether there's a single track selection.
+	qtractorTrack *singleTrackSelected();
+
 	// Clipboard methods.
 	void cutClipboard();
 	void copyClipboard();
@@ -111,16 +116,19 @@ public:
 	// Overall contents reset.
 	void clear();
 
+	// Zoom modes.
+	enum { ZoomNone = 0, ZoomHorizontal = 1, ZoomVertical = 2, ZoomAll = 3 };
+
+	// Zoom view actuators.
+	void zoomIn(int iZoomMode = ZoomAll);
+	void zoomOut(int iZoomMode = ZoomAll);
+	void zoomReset(int iZoomMode = ZoomAll);
+
 public slots:
 
 	// Track button notification.
 	void trackButtonToggledSlot(qtractorTrackButton *pTrackButton, bool bOn);
 
-	// Zoom view slots.
-	void zoomIn();
-	void zoomOut();
-	void zoomReset();
-
 protected:
 
 	// Zoom factor constants.
@@ -134,6 +142,19 @@ protected:
 	// (usually after zoom change)
 	void centerContents();
 
+	// Multi-clip command builders.
+	bool normalizeClipCommand(
+		qtractorClipCommand *pMidiClipCommand, qtractorClip *pClip);
+	bool quantizeClipCommand(
+		qtractorMidiClipCommand *pMidiClipCommand, qtractorClip *pClip);
+
+	// Common clip-export/merge methods.
+	bool mergeExportClips(qtractorClipCommand *pClipCommand);
+
+	// Specialized clip-export/merge methods.
+	bool mergeExportAudioClips(qtractorClipCommand *pClipCommand);
+	bool mergeExportMidiClips(qtractorClipCommand *pClipCommand);
+
 protected slots:
 
 	// Zoom view slots.
