@@ -892,6 +892,9 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 	Qt::WindowFlags wflags = Qt::Window
 	#if QT_VERSION >= 0x040200
 		| Qt::CustomizeWindowHint
+	#if QT_VERSION >= 0x040500
+		| Qt::WindowCloseButtonHint
+	#endif
 	#endif
 		| Qt::WindowTitleHint
 		| Qt::WindowSystemMenuHint
